/* CHK=0xD722 */
#define HONEYDANBER /* means use ASCII pids in lock files */
#if defined(WHT)
#define LOG_LOCKS
#endif
/*+-----------------------------------------------------------------------
	ecuLCK.c -- lock file management
	wht%n4hgf@gatech.edu

  Defined functions:
	is_active_lock(name)
	line_locked(ttyname)
	make_lock_name(ttyname,lock_file_name)

Lock files under XENIX are supposed to use the direct line name
(lower-case last letter); we create only the lower-case case, but
check for both.
------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-20-1990-17:11-wht@n4hgf-correct header function list */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:11-24-1989-20:21-wht-considerations for UNIX 3.2 utmp */
/*:11-23-1989-03:11-wht-conditional compile for M_XENIX or M_UNIX */
/*:11-19-1989-15:39-wht-log more stuff if WHT */
/*:11-17-1989-23:08-wht-put checks back in for both lock files */
/*:11-13-1989-16:38-wht-fix chmod problem with new code */
/*:11-08-1989-17:53-wht-revamp old hacked cruddy strategy */
/*:11-08-1989-14:51-wht-because they were not chmod 444 */
/*:10-17-1989-02:58-wht-why is uucp clobbering my lock files ?? */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-30-1989-01:06-wht-ecuungetty interface SEEMS to be working now */
/*:06-29-1989-00:42-wht-add ungetty handler */
/*:06-24-1989-16:52-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "utmpstatus.h"

extern int errno;
extern char ungetty_ttyname[];

/*+-------------------------------------------------------------------------
	make_lock_name(ttyname,lock_file_name)
--------------------------------------------------------------------------*/
make_lock_name(ttyname,lock_file_name)
char *ttyname;
char *lock_file_name;
{

	if((ulindex(ttyname,"/dev/tty")) != 0)
		return(LOPEN_INVALID);

	strcpy(lock_file_name,"/usr/spool/uucp/LCK..");
	strcat(lock_file_name,ttyname + 5);
	return(0);

}	/* end of make_lock_name */

/*+-------------------------------------------------------------------------
	is_active_lock(name) - check to see if lock still active

if not unlink any old lock name
--------------------------------------------------------------------------*/
is_active_lock(name)
register char *name;
{
register itmp;
int lockpid;
int fd;
int status = 0;
char pidstr[12];

	errno = 0;
	if((fd = open(name,O_RDONLY,0)) < 0)
	{
		if(errno != ENOENT)
			status = LOPEN_LCKERR;
		goto RETURN_STATUS;
	}

#if defined(HONEYDANBER)
	itmp = read(fd,(char *)pidstr,11);
	pidstr[11] = 0;
	close(fd);
	if(itmp != 11)
		goto UNLINK_OLD_LOCK;
	lockpid = atoi(pidstr);
#else
	itmp = read(fd,(char *)&lockpid,sizeof(int));
	close(fd);
	if(itmp != sizeof(int))
		goto UNLINK_OLD_LOCK;
#endif

	if((!(itmp = kill(lockpid,0))) || (errno != ESRCH))
	{
		errno = EACCES;		/* for hangup() */
		status = lockpid;
		goto RETURN_STATUS;
	}

UNLINK_OLD_LOCK:
	if(unlink(name))
		status = LOPEN_LCKERR;

RETURN_STATUS:

#if defined(LOG_LOCKS)
{ char s128[128];
  extern char *errno_text();
	sprintf(s128,"ISLOCK %s status=%d errno=%s",
		name,status,errno_text(errno));
	ecu_log_event(s128);
}
#endif

	return(status);
}	/* end of is_active_lock */

/*+-----------------------------------------------------------------------
	line_locked(ttyname) - boolean test for locked line

  ttyname must be of style "/dev/ttyxx"
  Returns locking pid if locked else LOPEN lock error code (< 0) else 0
  Either modem or direct name might be locked, so check both
------------------------------------------------------------------------*/
int
line_locked(ttyname)
char *ttyname;
{
register itmp;
char lock_file_name[64];
	
	make_lock_name(ttyname,lock_file_name);
	if(itmp = is_active_lock(lock_file_name))
		return(itmp);

#if defined(M_XENIX) || defined(M_UNIX)
	if(isalpha(lock_file_name[itmp = strlen(lock_file_name) - 1]))
	{
		lock_file_name[itmp] ^= 0x20;	/* lower->upper, upper->lower */
		if(itmp = is_active_lock(lock_file_name))
			return(itmp);
	}
#endif
	return(0);

}	/* end of line_locked */

/* end of ecuLCK.c */
/* vi: set tabstop=4 shiftwidth=4: */
