/* CHK=0x36CE */
/*+-----------------------------------------------------------------------
	ecudump.c  -- very generic hex/graphics dump development aid
	wht%n4hgf@gatech.edu

  Defined functions:
	dump_putc(ch)
	dump_puts(str)
	hex_dump(str,len,title,terse_flag)
	hex_dump_fp(fp,str,len,title,terse_flag)
	hex_dump16(int16)
	hex_dump32(int32)
	hex_dump4(int4)
	hex_dump8(int8)

------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:03-28-1990-04:50-wht@n4hgf-include hex dump output in log if logging */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:07-27-1989-15:49-wht-fix too many hyphens on odd-length titles */
/*:07-11-1989-16:52-wht-no carriage returns output if dumpfp != stderr */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:52-wht-flush edits --- ecu 1.95 */

#include "ecu.h"

static FILE *dumpfp;

/*+-------------------------------------------------------------------------
	dump_putc(ch)
--------------------------------------------------------------------------*/
void
dump_putc(ch)
char ch;
{
	if(dumpfp == se)
		pputc(ch);
	else
		fputc(ch,dumpfp);
}	/* end of dump_putc */


/*+-------------------------------------------------------------------------
	dump_puts(str)
--------------------------------------------------------------------------*/
void
dump_puts(str)
char *str;
{
	if(dumpfp == se)
		pputs(str);
	else
		fputs(str,dumpfp);
}	/* end of dump_puts */


/*+-----------------------------------------------------------------------
	hex_dump#... subservient routines
------------------------------------------------------------------------*/
void hex_dump4(int4)
uchar int4;
{
	int4 &= 15;
	dump_putc((int4 >= 10) ? (int4 + 'A' - 10) : (int4 + '0'));
}

void hex_dump8(int8)
uchar int8;
{
	hex_dump4(int8 >> 4);
	hex_dump4(int8);
}

void hex_dump16(int16)
ushort int16;
{
	hex_dump8(int16 >> 8);
	hex_dump8(int16);
}

void hex_dump32(int32)
ulong int32;
{
	hex_dump16(int32 >> 16);
	hex_dump16(int32);
}


/*+-----------------------------------------------------------------
	hex_dump_fp(fp,str,len,title,terse_flag)

  if 'title' not NULL, title is printed... 'terse_flag'
  controls whether or not the title is "conspicuous" with
  hyphens before and after it making title line >70 chars long
------------------------------------------------------------------*/
void
hex_dump_fp(fp,str,len,title,terse_flag)
FILE *fp;
char *str;
int len;
char *title;
int terse_flag;
{
int istr;
register ipos;
register itmp;

	dumpfp = fp;

	if(title && (istr = strlen(title)))
	{
		if(!terse_flag)
		{
			ipos = (73 - istr) / 2;
			itmp = ipos;
			while(itmp--)
				dump_putc('-');
			dump_putc(' ');
			if(istr & 1)
				ipos--;
		}
		dump_puts(title);
		if(!terse_flag)
		{
			dump_putc(' ');
			while(ipos--)
				dump_putc('-');
		}
		if(dumpfp == se)
			dump_puts("\r\n");
		else
			dump_puts("\n");

	}

	istr = 0;
	while(istr < len)
	{
		hex_dump16(istr);
		dump_putc(' ');
		for(itmp = 0; itmp < 16; ++itmp)
		{
			ipos = istr + itmp;
			if(ipos >= len)
			{
				if(!terse_flag)
					dump_puts("   ");
				continue;
			}
			dump_putc(' ');
			hex_dump8(str[ipos]);
		}
		dump_puts(" | ");
		for(itmp = 0; itmp < 16; ++itmp)
		{
			ipos = istr + itmp;
			if( (ipos) >= len)
			{
				if(!terse_flag)
					dump_putc(' ');
			}
			else
			{
				dump_putc((str[ipos] >= ' ' && str[ipos] < 0x7f)
		 			? str[ipos] : '.' );
			}
		}
		if(dumpfp == se)
			dump_puts(" |\r\n");
		else
			dump_puts(" |\n");
		istr += 16;
	}   /* end of while(istr < len) */

}	/* end of hex_dump_fp */

/*+-------------------------------------------------------------------------
	hex_dump(str,len,title,terse_flag)
--------------------------------------------------------------------------*/
void
hex_dump(str,len,title,terse_flag)
char *str;
int len;
char *title;
int terse_flag;
{
	hex_dump_fp(se,str,len,title,terse_flag);
}	/* end of hex_dump_fp */
/* end of ecudump.c */
/* vi: set tabstop=4 shiftwidth=4: */
