/* CHK=0xB11C */
/*+-----------------------------------------------------------------
	ecufkey.c -- XENIX function key definition
	wht%n4hgf@gatech.edu

  Defined functions:
	keyset_display()
	keyset_idnum(keystr)
	keyset_idstr(keyid)
	keyset_init()
	keyset_read(keyset_name)
	xf_to_keyid(xf)

------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:03-27-1990-15:18-wht@n4hgf-overhaul for/wait/SIGCLD handling */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-23-1990-14:51-wht@n4hgf-use strcmpi instead of ulcmpb */
/*:03-23-1990-14:34-wht@n4hgf-BkTab redisplay rcvr screen */
/*:11-23-1989-03:11-wht-conditional compile for M_XENIX or M_UNIX */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:52-wht-flush edits --- ecu 1.95 */

#if defined(M_XENIX)	/* the entire module is bound by this cond. compile */
#include "ecu.h"
#include "ecukey.h"
#include "ecufkey.h"
#include "ecuxkey.h"
#include "ecufork.h"

extern char kbdintr;	/* current input INTR */
extern char curr_dir[];	/* current working key defns */

KDE		keyset_table[KDE_COUNT];
char keyset_name[32] = "";

XF_KDE_NAME xf_kde_name[] =
{
	{ XFcurup,	KDE_CUU,	"CUU" },
	{ XFcurdn,	KDE_CUD,	"CUD" },
	{ XFcurrt,	KDE_CUR,	"CUR" },
	{ XFcurlf,	KDE_CUL,	"CUL" },
	{ XFcur5,	KDE_CU5,	"CU5" },
	{ XFend,	KDE_END,	"End" },
	{ XFpgdn,	KDE_PGDN,	"PgDn" },
	{ XFhome,	KDE_HOME,	"Home" },
	{ XFpgup,	KDE_PGUP,	"PgUp" },
	{ XFins,	KDE_INS,	"Ins" },
	{ XF1,		KDE_F1,		"F1" },
	{ XF2,		KDE_F2,		"F2" },
	{ XF3,		KDE_F3,		"F3" },
	{ XF4,		KDE_F4,		"F4" },
	{ XF5,		KDE_F5,		"F5" },
	{ XF6,		KDE_F6,		"F6" },
	{ XF7,		KDE_F7,		"F7" },
	{ XF8,		KDE_F8,		"F8" },
	{ XF9,		KDE_F9,		"F9" },
	{ XF10,		KDE_F10,	"F10" },
	{ XF11,		KDE_F11,	"F11" },
	{ XF12,		KDE_F12,	"F12" },
	{ XFbktab,	KDE_BKTAB,	"BkTab" },
	{ 0,0,"" }
};

/*+-------------------------------------------------------------------------
	keyset_init()
--------------------------------------------------------------------------*/
void
keyset_init()
{
register itmp;
register KDE *tkde;
char *make_char_graphic();

	for(itmp = 0; itmp < KDE_COUNT; itmp++)
	{
		tkde = &keyset_table[itmp];
		tkde->logical[0] = 0;
		tkde->outcount = 0;
		tkde->keyid = (uchar)itmp;
	}

	keyset_name[0] = 0;

	tkde = &keyset_table[KDE_BKTAB];
	strcpy(tkde->logical,"redisplay");
	tkde->outcount = KACT_REDISPLAY;

	tkde = &keyset_table[KDE_HOME];
	strcpy(tkde->logical,"ecu cmd");
	tkde->outcount = KACT_COMMAND;

	tkde = &keyset_table[KDE_INS];
	strcpy(tkde->logical,"local shell");
	tkde->outcount = KACT_LOCAL_SHELL;

	tkde = &keyset_table[KDE_CU5];
	strcpy(tkde->logical,"screen dump");
	tkde->outstr[0] = 0x7F;	/* this key is intercepted by kbd read routine */
	tkde->outcount = 0;

}	/* end of keyset_init */

/*+-------------------------------------------------------------------------
	keyset_idnum(keystr)
--------------------------------------------------------------------------*/
keyset_idnum(keystr)
char *keystr;
{
struct XF_KDE_NAME *xkn = xf_kde_name;
	while(xkn->xf != 0)
	{
		if(!strcmpi(xkn->name,keystr))
			return((int)xkn->kde);
		xkn++;
	}
	return(-1);
}	/* end of keyset_idnum */

/*+-------------------------------------------------------------------------
	keyset_idstr(keyid)
--------------------------------------------------------------------------*/
char *
keyset_idstr(keyid)
int keyid;
{
struct XF_KDE_NAME *xkn = xf_kde_name;
	while(xkn->xf != 0)
	{
		if((int)xkn->kde == keyid)
			return(xkn->name);
		xkn++;
	}
	return((char *)0);
}	/* end of keyset_idstr */

/*+-------------------------------------------------------------------------
	xf_to_keyid(xf)
--------------------------------------------------------------------------*/
int
xf_to_keyid(xf)
uchar xf;
{
struct XF_KDE_NAME *xkn = xf_kde_name;
	while(xkn->xf != 0)
	{
		if(xkn->xf == xf)
			return((int)xkn->kde & 0xFF);
		xkn++;
	}
	return(-1);
}	/* end of xf_to_keyid */

/*+-------------------------------------------------------------------------
	keyset_define_key(bufptr)
--------------------------------------------------------------------------*/
int
keyset_define_key(bufptr)
char *bufptr;
{
register itmp;
register token_number;
register KDE *tkde;
int keyid;
char token_separator[2];
char *token;
char *str_token();

	if(itmp = strlen(bufptr))	/* itmp = len; if > 0 ... */
		bufptr[--itmp] = 0;		/* ... strip trailing NL */
	if(!itmp)
		return(0);

	if(bufptr[0] == '#')		/* comment? */
		return(0);
	if((*bufptr != 0x20) && (*bufptr != TAB))	/* if no leading space */
		return(1);
	if(*bufptr == 0)					/* if line all blank, break */
		return(1);

	while((*bufptr == 0x20) || (*bufptr == TAB))	/* strip lding sp/tab */
		bufptr++;

	token_number = 0;
	strcpy(token_separator,":");
	while( (token = str_token(bufptr,token_separator)) != NULL)
	{
		bufptr = NULL;	/* further calls to str_token need NULL */
		switch(token_number)
		{
			case 0:		/* first field is key identifier */
				if((keyid = keyset_idnum(token)) < 0)
				{
					ff(se,"  %s is not a legal key identifier\r\n",token);
					return(0);
				}
				if(keyid == KDE_HOME)
				{
					ff(se,"  HOME cannot be redefined!\r\n");
					return(0);
				}
				if(keyid == KDE_CU5)
				{
					ff(se,"  CUR5 cannot be redefined!\r\n");
					return(0);
				}
				if(keyid == KDE_BKTAB)
				{
					ff(se,"  BkTab cannot be redefined!\r\n");
					return(0);
				}
				tkde = &keyset_table[keyid];
				tkde->logical[0] = 0;
				tkde->outcount = 0;
				break;

			case 1:		/* second field is logical key name */
				strncpy(tkde->logical,token,sizeof(tkde->logical));
				tkde->logical[sizeof(tkde->logical) - 1] = 0;
				strcpy(token_separator," "); /* space is tok sep now */
				break;

			default:	/* third and subsequent to define key */
				if(tkde->outcount == sizeof(tkde->outstr))
				{
					ff(se,"  %s: output count too long",
						keyset_idstr(keyid));
					return(0);
				}
				if((itmp = ascii_to_hex(token)) < 0)
				{
					ff(se,"  %s: '%s' invalid\r\n",
						keyset_idstr(keyid),token);
					return(0);
				}
				tkde->outstr[tkde->outcount] = itmp;
				tkde->outcount++;
				break;
		}	/* end of switch(token_number) */
		token_number++;
	}		/* end while not end of record */

	return(0);
}	/* end of keyset_define_key */

/*+-------------------------------------------------------------------------
	keyset_read(name)
returns 0 on success, -1 if no .ecu/keys, -2 if no 'name'
--------------------------------------------------------------------------*/
int
keyset_read(name)
char *name;
{
register itmp;
register char *cptr;
static char ecukeys_name[128];
char readkde_buf[128];
FILE *fp_keys;

	if(!ecukeys_name[0])
	{
		get_home_dir(ecukeys_name);
		strcat(ecukeys_name,"/.ecu/keys");
	}

	if((fp_keys = fopen(ecukeys_name,"r")) == NULL)
		return(-1);

/* find keyset name */
	while(fgets(readkde_buf,sizeof(readkde_buf),fp_keys) != NULL)
	{
		if(readkde_buf[0] == '#')		/* comment? */
			continue;
		if(itmp = strlen(readkde_buf))	/* itmp = len; if > 0 ... */
			readkde_buf[--itmp] = 0;	/* ... strip trailing NL */
		if(!itmp)
			continue;
		if(strcmp(readkde_buf,name) == 0)
		{
			itmp = 1;		/* indicate success */
			break;
		}
		itmp = 0;			/* if loop terminates w/o find, failure */
	}
	if(!itmp)
	{
		fclose(fp_keys);
		return(-2);
	}

	keyset_init();	/* clear any previous key defns */

/* read past any other keyset names matching this set */
	while(fgets(readkde_buf,sizeof(readkde_buf),fp_keys) != NULL)
	{
		cptr = readkde_buf;				/* first call to str_token, -> buff */
		if((*cptr == 0x20) || (*cptr == TAB))	/* if leading space */
		{
			(void)keyset_define_key(cptr);
			break;
		}
	}

/* we found the definition ... read it */
	while(fgets(readkde_buf,sizeof(readkde_buf),fp_keys) != NULL)
	{
		if(keyset_define_key(readkde_buf))
			break;
	}

	strncpy(keyset_name,name,sizeof(keyset_name));
	keyset_name[sizeof(keyset_name) - 1] = 0;
	fclose(fp_keys);
	return(0);
}	/* end of keyset_read */

/*+-------------------------------------------------------------------------
	ffso(str)
--------------------------------------------------------------------------*/
void
ffso(str)
char *str;
{
	stand_out();
	fputs(str,se);
	stand_end();
}	/* end of ffso */

/*+-------------------------------------------------------------------------
	keyset_display()

 F1  xxxxx  F2   xxxxx   HOME xxxxx  PGUP xxxxx
 F3  xxxxx  F4   xxxxx   END  xxxxx  PGDN xxxxx
 F5  xxxxx  F6   xxxxx   INS  xxxxx  CUR5 xxxxx
 F7  xxxxx  F8   xxxxx   BkTab xxxxx
 F9  xxxxx  F10  xxxxx   CUR^ xxxxx  CUR> xxxxx
 F11 xxxxx  F12  xxxxx   CUR< xxxxx  CURv xxxxx
--------------------------------------------------------------------------*/
void
keyset_display()
{
register itmp;
register itmp2;
int clen1 = 0;
char cfmt1[32];
int clen2 = 0;
char cfmt2[32];
int clen3 = 0;
char cfmt3[32];
char cfmt4[4];

	if(!keyset_name[0])
	{
		keyset_init();
		ff(se,"   no key definition active\r\n\r\n");
		ff(se,"HOME - command prefix\r\n");
		ff(se,"BkTab  - redisplay receiver screen\r\n");
		ff(se,"cursor down - local shell in %s\r\n",curr_dir);
		fputs("\r\n",se);
	}
	else
	{
		for(itmp = 0; itmp < KDE_COUNT; itmp++)
		{
		register KDE *tkde = &keyset_table[itmp];
			itmp2 = strlen(tkde->logical);
			switch(itmp)
			{
				case KDE_F1: case KDE_F3: case KDE_F5: case KDE_F7:
				case KDE_F9: case KDE_F11:
					if(clen1 < itmp2)
						clen1 = itmp2;
					break;

				case KDE_F2: case KDE_F4: case KDE_F6: case KDE_F8:
				case KDE_F10: case KDE_F12:
					if(clen2 < itmp2)
						clen2 = itmp2;
					break;

				case KDE_HOME: case KDE_END: case KDE_INS:
				case KDE_CUU: case KDE_CUL:
					if(clen3 < itmp2)
						clen3 = itmp2;
					break;
			}
		}
		sprintf(cfmt1," %%-%d.%ds",clen1,clen1);
		sprintf(cfmt2," %%-%d.%ds",clen2,clen2);
		sprintf(cfmt3," %%-%d.%ds",clen3,clen3);
		strcpy(cfmt4," %s");
		ff(se,"   key definition: %s\r\n\r\n",keyset_name);

		ffso(" F1  ");ff(se,cfmt1,keyset_table[KDE_F1].logical);
		fputs("  ",se);
		ffso(" F2  ");ff(se,cfmt2,keyset_table[KDE_F2].logical);
		fputs("  ",se);
		ffso(" Home ");ff(se,cfmt3,keyset_table[KDE_HOME].logical);
		fputs("  ",se);
		ffso(" PgUp ");ff(se,cfmt4,keyset_table[KDE_PGUP].logical);
		fputs("\r\n",se);

		ffso(" F3  ");ff(se,cfmt1,keyset_table[KDE_F3].logical);
		fputs("  ",se);
		ffso(" F4  ");ff(se,cfmt2,keyset_table[KDE_F4].logical);
		fputs("  ",se);
		ffso(" End  ");ff(se,cfmt3,keyset_table[KDE_END].logical);
		fputs("  ",se);
		ffso(" PgDn ");ff(se,cfmt4,keyset_table[KDE_PGDN].logical);
		fputs("\r\n",se);

		ffso(" F5  ");ff(se,cfmt1,keyset_table[KDE_F5].logical);
		fputs("  ",se);
		ffso(" F6  ");ff(se,cfmt2,keyset_table[KDE_F6].logical);
		fputs("  ",se);
		ffso(" Ins  ");ff(se,cfmt3,keyset_table[KDE_INS].logical);
		fputs("  ",se);
		ffso(" CUR5 ");ff(se,cfmt4,keyset_table[KDE_CU5].logical);
		fputs("\r\n",se);

		ffso(" F7  ");ff(se,cfmt1,keyset_table[KDE_F7].logical);
		fputs("  ",se);
		ffso(" F8  ");ff(se,cfmt2,keyset_table[KDE_F8].logical);
		fputs("  ",se);
		ffso(" BkTab");ff(se,cfmt3,keyset_table[KDE_BKTAB].logical);
		fputs("\r\n",se);

		ffso(" F9  ");ff(se,cfmt1,keyset_table[KDE_F9].logical);
		fputs("  ",se);
		ffso(" F10 ");ff(se,cfmt2,keyset_table[KDE_F10].logical);
		fputs("  ",se);
		ffso(" CUR^ ");ff(se,cfmt3,keyset_table[KDE_CUU].logical);
		fputs("  ",se);
		ffso(" CUR> ");ff(se,cfmt4,keyset_table[KDE_CUR].logical);
		fputs("\r\n",se);

		ffso(" F11 ");ff(se,cfmt1,keyset_table[KDE_F11].logical);
		fputs("  ",se);
		ffso(" F12 ");ff(se,cfmt2,keyset_table[KDE_F12].logical);
		fputs("  ",se);
		ffso(" CUR< ");ff(se,cfmt3,keyset_table[KDE_CUL].logical);
		fputs("  ",se);
		ffso(" CURv ");ff(se,cfmt4,keyset_table[KDE_CUD].logical);
		fputs("\r\n\r\n",se);
	}

}	/* end of keyset_display */
#endif

/* end of ecufkey.c */
/* vi: set tabstop=4 shiftwidth=4: */
