/* CHK=0xAF99 */
/* #define USE_S7 */
/*+-------------------------------------------------------------------------
	ecuhayes.c
	wht%n4hgf@gatech.edu

  Defined functions:
	hayes_ATH()
	hayes_autoanswer()
	hayes_dial()
	hayes_dial_pde(tpde)
	hayes_get_result(msec_to_wait)
	hayes_get_sreg_value(regnum)
	hayes_modem_init()
	hayes_modem_on_hook()
	hayes_read_modem_init()
	hayes_redial(arg,argc)
	hayes_report_iv_set(varnum)
	hayes_send_cmd(cmd)
	hayes_set_sreg(regnum,value)
	process_modem_init(str)
	show_modem_init_error(erc,iesd)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-26-1990-17:20-wht@n4hgf-xbell alarm on connect if enabled */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-02-1990-17:19-wht@n4hgf-hayes_modem_on_hook clobbered shm->Llogical */
/*:03-27-1990-15:18-wht@n4hgf-overhaul for/wait/SIGCLD handling */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-23-1990-14:51-wht@n4hgf-use strcmpi instead of ulcmpb */
/*:03-12-1990-04:28-wht@n4hgf-add Lrname usage */
/*:02-25-1990-20:41-wht@n4hgf-normal redial exits, too, on SIGINT */
/*:01-07-1990-16:28-wht-stop redial on fatal proc error (i.e., SIGINT) */
/*:11-14-1989-12:32-wht-add hack for credit card number trailer */
/*:10-14-1989-21:10-wht-allow user to change ecu color choices */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:52-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecukey.h"
#include "esd.h"
#include "var.h"
#include "ecupde.h"
#include "ecuerror.h"
#include "relop.h"

long time();
void hayes_ATH();

extern int rcvr_pid;
extern int interrupt;
extern int proc_interrupt;
extern int proctrace;
extern ulong colors_current;

int mi_line;
char mi_name[64];

#define MI_MAX_LEN 65
char Lmodem_init[MI_MAX_LEN] = "";		/* modem init string w/o trailing CR */
char Lmodem_dial[MI_MAX_LEN] = "";		/* modem dialing prefix */
char Lmodem_autoans[MI_MAX_LEN] = "";	/* modem autoanswer */

char *interrupted_string = "!Interrupted";

/*+-------------------------------------------------------------------------
	show_modem_init_error(erc,iesd)
--------------------------------------------------------------------------*/
void
show_modem_init_error(erc,iesd)
int erc;
ESD *iesd;
{
register itmp;

	pputs(mi_name);
	pprintf(" line %d: ",mi_line);
	proc_error(erc);
	pputs(iesd->pb);
	pputc(NL);
	itmp = iesd->old_index;
	while(itmp--)
		pputc(' ');
	pputs("^\n\n");

}	/* end of show_modem_init_error */

/*+-----------------------------------------------------------------------
	process_modem_init(str)

sample /usr/lib/ecu/tty??.mi lines:
init_9600:ATS11=47X4S0=0S7=30\Q0\X0\N0
init_>2400:ATS11=47X4S0=0S7=30\Q0\X0\N0
init_<=2400:ATS11=47X4S0=0S7=30\Q1\X1\N3
ATDT

return 0 if entire list read, else 1 if error (error msg in errmsg)
------------------------------------------------------------------------*/
void
process_modem_init(str)
char *str;
{
#define MI_INIT		1
#define MI_DIAL		2
#define MI_AUTOANS	3
int erc;
ESD sesd;
char typestr[32];
int relop;
int truth = 0;
int type;
long test_baud;

	sesd.pb = str;
	sesd.cb = strlen(str);
	sesd.maxcb = strlen(str);
	sesd.index = 0;
	sesd.old_index = 0;

	if(get_alpha_zstr(&sesd,typestr,sizeof(typestr)))
	{
		erc = eSyntaxError;
		goto SHOW_ERROR;
		return;
	}
	if(ulindex(typestr,"init_") == 0)
		type = MI_INIT;
	else if(ulindex(typestr,"dial_") == 0)
		type = MI_DIAL;
	else if(!strcmpi(typestr,"autoanswer"))
		type = MI_AUTOANS;
	else
	{
		erc = eSyntaxError;
		goto SHOW_ERROR;
	}

/* test for default ... if none, check baud rate */
	if(type == MI_AUTOANS)
		truth = 1;
	else if(ulindex(typestr,"_default") > 0)
	{
		truth = !( ((type == MI_INIT) && Lmodem_init[0]) ||
			((type == MI_DIAL) && Lmodem_dial[0]));
	}
	else
	{
		/* get optional operator */
		if(get_relop(&sesd,&relop))
			relop = OP_EQ;
		if(erc = gint_constant(&sesd,&test_baud))
			goto SHOW_ERROR;
		truth = test_truth_int((long)shm->Lbaud,relop,test_baud);
	}

/* if no match, skip this one */
	if(!truth)
		return;

/* skip over colon */
	if(erc = skip_colon(&sesd))
		goto SHOW_ERROR;

/* make sure init or dial string not empty or too long */
	if((erc = skip_cmd_break(&sesd)) && (type != MI_AUTOANS))
		goto SHOW_ERROR;

	if((sesd.cb - sesd.index) > (MI_MAX_LEN - 1))
	{
		erc = eBufferTooSmall;
		goto SHOW_ERROR;
	}

	erc = eDuplicateMatch;		/* in case of show error in switch */
	switch(type)
	{
		case MI_INIT:
			if(Lmodem_init[0])
				goto SHOW_ERROR;
			strcpy(Lmodem_init,sesd.pb + sesd.index);
			break;

		case MI_DIAL:
			if(Lmodem_dial[0])
				goto SHOW_ERROR;
			strcpy(Lmodem_dial,sesd.pb + sesd.index);
			break;

		case MI_AUTOANS:
			if(Lmodem_autoans[0])
				goto SHOW_ERROR;
			if(!sesd.cb)
				strcpy(Lmodem_autoans,"!null!");
			else
				strcpy(Lmodem_autoans,sesd.pb + sesd.index);
			break;
			
	}
	return;		/* <<<<====== done */

SHOW_ERROR:
	show_modem_init_error(erc,&sesd);

}	/* end of process_modem_init */

/*+-----------------------------------------------------------------------
	hayes_read_modem_init()
0123456789
/dev/ttyxx
------------------------------------------------------------------------*/
void
hayes_read_modem_init()
{
char *cptr;
register FILE *fp_modem;
char *skip_ld_break();
char buffer[128];

/* zap init information */
	Lmodem_init[0] = 0;
	Lmodem_dial[0] = 0;

/* build filename */
	strcpy(mi_name,"/usr/lib/ecu/");
	strcat(mi_name,shm->Lline + 5);
	strcat(mi_name,".mi");
		
/* read modem initialization */
	if((fp_modem = fopen(mi_name,"r")) == NULL)
		pperror(mi_name);
	else
	{
		mi_line = 0;
/*		while((!Lmodem_init[0] || !Lmodem_dial[0] || !Lmodem_autoans[0]) && */
		while((!Lmodem_init[0] || !Lmodem_dial[0]) &&
			fgets(buffer,sizeof(buffer),fp_modem))
		{
			mi_line++;
			buffer[strlen(buffer) - 1] = 0;
			cptr = skip_ld_break(buffer);
			/* skip comments and null lines */
			if(!strlen(cptr) || (*cptr == '#'))
				continue;
			process_modem_init(cptr);
		}
		fclose(fp_modem);
	}

/* default */
	if(!Lmodem_init[0])
	{
		strcpy(Lmodem_init,"ATE1Q0V1");
		pputs("modem init string not found (using default '");
		pputs(Lmodem_init);
		pputs("')\n");
	}
	if(!Lmodem_dial[0])
	{
		strcpy(Lmodem_dial,"ATDT");
		pputs("modem dial string not found (using default '");
		pputs(Lmodem_dial);
		pputs("')\n");
	}

	if(!Lmodem_autoans[0])
	{
		strcpy(Lmodem_autoans,"ATQ1S0=1");
#ifdef INUSE
		pputs("modem autoanswer string not found (using default '");
		pputs(Lmodem_autoans);
		pputs("')\n");
#endif
	}
	else if(!strcmp(Lmodem_autoans,"!null!"))
		Lmodem_autoans[0] = 0;

	if(proctrace > 1)
	{
		pprintf("init:       '%s'\n",Lmodem_init);
		pprintf("dial:       '%s'\n",Lmodem_dial);
		pprintf("autoanswer: '%s'\n",Lmodem_autoans);
	}
}	/* end of hayes_read_modem_init */

/*+-------------------------------------------------------------------------
	hayes_get_result(msec_to_wait)
return pointer to static buf containing result code
--------------------------------------------------------------------------*/
char *
hayes_get_result(msec_to_wait)
long msec_to_wait;
{
static char s32[32];
LRWT	lr;

	interrupt = 0;
	s32[0] = 0;
	lr.to1 = msec_to_wait;
	lr.to2 = 200L;
	lr.raw_flag = 0x80; /* allow interrupts */
	lr.buffer = s32;
	lr.bufsize = sizeof(s32);
	lr.delim = (char *)0;
	lr.echo = 0;
	lgets_timeout(&lr);
	return(lr.buffer);

/* reset any interrupt indication since we look at string */
	interrupt = 0;

}	/* end of hayes_get_result */

/*+-------------------------------------------------------------------------
	hayes_modem_init()
--------------------------------------------------------------------------*/
hayes_modem_init()
{
register itmp;
int retries = 0;
char *cmd;
char *cptr;

	if(shm->Lmodem_already_init)
		return(0);

	interrupt = 0;
	hayes_read_modem_init();

	lputs_paced(0,"\b\b\b\b\b\b\b\b\b\r");
	nap(200L);
	lflush(0);

	while(!shm->Lmodem_already_init)
	{
INIT_LOOP:
		if(retries > 3)
			return(-1);

		if(interrupt)
			return(-1);

		if(retries)
		{
			ltoggle_dtr();
			lputs_paced(0,"AT\r");
			nap(400L);
			lputs_paced(0,"ATQ0V1E1\r");
			nap(400L);
		}

		lflush(0);
		cmd = Lmodem_init;
		itmp = 0;
#ifdef NEUROTIC
		while(*cmd)
		{
			lputc_paced(0,*cmd++);
			if(++itmp < 2)
				nap(40L);
			if((itmp = lgetc_timeout(500L)) < 0)
			{
				if(interrupt)
					return(-1);
				retries++;
				goto INIT_LOOP;
			}
			pputc(itmp);
		}
#else
		lputs(cmd);
		pputs(cmd);
#endif

		pputc(NL);
		lputc_paced(0,CR);

		itmp = 0;
		while(itmp != CR)
		{
			if((itmp = lgetc_timeout(500L)) < 0)
			{
				if(interrupt)
					return(-1);
				pputs("missed reading \\r\n");
				retries++;
				goto INIT_LOOP;
			}
		}

		if(strcmp(cptr = hayes_get_result(1200L),"OK"))
		{
			if(!strcmp(cptr,interrupted_string))
			{
				interrupt = 1;
				return(-1);
			}
			pprintf("unexpected result: '%s'\n",cptr);
			retries++;
			continue;
		}
		shm->Lmodem_already_init = 1;
	}

	return(0);

}	/* end of hayes_modem_init */

/*+-------------------------------------------------------------------------
	hayes_send_cmd(cmd)
--------------------------------------------------------------------------*/
int
hayes_send_cmd(cmd)
register char *cmd;
{
register itmp;

	hayes_modem_init();
	nap(600L);

	lflush(0);
#ifdef NEUROTIC
	while(*cmd)
	{
		lputc_paced(20,*cmd++);
		if(++char_count < 2)
			nap(40L);
		if((itmp = lgetc_timeout(500L)) < 0)
			return(-1);
		pputc(itmp);
	}
#else
	lputs(cmd);
	pputs(cmd);
#endif
	pputc(NL);
	lputc_paced(20,CR);
	itmp = 0;
	while(itmp != CR)
	{
		if((itmp = lgetc_timeout(500L)) < 0)
		{
			pputs("missed reading \\r\n");
			return(-1);
		}
		if(itmp == CR)
			break;
	}
	return(0);

}	/* end of hayes_send_cmd */

/*+-------------------------------------------------------------------------
	hayes_report_iv_set(varnum)
--------------------------------------------------------------------------*/
void
hayes_report_iv_set(varnum)
int varnum;
{

	if(proctrace)
		pprintf("modem handler set $i%02d = %ld\n",varnum,iv[varnum]);
}	/* end of hayes_report_iv_set */

/*+-----------------------------------------------------------------------
	hayes_dial() - dial a remote or connect

  returns 0 on success (CONNECT),
          eConnectFailed if failure
          eCONINT on interrupt

  sets I0 to 0==connect,
             1==failed to connect,
             2==interrupted
             3==modem error
  sets S0 to modem result code or uucp status code string

  This function has gotten quite NASTY and needs rewriting!
------------------------------------------------------------------------*/
int
hayes_dial()
{
char s128[128];
int rcvr_was_alive = (rcvr_pid > 0) || (rcvr_pid == -2);
int itmp;
int erc;
int s7;
char *result = "";
ulong colors_at_entry = colors_current;
#ifdef WHT	/* hack for immediate need ... make a real feature later */
char *cptr;
FILE *fp;
char credit_file[128];
#endif
extern char kbdintr;		/* current input INTR */

	if(rcvr_was_alive && (rcvr_pid != -2))
	{
		kill_rcvr_process(SIGUSR1);
		nap(500L);
	}

	if(shm->Ldescr[0])
	{
		setcolor(colors_success);
		pprintf("%s %s (%u baud) on %s\n",
			(shm->Ltelno[0]) ? "Dialing" : "Connecting to",
			shm->Ldescr,shm->Lbaud,shm->Lline);
	}

	setcolor(colors_alert);

	if(!shm->Ltelno[0])	/* if no phone number, direct connect */
	{
		sprintf(s128,"CONNECT %u",shm->Lbaud);
		result = s128;
		iv[0] = 0;
		erc = 0;
		goto START_RCVR_PROCESS;
	}
 	else
	{
		itmp = hdb_dial(&result);
		lreset_ksr();	/* dialer may have changed parity */
		switch(itmp)
		{
			case 0:		/* success */
				goto CONNECTED;
			case 1:		/* failure -- iv[0] set by hdb_dial */
				hayes_report_iv_set(0);
				erc = eConnectFailed;
				goto START_RCVR_PROCESS;
			case 2:		/* interrupted -- iv[0] set by hdb_dial */
				hayes_report_iv_set(0);
				erc = eCONINT;
				goto START_RCVR_PROCESS;
			case 3:		/* modem error */
				setcolor(colors_error);
				pprintf("%s\n",result);
				goto CANNOT_TALK_TO_MODEM;
			case 4:		/* try local */
				break;
		}

		hayes_modem_init();

#if defined(USE_S7)
		if((s7 = hayes_get_sreg_value(7)) < 0)
			s7 = 30;
#else
		s7 = 30;
#endif

		/* build and send dial command */
		strcpy(s128,Lmodem_dial);
		strcat(s128,shm->Ltelno);
#ifdef WHT	/* hack for immediate need ... make a real feature later */
		if(*(cptr = s128 + strlen(s128) - 1) == '#')
		{
			*cptr = 0;
			get_home_dir(credit_file);
			strcat(credit_file,"/.ecu/.credit");
			if(fp = fopen(credit_file,"r"))
			{
				*cptr++ = ',';
				*cptr++ = ',';
				*cptr++ = ',';
				*cptr++ = ',';
				*cptr = 0;
				fgets(cptr,30,fp);
				fclose(fp);
			}
			if(!fp || !(*cptr))
			{
				result = "credit card error";
				goto CONNECT_FAILED;
			}
			*(cptr + strlen(cptr) - 1) = 0; /* kill NL */
		}
#endif /* WHT */
		if(hayes_send_cmd(s128))
			goto SEND_CMD_ERROR;

		/* some modems (ahem, the Hayes 2400) do not accurately honor S7
		so our timer is twice sreg 7 */
 		pprintf("Type %s to abort ... ",make_char_graphic(kbdintr,1));
		lflush(0);
		result = hayes_get_result(s7 * 2 * 1000L);

		if(!strcmp(result,interrupted_string))
		{
			setcolor(colors_error);
			pprintf("%s\n",result);
			lputc(0);				/* make modem go on hook */
			nap(40L);
			lputc(0);
			interrupt = 0;
			hayes_get_result(2000L);	/* wait for NO CARRIER */
			erc = eCONINT;
			iv[0] = 2;
			hayes_report_iv_set(0);
			goto START_RCVR_PROCESS;
		}
		if(!strncmp(result,"CONNECT",7))
		{
			if(strlen(result) > 7)
			{
				if(shm->Lbaud != (unsigned)atoi(result + 7))
				{
					setcolor(colors_error);
					pprintf("%s (incorrect rate)\n",result);
					iv[0] = 2;
					hayes_report_iv_set(0);
					goto START_RCVR_PROCESS;
				}
			}
CONNECTED:
			setcolor(colors_success);
			pprintf("%s\n",result);
			sprintf(s128,"CONNECT %s (%s) %u baud",
				shm->Llogical,shm->Ltelno,shm->Lbaud);
			strcpy(shm->Lrname,shm->Llogical);
			ecu_log_event(s128);
			if(isalpha(shm->Llogical[0]))
				if(keyset_read(shm->Llogical) != 0)
					keyset_init();
			shmx_connect();
			shm->xmit_chars_this_connect = 0L;
			shm->Loff_hook_time = time((long *)0);
			iv[0] = 0;
			hayes_report_iv_set(0);
			erc = 0;
			shm->Lmodem_off_hook = 1;
			goto START_RCVR_PROCESS;
		}
		setcolor(colors_error);
CONNECT_FAILED:
		pprintf("%s\n",result);
		iv[0] = 1;
		hayes_report_iv_set(0);
		erc = eConnectFailed;
		goto START_RCVR_PROCESS;
	}

	shm->Lmodem_off_hook = 1;

START_RCVR_PROCESS:	;
	setcolor(colors_at_entry);
	strcpy(sv[0]->pb,result);
	sv[0]->cb = strlen(result);
	if(rcvr_was_alive)
		start_rcvr_process(1);
	return(erc);

SEND_CMD_ERROR: ;
	if(interrupt || proc_interrupt)
	{
		result = interrupted_string;
		iv[0] = 2;
		hayes_report_iv_set(0);
		erc = eCONINT;
	}
	else
	{
CANNOT_TALK_TO_MODEM:
		setcolor(colors_error);
		pprintf("Cannot talk to modem\n");
		result = "!Modem Error";
		iv[0] = 3;
		hayes_report_iv_set(0);
		erc = eConnectFailed;
	}
	goto START_RCVR_PROCESS;

}	/* end of hayes_dial */

/*+-------------------------------------------------------------------------
	hayes_dial_pde(tpde) - curses access dial command
--------------------------------------------------------------------------*/
int
hayes_dial_pde(tpde)
register PDE *tpde;
{
	copy_pde_to_lvar(tpde);
	return(hayes_dial());
}	/* end of hayes_dial_pde */

/*+-------------------------------------------------------------------------
	hayes_redial(arg,argc)
--------------------------------------------------------------------------*/
hayes_redial(arg,argc)
char **arg;
int argc;
{
int erc = 0;
int delay = 60;
int retries = 10;
int throw_away;

	if(shm->Ltelno[0] == 0)
	{
		pprintf("   no previous number\n");
		return(-1);
	}

	if((argc > 1) && ((retries = atoi(arg[1])) == 0))
	{
		pprintf("  invalid retry count\n");
		return(-1);
	}

	if((argc > 2) && ((delay = atoi(arg[2])) == 0))
	{
		pprintf("  invalid delay\n");
		return(-1);
	}

	if(delay < 15)		/* try to be nice to telcos */
		delay = 15;		/* (they are our friends :-) */

	pprintf("  for %d retries, pause between: %d secs\n",
				retries,delay);

	kill_rcvr_process(SIGUSR1);		/* kill rcvr process */

	hayes_ATH();
	while(retries--)
	{
	register itmp;

#ifdef AUTO_DIAL_PROC
		if(!isdigit(shm->Llogical[0]) && find_procedure(shm->Llogical))
		{
		char *pargv[2];
		ulong colors_at_entry = colors_current;
			pargv[0] = shm->Llogical;
			pargv[1] = "!REDIAL;";
			ttymode(2);
			erc = do_proc(2,pargv);
			proc_file_reset();
			ttymode(1);
			setcolor(colors_notify);
			ff(se,"[procedure finished]");
			setcolor(colors_at_entry);
			ff(se,"\r\n");
			if(!erc)
			{
				start_rcvr_process(0);
				return(0);
			}
			if(erc >= e_FATAL)
				break;
		}
		else if(!(erc = hayes_dial()))
		{
#ifdef WHT
			bell_notify(XBELL_C);
#endif
			start_rcvr_process(1);
			return(0);
		}
#else
		if(!(erc = hayes_dial()))
		{
#ifdef WHT
			bell_notify(XBELL_C);
#endif
			start_rcvr_process(1);
			return(0);
		}
#endif

		if((retries == 0) || (erc >= e_FATAL))
			break;

		pprintf("%d %s left ... ",retries,(retries == 1) ? "retry" : "retries");
		pprintf("waiting %d seconds ... press any key to abort\n",delay);
		for(itmp = 0; itmp < delay; itmp++)
		{
			sleep(1);
			if(rdchk(TTYIN))
			{
				throw_away = ttygetc(1);
				retries = 0;
				break;
			}
		}
	}

	start_rcvr_process(1);
	return(-1);

}	/* end of hayes_redial */

/*+-------------------------------------------------------------------------
	hayes_modem_on_hook()
--------------------------------------------------------------------------*/
void
hayes_modem_on_hook()
{
char s128[128];
long connect_secs;
long time();
char *get_elapsed_time();

	if(shm->Lmodem_off_hook)
	{
		connect_secs = time((long *)0) - shm->Loff_hook_time;
		sprintf(s128,"DISCONNECT %s (%s) %ld %s",
			shm->Llogical,shm->Ltelno,connect_secs,
			get_elapsed_time(connect_secs));
		ecu_log_event(s128);
		shm->Lmodem_off_hook = 0;
	}
	shm->Lrname[0] = 0;

}	/* end of hayes_modem_on_hook */

/*+-------------------------------------------------------------------------
	hayes_ATH()
--------------------------------------------------------------------------*/
void
hayes_ATH()
{
int rcvr_was_alive = (rcvr_pid > 0);

	if(rcvr_was_alive)
	{
		kill_rcvr_process(SIGUSR1);
		nap(500L);
	}

	ltoggle_dtr();
	hayes_modem_on_hook();
	set_default_escape_prompt();
	if(rcvr_was_alive)
		start_rcvr_process(0);

}	/* end of hayes_ATH */

/*+-------------------------------------------------------------------------
	hayes_get_sreg_value(regnum)
 assumes rcvr process has been killed
--------------------------------------------------------------------------*/
int
hayes_get_sreg_value(regnum)
int regnum;
{
char s128[128];
LRWT	lr;

	sprintf(s128,"ATS%d?",regnum);
	hayes_send_cmd(s128);
	lflush(0);
	lr.to1 = 2000L;
	lr.to2 = 140L;
	lr.raw_flag = 0;
	lr.buffer = s128;
	lr.bufsize = sizeof(s128);
	lr.delim = (char *)0;
	lr.echo = 0;
	lgets_timeout(&lr);
	if(lr.count != 3)
		return(-1);
	return(atoi(s128));

}	/* end of hayes_get_sreg_value */

/*+-------------------------------------------------------------------------
	hayes_set_sreg(regnum,value)
 assumes rcvr process has been killed
 returns 0 if no error (reads back value set),
 else -1 and error message has been printed
--------------------------------------------------------------------------*/
int
hayes_set_sreg(regnum,value)
int regnum;
int value;
{
char s128[128];
int value2;
LRWT	lr;

	sprintf(s128,"ATS%d=%d",regnum,value);
	hayes_send_cmd(s128);
	lflush(0);
	lr.to1 = 2000L;
	lr.to2 = 140L;
	lr.raw_flag = 0;
	lr.buffer = s128;
	lr.bufsize = sizeof(s128);
	lr.delim = (char *)0;
	lr.echo = 0;
	lgets_timeout(&lr);
	value2 = hayes_get_sreg_value(regnum);
	if(value2 < 0)
		pprintf("PROBLEM setting modem S%d=%d; cannot talk to modem\n",
				regnum,value);
	else if(value != value2)
		pprintf("PROBLEM setting modem S%d=%d; got %d back\n",
				regnum,value,value2);
	return((value != value2) ? -1 : 0);

}	/* end of hayes_set_sreg */

/*+-------------------------------------------------------------------------
	hayes_autoanswer()
--------------------------------------------------------------------------*/
void
hayes_autoanswer()
{
	if(!Lmodem_autoans[0])
		return;
	nap(200L);
	lputs_paced(20,"AT\r");
	nap(100L);
	lputs_paced(20,Lmodem_autoans); /* quiet modem */
	lputs_paced(20,"\r");
	nap(200L);
	lputs_paced(20,Lmodem_autoans); /* quiet modem */
	lputs_paced(20,"\r");
	nap(200L);
	lflush(0);
}	/* end of hayes_autoanswer */

/* end of ecuhayes.c */
/* vi: set tabstop=4 shiftwidth=4: */
