/* CHK=0x3B8B */
/*+-------------------------------------------------------------------------
	ecuicmhist.c - ecu interactive command history handler
	wht%n4hgf@gatech.edu

  Defined functions:
	icmd_history_add(icmd_buf)
	icmd_history_manager(func,newicmd,icmdsize)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-17-1990-14:26-wht-creation */

#include <curses.h>
#include "pc_scr.h"

#define STDIO_H_INCLUDED
#define OMIT_TERMIO_REFERENCES
#include "ecu.h"

#include "ecukey.h"
#include "ecuxkey.h"

char *strdup();

#ifdef WHT
#define ICMDH_MAXCNT	100
#else
#define ICMDH_MAXCNT	50
#endif
#define ICMDH_MAXLEN	72

typedef struct icmd_hist
{
	struct icmd_hist *prev;
	struct icmd_hist *next;
	uchar *icmd;
} ICMDH;

ICMDH *icmdh_head = (ICMDH *)0;
ICMDH *icmdh_tail = (ICMDH *)0;
int icmdh_count = 0;

/*+-------------------------------------------------------------------------
	icmd_history_add(icmd_buf)
--------------------------------------------------------------------------*/
void
icmd_history_add(icmd_buf)
char *icmd_buf;
{
ICMDH *icmdh = (ICMDH *)malloc(sizeof(ICMDH));
char *strdup();

	if(!icmdh)
		return;
	if(!(icmdh->icmd = strdup(icmd_buf)))
	{
		free((char *)icmdh);
		return;
	}
	if(strlen(icmdh->icmd) > ICMDH_MAXLEN)
		icmdh->icmd[ICMDH_MAXLEN] = 0;
	if(icmdh_tail)
	{
		icmdh_tail->next = icmdh;
		icmdh->prev = icmdh_tail;
		icmdh->next = (ICMDH *)0;
		icmdh_tail = icmdh;
	}
	else
	{
		icmdh->prev = (ICMDH *)0;
		icmdh->next = (ICMDH *)0;
		icmdh_head = icmdh;
		icmdh_tail = icmdh;
	}
	if(++icmdh_count > ICMDH_MAXCNT)
	{
		icmdh = icmdh_head;
		icmdh_head = icmdh->next;
		icmdh_head->prev = (ICMDH *)0;
		free(icmdh->icmd);
		free((char *)icmdh);
		icmdh_count--;
	}
		
}	/* end of icmd_history_add */

/*+-------------------------------------------------------------------------
	icmd_history_manager(func,newicmd,icmdsize) - entered by Home Xkey

return new icmd string to execute
returns 0 if ok to exce new cmd, else 1 if not
(returns 0 if null or ESC, so caller can handle exit condition)
--------------------------------------------------------------------------*/
int
icmd_history_manager(func,newicmd,icmdsize)
uchar func;
uchar *newicmd;
int icmdsize;
{
register itmp;
register ICMDH *icmdh = icmdh_tail;

	if((func != XFcurup) && (func != XFhome))
	{
		ring_bell();
		return(1);
	}

	if(!icmdh)
	{
		ff(se,"no interactive commands saved\r\n");
		return(1);
	}
	while(1)
	{
		strncpy(newicmd,icmdh->icmd,icmdsize - 1);
		*(newicmd + icmdsize - 1) = 0;

		ttygets(newicmd,icmdsize,4+2);
		switch(*newicmd)
		{
		case ESC:
		case 0:
			return(0);
		case XFhome:
			icmdh = icmdh_head;
			*newicmd = 0;
			break;
		case XFend:
			icmdh = icmdh_tail;
			*newicmd = 0;
			break;
		case XFpgup:
		case XFpgdn:
			ring_bell();
			*newicmd = 0;
			break;
		case XFcurup:
			if(icmdh->prev)
				icmdh = icmdh->prev;
			*newicmd = 0;
			break;
		case XFcurdn:
			if(icmdh->next)
				icmdh = icmdh->next;
			*newicmd = 0;
			break;
		default:
			return(0);
		}

		itmp = strlen(newicmd);
		while(itmp--)
			fputc(BS,se);
		itmp = strlen(newicmd);
		while(itmp--)
			fputc(' ',se);
		itmp = strlen(newicmd);
		while(itmp--)
			fputc(BS,se);
	}
}	/* end of icmd_history_manager */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of ecuicmhist.c */
