/* CHK=0xCAB7 */
#define HONEYDANBER /* means use ASCII pids in lock files */
#if defined(WHT)
#define LOG_LOCKS
#endif
/*+-----------------------------------------------------------------------
	eculock.c -- lock file management
	wht%n4hgf@gatech.edu

  Defined functions:
	check_utmp()
	create_lock_file(name)
	lock_tty()
	unlock_tty()

Lock files under XENIX are supposed to use the direct line name
(lower-case last letter); we create only the lower-case case, but
check for both.
------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*+:EDITS:*/
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:03-26-1990-00:46-wht@n4hgf-if EACCES, suggest course of action to user */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-25-1990-14:05-wht@n4hgf-remove lock file BEFORE returning line to getty */
/*:11-24-1989-20:21-wht-considerations for UNIX 3.2 utmp */
/*:11-23-1989-03:11-wht-conditional compile for M_XENIX or M_UNIX */
/*:11-19-1989-15:39-wht-log more stuff if WHT */
/*:11-17-1989-23:08-wht-put checks back in for both lock files */
/*:11-13-1989-16:38-wht-fix chmod problem with new code */
/*:11-08-1989-17:53-wht-revamp old hacked cruddy strategy */
/*:11-08-1989-14:51-wht-because they were not chmod 444 */
/*:10-17-1989-02:58-wht-why is uucp clobbering my lock files ?? */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-30-1989-01:06-wht-ecuungetty interface SEEMS to be working now */
/*:06-29-1989-00:42-wht-add ungetty handler */
/*:06-24-1989-16:52-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "utmpstatus.h"

extern int errno;
extern char ungetty_ttyname[];
extern char lopen_err_str[];

/*+-------------------------------------------------------------------------
	check_utmp()
return 0 if line available, else LOPEN code
--------------------------------------------------------------------------*/
int
check_utmp()
{
register utstatus;
register status = 0;

	switch(utstatus = utmp_status(shm->Lline))
	{
		case US_DIALOUT:	/* enabled for login, currently dialout */
			status = LOPEN_DIALOUT_IN_USE;
			break;
		case US_LOGGEDIN:	/* enabled for login, in use */
			status = LOPEN_ENABLED_IN_USE;
			break;
		case US_NOTFOUND:	/* not in utmp, or getty dead */
			break;
		case US_LOGIN:		/* enabled for login, idle */
			status = ungetty_get_line();
			break;
	}

#if defined(LOG_LOCKS)
{ char s64[64];
	sprintf(s64,"UTMPCHK %s st=%d ut=%d",shm->Lline,status,utstatus);
	ecu_log_event(s64);
}
#endif

	return(status);

}	/* end of check_utmp */

/*+-----------------------------------------------------------------------
	void unlock_tty()
------------------------------------------------------------------------*/
void
unlock_tty()
{
	if(LLCKname[0] == 0)
	{
		ungetty_return_line();
		return;
	}

#ifdef M_UNIX
	ungetty_return_line();
	unlink(LLCKname);
	LLCKname[0] = 0;
#else
	unlink(LLCKname);
	LLCKname[0] = 0;
	ungetty_return_line();
#endif

}	/* end of unlock_tty */

/*+-------------------------------------------------------------------------
	create_lock_file(name)
--------------------------------------------------------------------------*/
int
create_lock_file(name)
char *name;
{
register fd;
int pid = getpid();
char LTMP_fname[64];
#if defined(HONEYDANBER)
char pid10str[12];

	sprintf(pid10str,"%10d\n",getpid());
#endif

	errno = 0;
	sprintf(LTMP_fname,"/usr/spool/uucp/LTMP.%05d",pid);
	if((fd = creat(LTMP_fname,0444)) < 0)
	{
		if(errno == EACCES)
			sprintf(lopen_err_str,
#if defined(M_UNIX)
			"lock error - try chmod 043777 /usr/spool/uucp"
#else
			"lock error - try chmod 0777 /usr/spool/uucp"
#endif
			);
		unlink(LTMP_fname);
		return(-1);
	}
#if defined(HONEYDANBER)
	write(fd,pid10str,11);
#else
	write(fd,(char *)&pid,sizeof(int));
#endif

	chmod(LTMP_fname,0444);
	close(fd);

	fd = link(LTMP_fname,name);		/* use 'fd' for link return code */
	unlink(LTMP_fname);
	chmod(name,0444);

#if defined(LOG_LOCKS)
{ char s128[128];
  extern char *errno_text();
	sprintf(s128,"CRLOCK %s status=%d errno=%s",name,fd,errno_text(errno));
	ecu_log_event(s128);
}
#endif

	return(fd);
}	/* end of create_lock_file */

/*+-------------------------------------------------------------------------
	lock_tty() - create lock files for tty name in 'shm->Lline'
--------------------------------------------------------------------------*/
lock_tty()
{
register itmp;

	if(itmp = make_lock_name(shm->Lline,LLCKname))
		return(itmp);

	if(itmp = check_utmp())
		return(itmp);

#if defined(GETTY_LOCKS_TTY)
	if(!ungetty_ttyname[0])	/* if getty did not lock line */
	{
#endif
		if(create_lock_file(LLCKname))
		{
			if(itmp = is_active_lock(LLCKname))
			{
				ungetty_return_line();
				errno = EACCES; /* for hangup() */
				return(itmp);
			}
			if(create_lock_file(LLCKname))
			{
				ungetty_return_line();
				errno = EACCES; /* for hangup() */
				return(LOPEN_LCKERR);
			}
		}
#if defined(GETTY_LOCKS_TTY)
	}
#endif

	return(0);
}	/* end of lock_tty */

/* end of eculock.c */
/* vi: set tabstop=4 shiftwidth=4: */
