/* CHK=0x4180 */
/*+-------------------------------------------------------------------------
	ecuphone.c -- visual phone dialer/directory editor
	wht%n4hgf@gatech.edu

  .---[ title ]------------modified-.<-- dirw "top line"
  |   stand out mode                |<-- dirw "header line"
  |                                 |<-- scrw first line
  |                                 |
  |                                 |
  |                                 |
  |                                 |<-- scrw last line
  |   stand out mode                |<-- dirw "cmd line"
  `---------------------------------'<-- dirw bottom line

  Defined functions:
	check_curr_pde()
	choose_line(baud)
	copy_pde_to_lvar(tpde)
	dirw_bot_msg(msg)
	dirw_cmd_line_setup(prompt)
	dirw_display_top()
	dirw_get_cmd(prompt)
	lookup_logical_telno()
	pde_add_or_edit(tpde,edit)
	pde_add_or_edit_read(prompt,edit,x,buf,max,delim)
	pde_cmd_add(tpde)
	pde_cmd_down()
	pde_cmd_dump_list()
	pde_cmd_find()
	pde_cmd_remove()
	pde_cmd_remove_oops()
	pde_cmd_save()
	pde_cmd_up()
	pde_display(line,tpde,stand_out)
	pde_display_logical(line,tpde,stand_out)
	pde_list_add(tpde)
	pde_list_erase()
	pde_list_manager()
	pde_list_read()
	pde_list_remove(tpde)
	pde_list_save_if_dirty()
	pde_list_search(logical,exact_flag)
	pde_list_set_dirty(flag)
	scrw_fill(tpde,curr_pde_line)
	scrw_fill_at(line_num,tpde,curr_pde_line)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-20-1990-17:58-wht@n4hgf-choose_line now only uses ACU lines */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-16-1990-01:16-wht@n4hgf-modify for true field editing */
/*:04-15-1990-19:34-wht@n4hgf-add cursor up and down to motion choices */
/*:04-14-1990-14:47-wht@n4hgf-refresh dialing directory on no-dial exit */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-23-1990-14:51-wht@n4hgf-use strcmpi instead of ulcmpb */
/*:03-12-1990-04:29-wht@n4hgf-add Lrname */
/*:02-15-1990-15:48-wht-better diagnostic for LOPEN_LCKERR */
/*:11-24-1989-14:41-wht-UNIX tcap-curses blinks with high bits set on addch */
/*:11-23-1989-03:11-wht-conditional compile for M_XENIX or M_UNIX */
/*:10-23-1989-18:31-wht-was not calling enddvent if getdvbaud succeeded */
/*:10-15-1989-16:52-wht-fix bug on adding to empty directory */
/*:08-31-1989-12:37-wht-was not always setting brate/parity properly */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include <curses.h>
#include "pc_scr.h"

#define STDIO_H_INCLUDED
#define OMIT_TERMIO_REFERENCES
#include "ecu.h"

#include "ecupde.h"
#include "ecukey.h"
#include "ecuxkey.h"
#include "ecuhangup.h"
#include "utmpstatus.h"
#include "dvent.h"

void dirw_bot_msg();

#if defined(pyr)
#define strchr index
#define strrchr rindex
#endif

char *strchr();
char *strrchr();

/* window definitions */
#define DIRW_LINES		(LINES - 1)
#define DIRW_COLS		(COLS)
#define DIRW_COLS		(COLS)
#define DIRW_TOP_LINES	2
#define DIRW_BOT_LINES	2
#define DIRW_CMD_LINE	(DIRW_LINES - 2)
#define SCRW_LINES		(DIRW_LINES - DIRW_TOP_LINES - DIRW_BOT_LINES)
#define SCRW_COLS		(DIRW_COLS)
#define SCRW_TLY		(DIRW_TOP_LINES)
#define SCRW_TLX		0

extern int windows_active;
extern int rcvr_pid;
extern char errmsg[];
extern int tcap_LINES;

WINDOW *dirw;
WINDOW *scrw;

PDE		*pde_list_head = (PDE *)0;	/* pointer to first pde in linked list */
PDE		*curr_pde = (PDE *)0;		/* current pde */
PDE		*remove_pde = (PDE *)0;		/* if non-zero, pde just removed */
int remove_dirty_flag;			/* pde_list_dirty at remove time */
int pde_list_quan = 0;			/* count of items in list now */
int pde_list_dirty = 0;			/* pde_list modified but not saved */
int scrw_curr_pde_line;			/* scrw line curr_pde is on */
char ecuphone_name[256];			/* phone directory name */

/*+-------------------------------------------------------------------------
	pde_list_erase()
--------------------------------------------------------------------------*/
void
pde_list_erase()
{
register PDE *pde = pde_list_head;
register PDE *next;

	while(pde)
	{
		next = pde->next;
		free((char *)pde);
		pde = next;
	}
	pde_list_head = (PDE *)0;
	pde_list_quan = 0;
}	/* end of pde_list_erase */

/*+-------------------------------------------------------------------------
	pde_list_set_dirty(flag)
0: clean, 1 dirty, -1 do not modify;update screen only
--------------------------------------------------------------------------*/
void
pde_list_set_dirty(flag)
int flag;
{
register itmp;
	if(flag != pde_list_dirty)
	{
		if(flag != -1)
			pde_list_dirty = flag;
		wmove(dirw,0,DIRW_COLS - 14);
		if(pde_list_dirty)
			waddstr(dirw," modified ");
		else
		{
			itmp = 10;
			while(itmp--)
				waddch(dirw,(unsigned)sHR);
		}
		wrefresh(dirw);
	}
}	/* end of pde_list_set_dirty */

/*+-------------------------------------------------------------------------
	pde_list_add(tpde) -- add to linked list
--------------------------------------------------------------------------*/
void
pde_list_add(tpde)
PDE *tpde;
{
register PDE *this = tpde;
register PDE *prev;
register PDE *next;

/* if empty, init list with this one and quit */
	if(pde_list_head == (PDE *)0)
	{
		pde_list_head = this;
		this->prev = (PDE *)0;
		this->next = (PDE *)0;
		pde_list_quan++;
		return;
	}

/* list not empty */
	prev = (PDE *)0;		/* no previous yet */
	next = pde_list_head;	/* init next to top of list */

	while(strcmp(next->logical,this->logical) < 0)
	{
		prev = next;
		next = prev->next;
		if(next == (PDE *)0)
			break;
	}

	if(prev)		/* if non-zero, we will not update the list head */
	{
		this->next = prev->next;
		this->prev = prev;
		prev->next = this;
		if(next)
			next->prev = this;
	}
	else	/* 'this' is to become the new list head (1st element) */
	{
		this->next = next;
		this->prev = (PDE *)0;
		if(next)
			next->prev = this;
		pde_list_head = this;
	}
	pde_list_quan++;

}	/* end of pde_add */

/*+-------------------------------------------------------------------------
	pde_list_remove(tpde) -- add to linked list
--------------------------------------------------------------------------*/
void
pde_list_remove(tpde)
register PDE *tpde;
{
register PDE *prev;
register PDE *next;

	prev = (PDE *)0;	/* there is no previous now */

	if((next = pde_list_head) == (PDE *)0)	/* if empty list */
		return;

	while(next != tpde)
	{
		prev = next;
		next = prev->next;
		if(next == (PDE *)0)
			return;
	}

/* take care of "current pde" */
	if(tpde == curr_pde)
	{
		if(tpde->next)
			curr_pde = tpde->next;
		else if(tpde->prev)
			curr_pde = tpde->prev;
		else
			curr_pde = (PDE *)0;
	}

/* unlink */

	if(prev)		/* if non-zero, we will not update the list head */
	{
		prev->next = tpde->next;
		if(tpde->next)
			(tpde->next)->prev = prev;
	}
	else
	{
		pde_list_head = tpde->next;
		if(tpde->next)
			(tpde->next)->prev = (PDE *)0;
	}

	tpde->next = (PDE *)0;
	tpde->prev = (PDE *)0;

	pde_list_quan--;
}	/* end of pde_list_remove */

/*+-----------------------------------------------------------------------
	PDE *pde_list_search(logical,exact_flag)
------------------------------------------------------------------------*/
PDE *
pde_list_search(logical,exact_flag)
char *logical;
int exact_flag;
{
register PDE *tpde;

	if(!pde_list_quan)
	{
		if(pde_list_read())
			return((PDE *)0);
	}

	tpde = pde_list_head;
	while(tpde)
	{
		/* only first few chars necessary for match with ulcmpb */
		if(exact_flag)
		{
			if(strcmp(tpde->logical,logical) == 0)
				return(tpde);
		}
		else
		{
			if(ulcmpb(tpde->logical,logical) < 0)
				return(tpde);
		}
		tpde = tpde->next;
	}
	if(!tpde)
		sprintf(errmsg,"'%s' not found",logical);
	return(tpde);

}	/* end of pde_list_search */

/*+-------------------------------------------------------------------------
	pde_cmd_dump_list()
--------------------------------------------------------------------------*/
#if defined(PDE_DEBUG)
void
pde_cmd_dump_list()
{
register PDE *tpde = pde_list_head;
int count = 0;

	curmove(LINES - 1,0);
	while(tpde)
	{
		ff(se,"p:%04x t:%04x n:%04x %c %s\r\n",
			tpde->prev,tpde,tpde->next,(tpde == curr_pde) ? '*' : ' ',
			tpde->logical);
		if((count++ % 15) == 14)
		{
			ff(se,"press return:  ");
			ttygetc(1);
			ff(se,"\r\n");
		}
		tpde = tpde->next;
	}
	ff(se,"press return:  ");
	ttygetc(1);
	touchwin(stdscr);
	touchwin(dirw);
	touchwin(scrw);
	wrefresh(stdscr);
	wrefresh(dirw);
	wrefresh(scrw);

}	/* end of pde_cmd_dump_list */
#endif

/*+-----------------------------------------------------------------------
	pde_list_read()
sample .ecu/phone:
#tri2:426-0624:2400:E:Tridom Pyramid        comment
arc:393-3083:2400:N:Atlanta Radio Club

return 0 if entire list read, else 1 if error (error msg in errmsg)
------------------------------------------------------------------------*/
int
pde_list_read()
{
register itmp;
register token_number;
register char *cptr;
register char *token;
char readpde_buf[128];
FILE *fp_phone;
PDE	 *tpde;
char *str_token();

	if(!ecuphone_name[0])
	{
		get_home_dir(ecuphone_name);
		strcat(ecuphone_name,"/.ecu/phone");
	}

	if( (fp_phone = fopen(ecuphone_name,"r")) == NULL)
	{
		perror_errmsg(ecuphone_name);
		return(1);
	}

/* we have an open .ecu/phone file */
	pde_list_erase();		/* clear any previous directory */
	while(fgets(readpde_buf,sizeof(readpde_buf),fp_phone) != NULL)
	{
		if(readpde_buf[0] == '#')		/* comment? */
			continue;
		if(itmp = strlen(readpde_buf))	/* itmp = len; if > 0 ... */
		{
			itmp--;
			readpde_buf[itmp] = 0;		/* ... strip trailing NL */
		}
		cptr = readpde_buf;				/* first call to str_token, -> buff */
		while((*cptr == 0x20) || (*cptr == TAB))
			cptr++;						/* strip leading spaces */
		if(*cptr == 0)					/* if line all blank, skip it */
			continue;

		if(!(tpde = (PDE *)malloc(sizeof(PDE))))
		{
			fclose(fp_phone);
			strcpy(errmsg,"Out of memory reading phone list");
			return(1);
		}

		tpde->descr[0] = 0;
		tpde->logical[0] = 0;
		tpde->telno[0] = 0;
		tpde->tty[0] = 0;
		tpde->parity = 0;
		tpde->baud = 2400;
		tpde->prev = (PDE *)0;
		tpde->next = (PDE *)0;

		token_number = 0;
		while((token = str_token(cptr,":")) != NULL)
		{
			cptr = NULL;	/* further calls to str_token need NULL */
			switch(token_number++)
			{
				case 0:		/* first field is logical name */
					strncpy(tpde->logical,token,sizeof(tpde->logical));
					tpde->logical[sizeof(tpde->logical) - 1] = 0;
					break;
				case 1:		/* second field is tpde->telno phone number */
					strncpy(tpde->telno,token,sizeof(tpde->telno));
					tpde->telno[sizeof(tpde->telno) - 1] = 0;
					break;
				case 2:		/* third field is line */
					strncpy(tpde->tty,token,sizeof(tpde->tty));
					tpde->tty[sizeof(tpde->tty) - 1] = 0;
					break;
				case 3:		/* fourth field is baud rate */
					tpde->baud = atoi(token);
					break;
				case 4:		/* fifth field is parity */
					switch(itmp = to_lower(token[0]))
					{
						case 'o':
						case 'e':
						case 'm':
						case 's':
							tpde->parity = itmp;
							break;
						default:
						case 'n':
							tpde->parity = 0;
							break;
					}
					break;
				default:
					strncpy(tpde->descr,token,sizeof(tpde->descr));
					tpde->descr[sizeof(tpde->descr) - 1] = 0;
					break;
			}	/* end of switch(token_number) */
		}		/* end while not end of record */

		pde_list_add(tpde);

	}			/* while records left to ready */

	fclose(fp_phone);
	return(0);
}	/* end of pde_list_read */

/*+-----------------------------------------------------------------------
	dirw_display_top()
------------------------------------------------------------------------*/
void
dirw_display_top()
{
	wmove(dirw,1,1);
	wstandout(dirw);
	wprintw(dirw,
" entry name %c telephone number %c tty %c baud P %c description                   ",
		sVR,sVR,sVR,sVR);
	wstandend(dirw);
	wrefresh(dirw);
}	/* end of dirw_display_top() */

/*+-------------------------------------------------------------------------
	dirw_bot_msg(msg)
--------------------------------------------------------------------------*/
void
dirw_bot_msg(msg)
char *msg;
{
register itmp;
register itmp2;
static last_msglen = 0;
#define DIRW_BOT_LINE_TLX 2
#define DIRW_BOT_LINE_MAX_MSGLEN	(DIRW_COLS - DIRW_BOT_LINE_TLX - 8)
char msg2[80];

	if(!last_msglen && !strlen(msg))
		return;

	wmove(dirw,DIRW_LINES - 1,DIRW_BOT_LINE_TLX);

	if((itmp = strlen(msg)) == 0)
	{
		itmp2 = last_msglen + 2;
		for(itmp = 0; itmp < itmp2; itmp++)
			waddch(dirw,(unsigned)sHR);
		last_msglen = 0;
	}
	else
	{
		waddch(dirw,' ');
		if(itmp > DIRW_BOT_LINE_MAX_MSGLEN)
		{
			strncpy(msg2,msg,DIRW_BOT_LINE_MAX_MSGLEN);
			msg2[DIRW_BOT_LINE_MAX_MSGLEN + 1] = 0;
			waddstr(dirw,msg2);
			itmp = strlen(msg2);
		}
		else
		{
			waddstr(dirw,msg);
			itmp = strlen(msg);
		}
		waddch(dirw,' ');
		if((itmp2 = last_msglen - itmp) > 0)
		{
			while(itmp2--)
				waddch(dirw,(unsigned)sHR);
		}
		last_msglen = itmp;		/* remember last message length */
	}
	wrefresh(dirw);
}	/* end of dirw_bot_msg */

/*+-------------------------------------------------------------------------
	pde_display_logical(line,tpde,stand_out)
--------------------------------------------------------------------------*/
void
pde_display_logical(line,tpde,stand_out)
int line;
PDE		*tpde;
int stand_out;
{
	wmove(scrw,line,0);
	waddch(scrw,(unsigned)sVR);

	if(stand_out)
		wstandout(scrw);
	wprintw(scrw," %-10.10s",tpde->logical);
	if(stand_out)
		wstandend(scrw);

}	/* end of pde_display_logical */

/*+-----------------------------------------------------------------------
	pde_display(win,line,tpde,stand_out)
00000000001111111111222222222233333333334444444444555555555566666666667777777777
01234567890123456789012345678901234567890123456789012345678901234567890123456789
| entry name | telephone number | tty | baud P | description                   |
| 0123456789 | 0123456789012345 | 01  | baud P | 01234567890123456789012345678 |
--------------------------------------------------------------------------*/
pde_display(line,tpde,stand_out)
int line;
PDE		*tpde;
int stand_out;
{

	pde_display_logical(line,tpde,stand_out);
	waddch(scrw,' ');
	waddch(scrw,(unsigned)sVR);
	waddch(scrw,' ');
	wprintw(scrw,"%-16.16s %c ",tpde->telno,sVR);
	if(tpde->tty[0])
		wprintw(scrw,"%-2.2s  %c",tpde->tty,sVR);
	else
		wprintw(scrw,"Any %c",sVR);
	wprintw(scrw,"%5u %c %c ",tpde->baud,
		(tpde->parity) ? to_upper(tpde->parity) : 'N',sVR);
	wprintw(scrw,"%-29.29s %c",tpde->descr,sVR);
	return(0);

}	/* end of pde_display */

/*+-----------------------------------------------------------------------
	scrw_fill(first_pde,curr_pde_line)
------------------------------------------------------------------------*/
void
scrw_fill(tpde,curr_pde_line)
register PDE *tpde;
int *curr_pde_line;
{
register line;
register is_curr_pde;

	*curr_pde_line = -1;
	for(line = 0; line < SCRW_LINES; line++)
	{
		if(tpde)
		{
			if(is_curr_pde = (tpde == curr_pde))
				*curr_pde_line = line;
			pde_display(line,tpde,(tpde == curr_pde));
			tpde = tpde->next;
		}
		else
		{
			wmove(scrw,line,0);
			waddch(scrw,(unsigned)sVR);
			wclrtoeol(scrw);
			wmove(scrw,line,SCRW_COLS - 1);
			waddch(scrw,(unsigned)sVR);
		}
	}
	wrefresh(scrw);

}	/* end of scrw_fill */

/*+-------------------------------------------------------------------------
	scrw_fill_at(line_num,first_pde,curr_pde_line)
--------------------------------------------------------------------------*/
void
scrw_fill_at(line_num,tpde,curr_pde_line)
int line_num;
register PDE *tpde;
int *curr_pde_line;
{
register itmp;

	if(!tpde)
	{
		scrw_fill(tpde,curr_pde_line);
/*
		wclear(scrw);
		wrefresh(scrw);
*/
		return;
	}
	for(itmp = 0; itmp < line_num; itmp++)
	{
		if(!tpde->prev)
			break;
		tpde = tpde->prev;
	}

	scrw_fill(tpde,curr_pde_line);

}	/* end of scrw_fill_at */

/*+-------------------------------------------------------------------------
	dirw_cmd_line_setup(prompt)
--------------------------------------------------------------------------*/
void
dirw_cmd_line_setup(prompt)
char *prompt;
{
register icol;
int y;
int x;

	wmove(dirw,DIRW_CMD_LINE,1);
	wstandout(dirw);
	waddch(dirw,' ');
	waddstr(dirw,prompt);
	waddch(dirw,' ');
	getyx(dirw,y,x);
	for(icol = x; icol < DIRW_COLS - 1; icol++)
		waddch(dirw,' ');
	wmove(dirw,y,x);
	wstandend(dirw);
	wrefresh(dirw);
}	/* end of dirw_cmd_line_setup */

/*+-------------------------------------------------------------------------
	dirw_get_cmd()
--------------------------------------------------------------------------*/
uchar
dirw_get_cmd(prompt)
char *prompt;
{
register uchar cmd;

	dirw_cmd_line_setup(prompt);
	cmd = ttygetc(1);
	dirw_bot_msg("");
	return(cmd);

}	/* end of dirw_get_cmd */

/*+-------------------------------------------------------------------------
	pde_cmd_save()
--------------------------------------------------------------------------*/
void
pde_cmd_save()
{
FILE *fpold;
FILE *fpnew;
PDE		*tpde;
char ecuphone_ntmp[256];		/* temp phone directory name */
char iobuf[128];
int count = 0;

	if(!pde_list_dirty)
	{
		dirw_bot_msg("directory has not been modified");
		return;
	}

	strcpy(ecuphone_ntmp,ecuphone_name);
	strcat(ecuphone_ntmp,".t");

	if((fpnew = fopen(ecuphone_ntmp,"w")) == NULL)	/* open old file */
	{
		sprintf(iobuf,"cannot open %s",ecuphone_ntmp);
		dirw_bot_msg(iobuf);
		return;
	}

/* retain commented entries */
	if((fpold = fopen(ecuphone_name,"r")) != NULL)	/* open old file */
	{
		while(fgets(iobuf,sizeof(iobuf),fpold) != NULL)
		{
			if(iobuf[0] == '#')
				fputs(iobuf,fpnew);
		}
		fclose(fpold);
	}

/* write new entries */
	tpde = pde_list_head;
	while(tpde)
	{
		sprintf(iobuf,"%d",count+1);
		dirw_bot_msg(iobuf);
		sprintf(iobuf,"%s:%s:%s:%u:%c:%s\n",tpde->logical,tpde->telno,
			tpde->tty,tpde->baud,
			(tpde->parity) ? to_upper(tpde->parity) : 'N',
			tpde->descr);
		fputs(iobuf,fpnew);
		tpde = tpde->next;
		count++;
	}

	fclose(fpnew);
	unlink(ecuphone_name);
	rename(ecuphone_ntmp,ecuphone_name);
	sprintf(iobuf,"saved %d entries",count);
	dirw_bot_msg(iobuf);
	pde_list_set_dirty(0);

}	/* end of pde_cmd_save */

/*+-------------------------------------------------------------------------
	pde_list_save_if_dirty()
--------------------------------------------------------------------------*/
void
pde_list_save_if_dirty()
{
int cmd = 0;

	if(pde_list_dirty)
	{
		while(!cmd)
		{
			ring_bell();
			cmd = dirw_get_cmd("directory modified: save?");
			if(isupper(cmd))
				cmd = tolower(cmd);
			switch(cmd)
			{
				case 'y': pde_cmd_save(); break;
				case 'n': break;
				default:
					ring_bell();
					cmd = 0;
					break;		
			}
		}
	}
}	/* end of pde_list_save_if_dirty */

/*+-------------------------------------------------------------------------
	pde_cmd_up()
--------------------------------------------------------------------------*/
void
pde_cmd_up()
{
register PDE *tpde;

	if((!curr_pde) || (curr_pde->prev == (PDE *)0))
	{
		ring_bell();
		return;
	}
	if(scrw_curr_pde_line)
	{
		pde_display_logical(scrw_curr_pde_line,curr_pde,0);
		scrw_curr_pde_line--;
		curr_pde = curr_pde->prev;
		pde_display_logical(scrw_curr_pde_line,curr_pde,1);
	}
	else
	{
		tpde = curr_pde;
		curr_pde = curr_pde->prev;
		scrw_fill_at(10,tpde,&scrw_curr_pde_line);
	}

}	/* end of pde_cmd_up */

/*+-------------------------------------------------------------------------
	pde_cmd_down()
--------------------------------------------------------------------------*/
void
pde_cmd_down()
{
register itmp;
register PDE *tpde;

	if((!curr_pde) || (curr_pde->next == (PDE *)0))
	{
		ring_bell();
		return;
	}
	if(scrw_curr_pde_line < (SCRW_LINES - 1))
	{
		pde_display_logical(scrw_curr_pde_line,curr_pde,0);
		scrw_curr_pde_line++;
		curr_pde = curr_pde->next;
		pde_display_logical(scrw_curr_pde_line,curr_pde,1);
	}
	else
	{
		tpde = curr_pde;
		curr_pde = curr_pde->next;
		scrw_fill_at(SCRW_LINES - 10,tpde,&scrw_curr_pde_line);
	}

}	/* end of pde_cmd_down */

/*+-------------------------------------------------------------------------
	check_curr_pde() -- return 1 if there is a current pde, else 0
--------------------------------------------------------------------------*/
check_curr_pde()
{
	if(!curr_pde)
	{
		dirw_bot_msg("no directory entry selected");
		return(0);
	}
	return(1);
}	/* end of check_curr_pde */

/*+-------------------------------------------------------------------------
	pde_add_or_edit_read(prompt,edit,x,buf,max,delim)
max must not wrap around to another line
--------------------------------------------------------------------------*/
pde_add_or_edit_read(prompt,edit,x,buf,max,delim)
char *prompt;
int edit;
int x;
char *buf;
int max;
char *delim;
{
register itmp;
char s82[82];

	while(1)
	{
		dirw_cmd_line_setup(prompt);
		wstandout(scrw);
		if(edit)
			strcpy(s82,buf);
		itmp = wingets(scrw,scrw_curr_pde_line,x,s82,max,delim,edit);
		wstandend(scrw);
		
		switch(*delim)
		{
			case ESC:
				return;

			case CR:
			case NL:
				if(edit && (itmp == 0))
				{
					*delim = TAB;
					return;
				}
				strcpy(buf,s82);
				return;

			case CTL_B:
				*buf = 0;
				return;

			case TAB:
				if(!edit)
					continue;
				*buf = 0;
				return;

			default:
				break;
		}
	}
}	/* end of pde_add_or_edit_read */

/*+-------------------------------------------------------------------------
	pde_add_or_edit(tpde,edit)
if called with edit == 1, tpde MUST == curr_pde !!!
00000000001111111111222222222233333333334444444444555555555566666666667777777777
01234567890123456789012345678901234567890123456789012345678901234567890123456789
| entry name | telephone number | tty | baud P | description                   |
| 0123456789 | 0123456789012345 | 01  | baud P | 01234567890123456789012345678 |
--------------------------------------------------------------------------*/
int
pde_add_or_edit(tpde,edit)
register PDE *tpde;
int edit;
{
register itmp;
int input_state = 0;
int changed;
char s50[50];
char delim;
int done = 0;
int y,x;
PDE *old_curr_pde;
uint baud;

	if(!edit)
	{
		dirw_bot_msg("ESC: abort  ^U: erase input");
		dirw_cmd_line_setup("Enter new directory entry name: ");
		getyx(dirw,y,x);
		wstandout(dirw);
		wingets(dirw,y,x,tpde->logical,10+1,&delim,0);
		wstandend(dirw);
		if((!strlen(tpde->logical)) || (tpde->logical[0] == ESC))
		{
			dirw_bot_msg("add aborted");
			return(0);
		}
	
		if(!isalpha(tpde->logical[0]))
		{
			dirw_bot_msg("first character must be alphabetic");
			return(0);
		}

		if(pde_list_search(tpde->logical,1))
		{
			sprintf(s50,"'%s' is already in the directory",tpde->logical);
			ring_bell();
			dirw_bot_msg(s50);
			return(0);
		}

		tpde->descr[0] = 0;
		tpde->telno[0] = 0;
		tpde->tty[0] = 0;
		tpde->parity = '-';
		tpde->baud = 0;				
	
		pde_list_add(tpde);
		old_curr_pde = curr_pde;
		curr_pde = tpde;
		scrw_fill_at(SCRW_LINES / 2,tpde,&scrw_curr_pde_line);
		tpde = curr_pde;
	}	/* end of add code */

	if(edit)
		dirw_bot_msg("ESC: exit  ^U: erase input  ^B: back  TAB: forward");
	else
		dirw_bot_msg("ESC: abort  ^U: erase input  ^B: back up");

/* add/edit common */
	while(!done)
	{
		changed = 0;
		switch(input_state)
		{
			case 0:
				if(edit)
					strcpy(s50,tpde->telno);
				pde_add_or_edit_read("Enter telephone number",
					edit,15,tpde->telno,16+1,&delim);
				if(edit && strcmp(tpde->telno,s50))
					changed = 1;
				break;

			case 1:
				if(edit)
					strcpy(s50,(tpde->tty[0]) ? tpde->tty : "Any");
				pde_add_or_edit_read("Enter tty identifier (i.e., 1a)",
					edit,34,s50,3+1,&delim);
				if(edit && strcmp((tpde->tty[0]) ? tpde->tty : "Any",s50))
					changed = 1;
				if(!strcmpi(s50,"any"))
					tpde->tty[0] = 0;
				else
				{
					strncpy(tpde->tty,s50,2);
					tpde->tty[2] = 0;
				}
				break;

			case 2:
				if(edit)
					sprintf(s50,"%u",tpde->baud);
				pde_add_or_edit_read(
				"Enter baud rate (110,300,600,1200,2400,4800,9600,19200,38400)",
					edit,39,s50,5+1,&delim);
				if(valid_baud_rate(baud = atoi(s50)) < 0)
				{
					ring_bell();
					continue;
				}
				if(edit && (tpde->baud != baud ))
					changed = 1;
				tpde->baud = baud;
				break;

			case 3:
				if(edit)
					sprintf(s50,"%c",(tpde->parity) ? tpde->parity : 'n');
				pde_add_or_edit_read("Enter parity (n,o,e)",
					edit,45,s50,1+1,&delim);
				switch(s50[0] = to_lower(s50[0]))
				{
					case 'n':
						s50[0] = 0;
					case 'o':
					case 'e':
						if(edit && (tpde->parity != s50[0]))
							changed = 1;
						tpde->parity = s50[0];
						break;
					default:
						ring_bell();
						continue;
				}
				break;

			case 4:
				if(edit)
					strcpy(s50,tpde->descr);
				pde_add_or_edit_read("Enter description",
					edit,49,tpde->descr,29+1,&delim);
				if(edit && strcmp(tpde->descr,s50))
					changed = 1;
				break;
		}

		pde_display(scrw_curr_pde_line,tpde,1);	/* regardless of delimiter */

		switch(delim) /* process delimiter */
		{
			case CTL_L:
			case CTL_R:
				clear_screen();
				touchwin(stdscr);
				wrefresh(stdscr);
				touchwin(dirw);
				wrefresh(dirw);
				touchwin(scrw);
				wrefresh(scrw);
				break;

			case CTL_B:
				if(input_state)
					input_state--;
				break;

			case TAB:
				input_state++;
				input_state %= 5;
				break;

			case ESC:
				if(edit)
				{
					dirw_bot_msg("edit finished");
					done = 1;
				}
				else
				{
					pde_list_remove(tpde);
					curr_pde = old_curr_pde;
					scrw_fill_at(scrw_curr_pde_line + 1,curr_pde,
							&scrw_curr_pde_line);
					dirw_bot_msg("add aborted");
					return(0);
				}
				break;

			case NL:
				if(edit && changed)
					pde_list_set_dirty(1);
				/* fall thru */
			default:
				if(!edit && (input_state == 4))
					done = 1;
				else 
				{
					input_state++;
					input_state %= 5;
				}
				break;
		}
	}
	return(1);

}	/* end of pde_add_or_edit */

/*+-------------------------------------------------------------------------
	pde_cmd_add(tpde)
if tpde != 0, it is an already valid pde that is to be added
else if == 0, interactive add
--------------------------------------------------------------------------*/
void
pde_cmd_add(tpde)
register PDE *tpde;
{
register itmp;

	if(tpde)
	{
		pde_list_add(tpde);
	}
	else
	{
		if(!(tpde = (PDE *)malloc(sizeof(PDE))))
		{
			dirw_bot_msg("Out of memory -- cannot add new entry");
			return;
		}
		if(!pde_add_or_edit(tpde,0))	/* routine will add to list ... */
		{								/* ... if good return */
			free((char *)tpde);
			return;
		}
	}
	pde_list_set_dirty(1);

	curr_pde = tpde;
	scrw_fill_at(SCRW_LINES / 2,curr_pde,&scrw_curr_pde_line);

}	/* end of pde_cmd_add */

/*+-------------------------------------------------------------------------
	pde_cmd_remove_oops()
--------------------------------------------------------------------------*/
void
pde_cmd_remove_oops()
{
	if(!remove_pde)
	{
		dirw_bot_msg("no removed entry to restore");
		return;
	}
	pde_cmd_add(remove_pde);
	pde_list_set_dirty(remove_dirty_flag);
	remove_pde = (PDE *)0;
}	/* end of pde_cmd_remove_oops */

/*+-------------------------------------------------------------------------
	pde_cmd_remove()
--------------------------------------------------------------------------*/
void
pde_cmd_remove()
{
register itmp;
register PDE *tpde;
char s80[80];

	if(!check_curr_pde())
		return;

	remove_pde = curr_pde;
	remove_dirty_flag = pde_list_dirty;
	pde_list_remove(curr_pde);
	pde_list_set_dirty(1);

	if(pde_list_quan)
		scrw_fill_at(scrw_curr_pde_line + 1,curr_pde,&scrw_curr_pde_line);
	else
		scrw_fill((PDE *)0,&scrw_curr_pde_line);

	ring_bell();
	sprintf(s80,"if you did not mean to to remove '%s', press 'o' (oops) NOW!",
			remove_pde->logical);
	dirw_bot_msg(s80);

}	/* end of pde_cmd_remove */

/*+-------------------------------------------------------------------------
	pde_cmd_find()
--------------------------------------------------------------------------*/
void
pde_cmd_find()
{
register itmp;
register PDE *tpde;
char findname[12];
char s50[50];
char delim;
int y,x;

	dirw_bot_msg("ESC: abort  ^U: erase input");
	dirw_cmd_line_setup("Directory entry name to find: ");
	getyx(dirw,y,x);
	wstandout(dirw);
	wingets(dirw,y,x,findname,10+1,&delim,0);
	wstandend(dirw);
	if((!strlen(findname)) || (delim == ESC))
		return;

	if(! (tpde = pde_list_search(findname,0)))
	{
		dirw_bot_msg(errmsg);
		return;
	}
	curr_pde = tpde;
	scrw_fill_at(SCRW_LINES / 2,tpde,&scrw_curr_pde_line);

}	/* end of pde_cmd_find */

/*+-------------------------------------------------------------------------
	pde_list_manager()
--------------------------------------------------------------------------*/
void
pde_list_manager()
{
register uchar cmd = 0;
int rcvr_active = (rcvr_pid > 0) || (rcvr_pid == -2);
int done;
char s80[80];
WINDOW *window_create();
char *find_procedure();
extern int proc_level;

	if(rcvr_active && (rcvr_pid != -2))
		kill_rcvr_process(SIGUSR1);

	windows_start();
	dirw = window_create("dialing directory",3,0,0,DIRW_LINES,DIRW_COLS);
	dirw_display_top();

	scrw = subwin(dirw,SCRW_LINES,SCRW_COLS,SCRW_TLY,SCRW_TLX);
	scrollok(scrw,0);
	if(!pde_list_quan)
	{
		if(pde_list_read())
			dirw_bot_msg(errmsg);
		else if(!pde_list_quan)
			dirw_bot_msg("directory empty");
	}

	if(pde_list_quan)
	{
		if(curr_pde)
			scrw_fill_at(scrw_curr_pde_line,curr_pde,&scrw_curr_pde_line);
		else
		{
			curr_pde = pde_list_head;
			scrw_fill(curr_pde,&scrw_curr_pde_line);
		}
	}

	pde_list_set_dirty(-1);

	done = 0;
	while(!done)
	{
		cmd = dirw_get_cmd(
#if defined(M_XENIX) || defined(M_UNIX)
	"d,j:down u,k:up e:edit a:add r:remove s:save f:find END:dial ESC,q:quit");
#else
	"d,j:down u,k:up e:edit a:add r:remove s:save f:find ^D:dial  ESC,q:quit");
#endif
		if((cmd != 'o') && (remove_pde))
		{
			free((char *)remove_pde);
			remove_pde = (PDE *)0;
		}

		switch(cmd)
		{
			case XFcurdn:
			case 'd':
			case 'j': pde_cmd_down(); break;

			case XFcurup:
			case 'u':
			case 'k': pde_cmd_up(); break;

			case 's': pde_cmd_save(); break;

			case CR:
			case NL:
			case XFend:
				cmd = XFend;
				if(!check_curr_pde())
					break;
				pde_list_save_if_dirty();
				windows_end(dirw);

/* dont do this here
				redisplay_rcvr_screen();
*/

				done = 1;
#ifdef AUTO_DIAL_PROC
				if(!proc_level && find_procedure(curr_pde->logical))
				{
				char *pargv[2];
					pargv[0] = curr_pde->logical;
					pargv[1] = "!MENU";
					rcvr_pid = 0;
					do_proc(2,pargv);
					start_rcvr_process(1);
				}
				else
				{
					if(rcvr_active)
						rcvr_pid = -2;	/* hayes_dial shud start rcvr */
					hayes_dial_pde(curr_pde);
				}
#else
				if(rcvr_active)
					rcvr_pid = -2;	/* hayes_dial shud start rcvr */
				hayes_dial_pde(curr_pde);
#endif
				break;

			case ESC:
			case CTL_C:
				cmd = 'q';
			case 'q':
				done = 1;
				continue;

			case 'f': pde_cmd_find(); break;

			case 'r': pde_cmd_remove(); break;
			case 'o': pde_cmd_remove_oops(); break;

			case 'a': pde_cmd_add((PDE *)0); break;

			case 'e': pde_add_or_edit(curr_pde,1); break;

#if defined(PDE_DEBUG)
			case CTL_BSLASH:	/* undocumented ^\ */
				pde_cmd_dump_list(); break;
#endif

			case CTL_L:
			case CTL_R:
				touchwin(stdscr);
				wrefresh(stdscr);
				touchwin(dirw);
				wrefresh(dirw);
				touchwin(scrw);
				wrefresh(scrw);
				break;

			default:
				sprintf(s80,"illegal command: %c",
					(isprint(cmd)) ? cmd : '?');
				dirw_bot_msg(s80);
				break;
		}
	}
	if(cmd == XFend)
		return;
	pde_list_save_if_dirty();
	windows_end(dirw);

	redisplay_rcvr_screen();

	if(rcvr_active)
		start_rcvr_process(0);
}	/* end of pde_list_manager */

/*+-------------------------------------------------------------------------
	choose_line(baud) - user is dialing a remote and will take 'Any' line
--------------------------------------------------------------------------*/
void
choose_line(baud)
uint baud;
{
struct dvent *dve;
char s32[32];

/* check out current line choice */
	if(access(shm->Lline,6))
		goto GET_LINE_THAT_MATCHES_BAUD;
	dve = getdvline(shm->Lline + 8);
	enddvent();
	if(!dve)	/* if no match in Devices, let ecu fend for itself */
		return;

/* if shm->Lline is open and baud rate ok, no further */
	if(shm->Liofd > 0)
	{
		if((dve->low_baud <= baud) && (baud <= dve->high_baud))
			return;
		lclose();	/* won't do ... gotta find a new one */
		goto GET_LINE_THAT_MATCHES_BAUD;
	}

	if(!shm->Lline[0])
		memset(shm->Lline,0,sizeof(shm->Lline));

/* see if shm->Lline in use by someone else; if not and baud rate ok, no further */
	switch(utmp_status(shm->Lline))
	{
		case US_NOTFOUND:	/* not in utmp, or getty dead */
			if(line_locked(shm->Lline))
				goto GET_LINE_THAT_MATCHES_BAUD;
		case US_LOGIN:		/* enabled for login, idle */
			if((dve->low_baud <= baud) && (baud <= dve->high_baud))
				return;
	}

/* we've got to pick a new line */
GET_LINE_THAT_MATCHES_BAUD:

	strcpy(s32,"/dev/tty");
	while(1)
	{
		if(!(dve = getdvbaud(baud)))
		{
			enddvent();		/* no match ... */
			return;			/* ... so let current shm->Lline fail */
		}
/* by now, we know shm->Lline wont work */
		if(!strcmp(dve->line,shm->Lline + 5))
			continue;
/* if not acu, dont use it */
		if(ulindex(dve->type,"ACU") < 0)
			continue;
		strcpy(&s32[8],dve->line + 3);	/* yeech ... make "/dev/ttyxx" */
		switch(utmp_status(s32))
		{
			case US_NOTFOUND:	/* not in utmp, or getty dead */
				if(line_locked(s32))
					continue;
			case US_LOGIN:		/* enabled for login, idle */
				strcpy(shm->Lline,s32);	/* @@@@@@@@@@@ MATCH @@@@@@@@@ */
				shm->Lline[9] = to_lower(shm->Lline[9]);
				enddvent();	
				return;
		}
	}
	/*NOTREACHED*/

}	/* end of choose_line */

/*+-------------------------------------------------------------------------
	copy_pde_to_lvar(tpde)

if changing line, close old line and open new one
--------------------------------------------------------------------------*/
void
copy_pde_to_lvar(tpde)
register PDE *tpde;
{
int reopen = 0;
int lerr;

	if(tpde->tty[0])
	{
		if(shm->Lline[0] && shm->Lline[8] && strcmp(tpde->tty,shm->Lline + 8))
		{
			reopen = 1;
			shm->Lmodem_already_init = 0;
			lclose();
		}
		strcpy(shm->Lline,"/dev/tty");
		strcat(shm->Lline,tpde->tty);
	}
	else
		choose_line(tpde->baud);

	if(shm->Liofd < 0)
		reopen = 1;
	strcpy(shm->Llogical,tpde->logical);
	strcpy(shm->Ldescr,tpde->descr);
	strcpy(shm->Ltelno,tpde->telno);
	if(!shm->Ldescr[0])
		strcpy(shm->Ldescr,shm->Llogical);
	shm->Lparity = tpde->parity;
	if(shm->Lbaud != tpde->baud)
		shm->Lmodem_already_init = 0;
	shm->Lbaud =tpde->baud;	
	if(reopen)
	{
		if(lerr = lopen())
		{
			curbotleft();
			if(lerr == LOPEN_LCKERR || lerr == LOPEN_UNKPID)
				pputs("port not available\n");
			pprintf("%s: %s\n",shm->Lline,lopen_err_text(lerr));
			hangup(HANGUP_LINE_OPEN_ERROR);
		}
	}
	else
	{
		lset_baud_rate(1);
		lset_parity(1);
	}

}	/* end of copy_pde_to_lvar */

/*+-----------------------------------------------------------------------
	lookup_logical_telno()

A logical telephone number is either a symbolic identifer
or an lcb->telno telephone number.  A symbolic identifer is a string
whose initial character is a letter.
An lcb->telno telephone number begins with a numeral.

This function converts a logical telephone number 
to a telephone number (suitable for dialing by a Hayes or
compatible modem).  It is called by command line processing
when 'ecu logical-name' is specified or in response to
a %dial logical-name.  It is not used by the curses directory
manager.

If the first character of 'shm->Llogical' is a digit, the entire
input string is copied to 'shm->Ltelno'.  If 'shm->Llogical' has a 
non-numeric first character, then the user's home directory is
searched for the file .ecu/phone (~/.ecu/phone). The file is a series
of records terminated with a newline.  Each record has two or three
fields separated by colons.  The first field is the logical telephone
number and the second field is the telephone number.  The third
(optional) field contains a string to further identify the telephone
number being called.

CASE IS INSIGNIFICANT in logical entry names for this procedure

The function returns one of the following:

  1		if no error occurs
  0		if not numeric phone number and logical string not found in file.

------------------------------------------------------------------------*/
lookup_logical_telno()
{
register PDE *tpde;

	if(!pde_list_quan)
	{
		if(pde_list_read())
			return(0);
	}

/* if literal phone number, return it immediately */

	if(isdigit(shm->Llogical[0]))
	{
		strcpy(shm->Ltelno,shm->Llogical);
		strcpy(shm->Ldescr,shm->Llogical);
		return(1);
	}

/* if logical phone number */
	if(tpde = pde_list_search(shm->Llogical,0))	/* inexact search */
	{
		copy_pde_to_lvar(tpde);
		return(1);
	}

	shm->Llogical[0] = 0;
	shm->Lrname[0] = 0;
	shm->Ltelno[0] = 0;
	shm->Ldescr[0] = 0;
	return(0);

}	/* end of lookup_logical_telno */

/* end of ecuphone.c */
/* vi: set tabstop=4 shiftwidth=4: */
