/* CHK=0x8BBA */
/*+-------------------------------------------------------------------------
	ecurcvr.c -- rcvr process + ANSI filter
	wht%n4hgf@gatech.edu

  Defined functions:
	accumulate_ansi_sequence(rchar)
	ansi_CUB()
	ansi_CUD()
	ansi_CUF()
	ansi_CUP()
	ansi_CUU()
	ansi_DSR()
	ansi_ED()
	ansi_EL()
	ansi_SGR()
	is_ansi_terminator(rchar)
	lgetc_rcvr()
	process_ansi_sequence()
	process_rcvd_char(rchar)
	rcvd_ESC()
	rcvr()
	rcvr_log_open()
	redisplay_rcvr_screen()
	saved_cursor_restore_cursor()
	saved_cursor_save_cursor()
	spaces(buf,buflen)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-12-1990-14:00-wht@n4hgf-fseek logfile on received backspace if not raw */
/*:04-10-1990-17:34-wht@n4hgf-provisions for ecu xmtr logging when rcvr dead */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-23-1990-13:58-wht@n4hgf-add redisplay_rcvr_screen */
/*:03-16-1990-03:43-wht@n4hgf-log open msg has more info */
/*:01-17-1990-05:53-wht-keep shm-screen accurate during scrolling */
/*:01-17-1990-05:46-wht-fix EL erase in line */
/*:11-21-1989-19:12-wht-fix bug with CSI= sequences */
/*:11-21-1989-18:54-wht-ensure bogus CUP doesnt make for bad cursor_{y,x} */
/*:11-10-1989-21:36-wht-restructure rcvr process */
/*:11-05-1989-16:48-wht-rename ansi_y to cursor_y, ansi_x to cursor_x */
/*:11-05-1989-16:08-wht-keep screen in shared memory */
/*:11-04-1989-07:00-wht-keep ansi cursor position in shm */
/*:11-04-1989-06:55-wht-externalize shm addr + count rcvd chars here */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecukey.h"

extern int errno;
extern char rcvr_log_file[];	/* if rcvr_log!= 0,log filename */
extern int rcvr_log;			/* rcvr log active if != 0 */
extern FILE *rcvr_log_fp;		/* rcvr log file */
extern int rcvr_log_raw;		/* if true, log all, else filter ctl chrs */
extern int rcvr_log_append;		/* if true, append, else scratch */
extern int rcvr_log_flusheach;	/* if true, flush log on each char */
extern int rcvr_log_gen_title;

extern int want_bell_notify;
extern int tcap_LINES;	/* terminal line quantity -- see ecutcap.c */
extern int tcap_COLS;	/* terminal column quantity -- see ecutcap.c */

static char esc = ESC;
#define MAX_ANSI_LEN	30	/* generous */
char ansibuf[MAX_ANSI_LEN];
char *ansi;
int ansilen = 0;
int in_ansi_accumulation = 0;

int saved_cursor_y;
int saved_cursor_x;

#define RCVR_RDQUAN		250
char lgetc_buf[RCVR_RDQUAN];
char *lgetc_ptr;
int lgetc_count = 0;
int vmin;				/* quick copy of current vmin value */

#ifdef ANSI_DEBUG
FILE *wfp = 0;
#endif

/*+-------------------------------------------------------------------------
	redisplay_rcvr_screen() - redisplay logical receiver screen
As of writing, this function is called only by the XMTR process
--------------------------------------------------------------------------*/
void
redisplay_rcvr_screen()
{
register y;

	setcolor(colors_current);
	stand_end();
	for(y = 0; y < tcap_LINES; y++)
	{
		curmove(y,0);
		fwrite(&shm->screen[y][0],
			((y != tcap_LINES - 1) ? tcap_COLS : tcap_COLS - 1),
			1,se);
	}
	eeol();
	curmove(shm->cursor_y,shm->cursor_x);
}	/* end of redisplay_rcvr_screen */

/*+-------------------------------------------------------------------------
	spaces(buf,buflen) - fill with spaces
--------------------------------------------------------------------------*/
void
spaces(buf,buflen)
register char *buf;
register buflen;
{
	if(!buflen)
		return;
	while(buflen--)
		*buf++ = ' ';
}	/* end of spaces */

/*+-------------------------------------------------------------------------
	lgetc_rcvr() -- rcvr version of get char from line
--------------------------------------------------------------------------*/
int
lgetc_rcvr()
{
extern int errno;
char char_rtnd;

	if(!lgetc_count)
	{
		if(vmin == 1)
		{
			char_rtnd = lgetc_xmtr();
			return(char_rtnd);
		}
		lgetc_ptr = lgetc_buf;
READ_AGAIN:
		errno = 0;
		if((lgetc_count =
			read(shm->Liofd,lgetc_buf,RCVR_RDQUAN)) < 0)
		{
			if(errno == EINTR)			/* if signal interrupted, ... */
				goto READ_AGAIN;
			hangup(HANGUP_LINE_READ_ERROR);
		}
		shm->rcvd_chars += lgetc_count;
		shm->rcvd_chars_this_connect += lgetc_count;

	}
	if(!lgetc_count)
		goto READ_AGAIN;

	lgetc_count--;
	return(*lgetc_ptr++);
}	/* end of lgetc_rcvr */

/*+-------------------------------------------------------------------------
	ansi_SGR()

The PC world expects to be able to be able to chain 0,1 and 3x,4x params
together with semicolons.
--------------------------------------------------------------------------*/
void
ansi_SGR()
{
register itmp;
register char *cptr;
char SGRstr[MAX_ANSI_LEN];
char *token;
char *str_token();

	if(ansilen <= 3)	/* 'ESC[<0-9>m' and 'ESC[m' -- quickly handled */
	{
		write(TTYERR,&esc,1);
		write(TTYERR,ansibuf,ansilen);
		return;
	}

/* check XENIX 'ESC[<2,3,7>m' extensions */
	switch(itmp = atoi(ansibuf + 1))
	{
		case 7: /* XENIX 'ESC[7;<0-15>;<0-15>m' set fore/background color */
			itmp = atoi(ansibuf + 3);	/* second parameter */
			if(itmp > 15)				/* not XENIX extension */
				break;
			/* fall through */
		case 2:	/* XENIX 'ESC[2;<0-15>;<0-15>m' set fore/background color */
		case 3:	/* XENIX 'ESC[3;<0-1>m' color only set/clear blink */
			write(TTYERR,&esc,1);
			write(TTYERR,ansibuf,ansilen);
			return;
		default:
			break;
	}

/* not XENIX extension */
	ansibuf[ansilen - 1] = 0;	/* get rid of 'm' */
	cptr = ansibuf + 1;			/* get rid of '[' */

	while((token = str_token(cptr,";")) != NULL)
	{
		cptr = (char *)0;	/* further calls to str_token need NULL */
		sprintf(SGRstr,"\033[%sm",token);
		write(TTYERR,SGRstr,strlen(SGRstr));
	}

}	/* end of ansi_SGR */

/*+-------------------------------------------------------------------------
	ansi_CUP() -- cursor position (also HVP horiz/vertical position)
--------------------------------------------------------------------------*/
void
ansi_CUP()
{
register param_count = 0;
char ansicopy[MAX_ANSI_LEN];
register char *cptr = ansicopy;
register char *token;
char *str_token();

	strcpy(cptr,ansibuf + 1);
	*(cptr + ansilen - 2) = 0;

	while((token = str_token(cptr,";")) != NULL)
	{
		cptr = NULL;	/* further calls to str_token need NULL */
		switch(++param_count)
		{
			case 1:	shm->cursor_y = atoi(token) - 1; break;
			case 2:	shm->cursor_x = atoi(token) - 1; break;
		}
	}
	switch(param_count)
	{
		case 0:
			shm->cursor_y = 0;
		case 1:
			shm->cursor_x = 0;
	}
	if((unsigned)shm->cursor_x > 79)
		shm->cursor_x = 0;
	if((unsigned)shm->cursor_y > 42)
		shm->cursor_y = 0;

}	/* end of ansi_CUP */

/*+-------------------------------------------------------------------------
	ansi_CUU() - cursor up
--------------------------------------------------------------------------*/
void
ansi_CUU()
{
register count;

	if(ansilen == 2)		/* no param */
		count = 1;
	else
		count = atoi(ansibuf + 1);

	shm->cursor_y -= count;
	if(shm->cursor_y < 0)
		shm->cursor_y = 0;

}	/* end of ansi_CUU */

/*+-------------------------------------------------------------------------
	ansi_CUD()
--------------------------------------------------------------------------*/
void
ansi_CUD()
{
register count;
	if(ansilen == 2)		/* no param */
		count = 1;
	else
		count = atoi(ansibuf + 1);

	shm->cursor_y += count;
	if(shm->cursor_y > tcap_LINES - 1)
		shm->cursor_y = tcap_LINES - 1;
}	/* end of ansi_CUD */

/*+-------------------------------------------------------------------------
	ansi_CUF() -- cursor forward
--------------------------------------------------------------------------*/
void
ansi_CUF()
{
register count;
	if(ansilen == 2)		/* no param */
		count = 1;
	else
		count = atoi(ansibuf + 1);

	shm->cursor_x += count;
	if(shm->cursor_x > tcap_COLS - 1)
		shm->cursor_x = tcap_COLS - 1;

}	/* end of ansi_CUF */

/*+-------------------------------------------------------------------------
	ansi_CUB()
--------------------------------------------------------------------------*/
void
ansi_CUB()
{
register count;
	if(ansilen == 2)		/* no param */
		count = 1;
	else
		count = atoi(ansibuf + 1);

	shm->cursor_x -= count;
	if(shm->cursor_x < 0)
		shm->cursor_x = 0;

}	/* end of ansi_CUB */

/*+-------------------------------------------------------------------------
	ansi_DSR() -- device status report
--------------------------------------------------------------------------*/
void
ansi_DSR()
{
char response[MAX_ANSI_LEN];
	sprintf(response,"\033[%d;%dR",shm->cursor_y + 1,shm->cursor_x + 1);
	write(shm->Liofd,response,strlen(response));
}	/* end of ansi_DSR */

/*+-------------------------------------------------------------------------
	ansi_ED() -- erase in display
--------------------------------------------------------------------------*/
void
ansi_ED()
{
register param;
	if(ansilen == 2)		/* no param */
		param = 0;
	else
		param = atoi(ansibuf + 1);

	switch(param)
	{
		case 0:	/* erase to end of display */
			spaces(&shm->screen[shm->cursor_y][shm->cursor_x],
				sizeof(shm->screen) - ((shm->cursor_y * 80) + shm->cursor_x));
			break;
		case 1:	/* erase from beginning of display */
			spaces((char *)shm->screen,(shm->cursor_y * 80) + shm->cursor_x);
			break;
		case 2:	/* clear display */
			shm->cursor_y = 0;
			shm->cursor_x = 0;
			spaces((char *)shm->screen,sizeof(shm->screen));
			break;
	}

}	/* end of ansi_ED */

/*+-------------------------------------------------------------------------
	ansi_EL() -- erase in line
--------------------------------------------------------------------------*/
void
ansi_EL()
{
register param;

	if(ansilen == 2)		/* no param */
		param = 0;
	else
		param = atoi(ansibuf + 1);

	switch(param)
	{
		case 2:	/* clear line */
			shm->cursor_x = 0;
		case 0:	/* erase to end of line */
			spaces(&shm->screen[shm->cursor_y][shm->cursor_x],
				80 - shm->cursor_x);
			break;
		case 1:	/* erase from beginning of line */
			spaces(&shm->screen[shm->cursor_y][0],shm->cursor_x);
			break;
	}

}	/* end of ansi_EL */

/*+-------------------------------------------------------------------------
	saved_cursor_save_cursor() -- part 1 of why this crap is necessary
--------------------------------------------------------------------------*/
void
saved_cursor_save_cursor()
{
	saved_cursor_y = shm->cursor_y;
	saved_cursor_x = shm->cursor_x;
}	/* end of saved_cursor_save_cursor */

/*+-------------------------------------------------------------------------
	saved_cursor_restore_cursor() - part 2 of why this crap is necessary
--------------------------------------------------------------------------*/
void
saved_cursor_restore_cursor()
{
char crock[MAX_ANSI_LEN];
	shm->cursor_y = saved_cursor_y;
	shm->cursor_x = saved_cursor_x;
	sprintf(crock,"\033[%d;%dH",shm->cursor_y + 1,shm->cursor_x + 1);
	write(TTYERR,crock,strlen(crock));
}	/* end of saved_cursor_restore_cursor */

/*+-------------------------------------------------------------------------
	rcvd_ESC() - ESC seen-prepare to accumulate ansi sequence
--------------------------------------------------------------------------*/
void
rcvd_ESC()
{
#ifdef ANSI_DEBUG
	if(wfp)
		fprintf(wfp,"ESC\n");
#endif
	ansi = ansibuf;
	ansilen = 0;
	in_ansi_accumulation = 1;

}	/* end of rcvd_ESC */

/*+-------------------------------------------------------------------------
	is_ansi_terminator(rchar) - is character terminator for ansi sequence?
--------------------------------------------------------------------------*/
is_ansi_terminator(rchar)
register uint rchar;
{
	return(isalpha(rchar) || (rchar == '@'));
}	/* end of is_ansi_terminator */

/*+-------------------------------------------------------------------------
	accumulate_ansi_sequence(rchar)
--------------------------------------------------------------------------*/
void
accumulate_ansi_sequence(rchar)
uint rchar;
{
	if(ansilen == MAX_ANSI_LEN - 2)
	{
		in_ansi_accumulation = 0;
		return;
	}

#ifdef ANSI_DEBUG
	if(wfp)
		fprintf(wfp,"aas: %02x %c ansilen=%d\n",
			rchar,(rchar & 0x7f < 0x20) ? '.' : (rchar & 0x7f),ansilen);
#endif
	*ansi++ = (uchar)rchar;
	*ansi   = 0;
	ansilen++;

}	/* end of accumulate_ansi_sequence */

/*+-------------------------------------------------------------------------
	process_ansi_sequence() - a full ansi sequence is to be decoded
--------------------------------------------------------------------------*/
void
process_ansi_sequence()
{
register itmp;

#ifdef ANSI_DEBUG
	if(wfp)
		fprintf(wfp,"pas: ansilen=%d trigger=%02x %c\n",ansilen,
			ansibuf[ansilen - 1],ansibuf[ansilen - 1]);
#endif
	if(!in_ansi_accumulation)
		return;

	in_ansi_accumulation = 0;

	itmp = 1;		/* assume write needed */
	if((ansilen > 1) && (ansibuf[1] == '='))
		;
	else switch(ansibuf[ansilen - 1])
	{
		case 'm': ansi_SGR(); itmp = 0; break;
		case 'f': case 'H': ansi_CUP(); break;
		case 'A': ansi_CUU(); break;
		case 'B': ansi_CUD(); break;
		case 'C': ansi_CUF(); break;
		case 'D': ansi_CUB(); break;
		case 's': saved_cursor_save_cursor(); itmp = 0; break;
		case 'u': saved_cursor_restore_cursor(); itmp = 0; break;
		case 'n': ansi_DSR(); itmp = 0; break;
		case 'J': ansi_ED(); break;
		case 'K': ansi_EL(); break;
		default:
			break;
	}

	if(itmp)
	{
		write(TTYERR,&esc,1);
		write(TTYERR,ansibuf,ansilen);
	}

#ifdef ANSI_DEBUG
	if(wfp)
		fprintf(wfp,"pas: y,x=%d,%d\n",shm->cursor_y,shm->cursor_x);
#endif
}	/* end of process_ansi_sequence */

/*+-------------------------------------------------------------------------
	rcvr_log_open()
--------------------------------------------------------------------------*/
void
rcvr_log_open()
{

	if(rcvr_log)		/* if xmtr set us up for logging */
	{
		rcvr_log_fp = fopen(rcvr_log_file,rcvr_log_append ? "a" : "w");
		rcvr_log_append = 1;	/* until next %log -s */
		if(!rcvr_log_fp)
		{
			ff(se,"ecu RCVR: Could not open log file: %s\r\n",rcvr_log_file);
			ff(se,"recording aborted.\r\n");
		}
		else if(!rcvr_log_raw && rcvr_log_gen_title)
		{
		char tstr[80];
			get_tod(2,tstr);
			fprintf(rcvr_log_fp,"\n====> %s (%s, %s, %s) %s\n\n",
				shm->Lrname,shm->Llogical,
				shm->Ldescr,(shm->Ltelno[0]) ? shm->Ltelno : "NONE",tstr);
		}
		rcvr_log_gen_title = 0;
	}
}	/* end of rcvr_log_open */

/*+-------------------------------------------------------------------------
	process_rcvd_char(rchar)
--------------------------------------------------------------------------*/
process_rcvd_char(rchar)
register uint rchar;
{
register itmp;

	if(shm->Lparity)
		rchar &= 0x7F;

	if(want_bell_notify == 2)
	{
		shmr_set_xmtr_bn_1();
		want_bell_notify = 1;
		bell_notify(XBELL_3T);
	}
	else if(rchar == 7)
	{
		bell_notify(XBELL_ATTENTION);
		return(0);
	}

	if(rchar == ESC)
	{
		rcvd_ESC();
		return(1);
	}
	else if(in_ansi_accumulation)
	{
		accumulate_ansi_sequence(rchar);
		if(is_ansi_terminator(rchar))
			process_ansi_sequence();
		return(1);
	}

#ifdef TANDEM_ENQ_ACK
	if(rchar == 5)		/* ENQ? */
	{
		lputc(6);		/* ACK */
		return(0);
	}
#endif

	if(rchar < SPACE)
	{
		switch(rchar)
		{
			case CTL_L:
				spaces((char *)shm->screen,sizeof(shm->screen));
				shm->cursor_y = 0;
				shm->cursor_x = 0;
				break;

			case BS:
				if(shm->cursor_x)
					shm->cursor_x--;
				break;

			case NL:
				if(shm->cursor_y != tcap_LINES - 1)
					shm->cursor_y++;
				else
				{
					memcpy((char *)shm->screen,(char *)shm->screen + 80,
						sizeof(shm->screen) - 80);
					spaces(&shm->screen[shm->cursor_y][shm->cursor_x],80);
				}
				break;

			case CR:
				shm->cursor_x = 0;
				break;

			case TAB:
				itmp = 8 - (shm->cursor_x % 8);
				shm->cursor_x += itmp;
				if(shm->cursor_x >= tcap_COLS)
				{
					shm->cursor_x = 0;
					if(++shm->cursor_y > tcap_LINES - 1)
						shm->cursor_y = tcap_LINES - 1;
				}
				else
					spaces(&shm->screen[shm->cursor_y][shm->cursor_x],itmp);
				break;
		}
	}
	else
	{
		shm->screen[shm->cursor_y][shm->cursor_x++] = rchar;
		if(shm->cursor_x > tcap_COLS - 1)
		{
			shm->cursor_x = 0;
			if(shm->cursor_y != tcap_LINES - 1)
				shm->cursor_y++;
			else
			{
				memcpy((char *)shm->screen,(char *)shm->screen + 80,
					sizeof(shm->screen) - 80);
				spaces(&shm->screen[shm->cursor_y][shm->cursor_x],80);
			}
		}
	}
#ifdef ANSI_DEBUG
	if(wfp)
		fprintf(wfp,"prc norm: %02x y,x=%d,%d\n",rchar,
			shm->cursor_y,shm->cursor_x);
#endif

	if(rcvr_log && rcvr_log_fp)
	{
		/* if raw mode or character not excluded from "cooked" logging */
		if(rcvr_log_raw ||
			((rchar >= SPACE) && (rchar <= '~')) ||
			 (rchar == NL) || (rchar == TAB))
		{
			putc(rchar,rcvr_log_fp);
		}
		/* back if log file if not raw and char is backspace */
		else if(!rcvr_log_raw && (rchar == 0x08))
		{
		long logpos = 0;
			if(logpos = ftell(rcvr_log_fp))
				fseek(rcvr_log_fp,logpos - 1,0);
		}

		if(rcvr_log_flusheach)
			fflush(rcvr_log_fp);
	}
	return(0);

}	/* end of process_rcvd_char */

/*+-----------------------------------------------------------------------
	rcvr() -- copy characters from remote line to so (fd == 1)
------------------------------------------------------------------------*/
void
rcvr()
{
register itmp;
uchar rchar;
uchar nlchar = NL;
long lseek();

#ifdef ANSI_DEBUG
	wfp = fopen("/tmp/ansi","a");
	fprintf(wfp,"***************\n");
#endif

	lgetc_count = 0;
	in_ansi_accumulation = 0;
	ansi = ansibuf;
	ansilen = 0;

	rcvr_signals();
	rcvr_log_open();

	saved_cursor_y = shm->cursor_y;
	saved_cursor_x = shm->cursor_x;

	while(1)		/* receive loop -- keep tight as possible! */
	{
		rchar = lgetc_rcvr();

		if(process_rcvd_char(rchar))
			continue;

		write(TTYERR,&rchar,1);

		if(shm->Ladd_nl_incoming && (rchar == CR))
			write(TTYERR,&nlchar,1);

	}
}	/* end of rcvr */

/* end of ecurcvr.c */
/* vi: set tabstop=4 shiftwidth=4: */
