/* CHK=0x2B53 */
/*+-----------------------------------------------------------------------
	ecusighdl.c - xmtr/rcvr individual process signal handlers
	wht%n4hgf@gatech.edu

  Defined functions:
	child_signals()
	hangup(sig)
	kill_rcvr_process(sig)
	rcvr_SIGHUP_handler()
	rcvr_SIGINT_handler()
	rcvr_SIGTERM_handler()
	rcvr_SIGUSR1_handler()
	rcvr_SIGUSR2_handler()
	rcvr_common_signal_handler()
	rcvr_signals()
	start_rcvr_process(notify_flag)
	xmtr_SIGHUP_handler(sig)
	xmtr_SIGINT_handler()
	xmtr_SIGTERM_handler(sig)
	xmtr_SIGUSR2_handler()
	xmtr_bsd4_SIGCHLD_handler()
	xmtr_signals()
	xmtr_SIGCLD_handler()

------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-10-1990-17:34-wht@n4hgf-provisions for ecu xmtr logging when rcvr dead */
/*:03-27-1990-15:18-wht@n4hgf-overhaul for/wait/SIGCLD handling */
/*:03-26-1990-15:07-wht@n4hgf-print lopen_err_str on hangup if not null */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-25-1990-14:10-wht@n4hgf-document ungetty_return_line stuff a bit */
/*:02-15-1990-15:58-wht-better hangup() */
/*:11-19-1989-16:59-wht-if not hangup code 0, position cursor and eeod */
/*:11-04-1989-04:24-wht-when kill rcvr other than SIGUSR2, make sure it dies */
/*:10-14-1989-21:10-wht-allow user to change ecu color choices */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecufork.h"

extern int errno;
extern int rcvr_pid;
extern int xmtr_pid;
extern int windows_active;
extern int current_ttymode;
extern char lopen_err_str[];
extern int rcvr_log;
extern FILE *rcvr_log_fp;
extern char rcvr_log_file[];	/* if rcvr_log!= 0,log filename */

int interrupt = 0;			/* interrupt indicator */
int proc_interrupt = 0;		/* procedure interrupt indicator */
int last_child_wait_status = 0;
int last_child_wait_pid = 0;

void rcvr();
void hangup();
void xmtr_signals();

void rcvr_common_signal_handler();
void rcvr_SIGTERM_handler();
void rcvr_SIGINT_handler();
void rcvr_SIGUSR1_handler();
void rcvr_SIGUSR2_handler();
void rcvr_SIGHUP_handler();
void xmtr_SIGINT_handler();
void xmtr_SIGHUP_handler();
void xmtr_SIGTERM_handler();
void xmtr_SIGCLD_handler();

/*+-----------------------------------------------------------------------
	start_rcvr_process(notify_flag)
------------------------------------------------------------------------*/
start_rcvr_process(notify_flag)
int notify_flag;
{
char s40[40];
extern ulong colors_current;
ulong colors_at_entry = colors_current;

	fflush(so);
	fflush(se);

	if(rcvr_pid > 0)			/* if process already active,just ... */
		return(rcvr_pid);		/* ... return the process id */

	if(rcvr_log && rcvr_log_file[0] && rcvr_log_fp)
	{
		fclose(rcvr_log_fp);
		rcvr_log_fp = NULL;
	}

	rcvr_pid = fork();
	if(rcvr_pid == 0)		/* if we are the (spawned) rcvr process */
	{
		if(notify_flag)
		{
			setcolor(colors_notify);
			fputs("[interactive mode]",se);
			setcolor(colors_at_entry);
			fputs("\r\n",se);
		}

		rcvr();  			    /* run until killed */
		/*NOTREACHED*/
	}
	else if(rcvr_pid > 0) 	/* we are the father (xmtr) process */
	{
#if defined(FORK_DEBUG)
		sprintf(s40,"DEBUG rcvr pid %d",rcvr_pid);
		ecu_log_event(s40);		/* rcvr */
#endif
		xmtr_signals();
		return(rcvr_pid);
	}
	else
	{
		rcvr_pid = -1;		/* no receiver active */
		ff(se,"\r\n\ncould not fork for receive\r\n");
		hangup(HANGUP_NO_FORK_FOR_RCVR);
		/*NOTREACHED*/
	}
}	/* end of start_rcvr_process */

/*+-----------------------------------------------------------------------
	kill_rcvr_process(sig) -- kill rcvr process with signal 'sig'
------------------------------------------------------------------------*/
void
kill_rcvr_process(sig)
int sig;
{
int wait_count = 70;

	if(rcvr_pid > 0)		/* if we have forked a rcvr process */
	{
		rcvr_log_fp = NULL;
		xmtr_signals();
		kill(rcvr_pid,sig);
		if(sig != SIGUSR2)	/* rcvr does not die on SIGUSR2 */
		{
			errno = 0;
			while(wait_count)
			{
				if(kill(rcvr_pid,0) && (errno == ESRCH))
					break;
				errno = 0;
				nap((long)40);
				wait_count--;
			}
			if(!wait_count)
			{
				while(!kill(rcvr_pid,SIGKILL))
				{
					wait((int *)0);
					nap((long)20);
				}
			}
			rcvr_pid = -1;
			if(rcvr_log && rcvr_log_file[0])
				rcvr_log_fp = fopen(rcvr_log_file,"a");
		}
	}

}	/* end of kill_rcvr_process */

/*+-------------------------------------------------------------------------
	hangup_code_text(code)
--------------------------------------------------------------------------*/
char *
hangup_code_text(code)
int code;
{
static char errant[16];
char *signal_name_text();

	if((code >= HANGUP_SIG1) && (code <= HANGUP_SIGN))
		return(signal_name_text(code));

	switch(code)
	{
		case HANGUP_LINE_READ_ERROR: return("line read error");
		case HANGUP_XMTR_WRITE_ERROR: return("line write error");
		case HANGUP_XMTR_LOGIC_ERROR: return("LOGIC ERROR");
		case HANGUP_BSD4_IOCTL: return("BSD4 ioctl error");
		case HANGUP_SHM_ABL: return("SHM ABL error");
		case HANGUP_SHM_RTL: return("SHM RTL error");
		case HANGUP_NO_FORK_FOR_RCVR: return("can't fork for RCVR");
		case HANGUP_TTYIN_READ_ERROR: return("keyboard read error");
		case HANGUP_LINE_OPEN_ERROR: return("line open error");
		case HANGUP_PWENT_ERROR: return("password entry error");
		case HANGUP_USAGE: return("usage");
		case HANGUP_INIT_PROC_ERROR: return("error during initial procedure");
		default:
			sprintf(errant,"code %u?",code);
			return(errant);
	}

}	/* end of hangup_code_text */

/*+-----------------------------------------------------------------------
	hangup(sig) -- terminate program (with comm line cleanup)

  close comm line
  return any ungetty'd line
  return user's console to normal status
  unlink shm name
  terminate program
  see ecuhangup.h
------------------------------------------------------------------------*/
void
hangup(sig)
int sig;
{
int isig;
int save_errno = errno;
extern char initial_procedure[];
extern int tcap_LINES;

	if(xmtr_pid == getpid())		/* if we are xmtr */
	{
		for(isig = 1; isig < NSIG; isig++)
			signal(isig,SIG_IGN);
		kill_rcvr_process(SIGUSR1);
		if(shm)
		{
			if(shm->Lmodem_off_hook)
				hayes_ATH();
			if(shm->Liofd != -1)
				lclose();			/* close line */
		}

		/* make SURE we release any line acquired from getty */
		ungetty_return_line();	/* lclose() calls this via unlock_tty(),
								 * but ok to make sure
								 */

		if(windows_active)
			windows_end_signal();
		curmove(tcap_LINES - 1,0);
		eeod();
		ttymode(0);			/* normal tty status */
		shm_done();
		if(sig > NSIG)
		{
		char s64[64];
			setcolor(colors_error);
			if(sig == HANGUP_INIT_PROC_ERROR)
				pprintf("initial procedure '%s' failed\n",initial_procedure);
			else if((sig > HANGUP_INIT_PROC_ERROR) &&
					(sig <= HANGUP_INIT_PROC_ERROR + 32))
			{
				pprintf("procedure command: exit %d\n",
					sig - HANGUP_INIT_PROC_ERROR);
			}
			else
			{
				sprintf(s64,"## XMTR %s, errno = %d",hangup_code_text(sig),
					save_errno);
				pputs(s64);
				pputs("\n");
				if(lopen_err_str[0])
				{
					pputs(lopen_err_str);
					pputs("\n");
				}
				ecu_log_event(s64);
				errno = save_errno;
				if(errno)
					pperror("errno may not apply, but");
			}
		}
		setcolor(colors_normal);
	}
	else							/* we are rcvr */
	{
		if(sig > NSIG)
		{
		char s64[64];
			sprintf(s64,"## RCVR %s, errno = %d",hangup_code_text(sig),
				save_errno);
			setcolor(colors_error);
			pputs(s64);
			pputs("\n");
			ecu_log_event(s64);
			errno = save_errno;
			if(errno)
				pperror("errno may not apply, but");
		}
		setcolor(colors_normal);
		kill(xmtr_pid,SIGHUP);
	}
	exit(sig);
	/*NOTREACHED*/

}	/* end of hangup */

void
rcvr_common_signal_handler()
{
extern int rcvr_log;
extern int rcvr_log_raw;
extern FILE *rcvr_log_fp;

	if(rcvr_log)
	{
		if(!rcvr_log_raw)
			fputs("\n",rcvr_log_fp);
		fclose(rcvr_log_fp);
	}

	exit(0);
}
void
rcvr_SIGTERM_handler()
{
	rcvr_common_signal_handler();
}
void
rcvr_SIGINT_handler()
{
	signal(SIGINT,rcvr_SIGINT_handler);
}
void
rcvr_SIGUSR1_handler()
{
	rcvr_common_signal_handler();
}
void
rcvr_SIGUSR2_handler()
{
	signal(SIGUSR2,rcvr_SIGUSR2_handler);
	shmr_process_rcvr_SIGUSR2();
}
void
rcvr_SIGHUP_handler()
{
	rcvr_common_signal_handler();
}
/*+-------------------------------------------------------------------------
	xmtr_SIGINT_handler()
--------------------------------------------------------------------------*/
void
xmtr_SIGINT_handler()
{
	signal(SIGINT,xmtr_SIGINT_handler);
	interrupt = 1;
	proc_interrupt = 1;
}	/* end of xmtr_SIGINT_handler */

void
xmtr_SIGHUP_handler(sig)
int sig;
{
	hangup(sig);
}
void
xmtr_SIGTERM_handler(sig)
int sig;
{
	hangup(sig);
}

void
xmtr_SIGUSR2_handler()
{
	signal(SIGUSR2,xmtr_SIGUSR2_handler);
	shmx_process_xmtr_SIGUSR2();
}

/*+-------------------------------------------------------------------------
	xmtr_SIGCLD_handler()
--------------------------------------------------------------------------*/
void
xmtr_SIGCLD_handler()
{

WAIT:
	errno = 0;
	if((last_child_wait_pid = wait(&last_child_wait_status)) < 0)
	{
		if(errno == EINTR)
			goto WAIT;
	}

#if defined(FORK_DEBUG)
	sprintf(s40,"DEBUG fork SIGCLD pid %d term %x",
		last_child_wait_pid,last_child_wait_status);
	ecu_log_event(s40);		/* xmtr_SIGCLD_handler() */
#endif
	signal(SIGCLD,xmtr_SIGCLD_handler);

}	/* end of xmtr_SIGCLD_handler */

/*+-------------------------------------------------------------------------
	child_signals() - signal() calls for children processes
--------------------------------------------------------------------------*/
void
child_signals()
{
	signal(SIGHUP,SIG_DFL);
	signal(SIGINT,SIG_DFL);
	signal(SIGTERM,SIG_DFL);
	signal(SIGQUIT,SIG_DFL);
	signal(SIGUSR1,SIG_DFL);
	signal(SIGUSR2,SIG_DFL);
	signal(SIGCLD,SIG_DFL);
}	/* end of child_signals */

/*+-------------------------------------------------------------------------
	xmtr_signals()
--------------------------------------------------------------------------*/
void
xmtr_signals()
{
	signal(SIGHUP,xmtr_SIGHUP_handler);
	signal(SIGQUIT,SIG_DFL);
	signal(SIGINT,xmtr_SIGINT_handler);
	signal(SIGTERM,xmtr_SIGTERM_handler);
	signal(SIGCLD,xmtr_SIGCLD_handler);
	signal(SIGUSR2,xmtr_SIGUSR2_handler);
}	/* end of xmtr_signals */

/*+-------------------------------------------------------------------------
	rcvr_signals()
--------------------------------------------------------------------------*/
void
rcvr_signals()
{
	signal(SIGHUP,rcvr_SIGHUP_handler);
	signal(SIGQUIT,SIG_DFL);
	signal(SIGTERM,rcvr_SIGTERM_handler);
	signal(SIGINT,rcvr_SIGINT_handler);
	signal(SIGUSR1,rcvr_SIGUSR1_handler);
	signal(SIGUSR2,rcvr_SIGUSR2_handler);
}	/* end of rcvr_signals */

/* vi: set tabstop=4 shiftwidth=4: */
