/* CHK=0x0D6C */
/*+-------------------------------------------------------------------------
	ecutime.c -- ecu time-related functions
	wht%n4hgf@gatech.edu

  Defined functions:
	epoch_secs_to_str(epoch_secs,type,buf)
	get_elapsed_time(elapsed_seconds)
	get_tod(type,buf)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:23-wht@n4hgf---- 2.80 --- */
/*:04-05-1990-14:10-wht@n4hgf-add zflag to get_day */
/*:04-05-1990-13:53-wht@n4hgf-add epoch_secs_to_str types 6 7 and 8 */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include <sys/types.h>
#include <time.h>
#include <sys/timeb.h>

struct tm *gmtime();
struct tm *localtime();

/*+-------------------------------------------------------------------------
	get_month(zflag) - month 1-12 - zflag true for UTC (Z)), else local time
--------------------------------------------------------------------------*/
int
get_month(zflag)
int zflag;
{
long time();
long epoch_secs = time((long *)0);
struct tm *tod = (zflag) ? gmtime(&epoch_secs) : localtime(&epoch_secs);
	return(tod->tm_mon + 1);
}	/* end of get_month */

/*+-------------------------------------------------------------------------
	get_day(zflag) - day 0-6 - zflag true for UTC (Z)), else local time
--------------------------------------------------------------------------*/
int
get_day(zflag)
int zflag;
{
long time();
long epoch_secs = time((long *)0);
struct tm *tod = (zflag) ? gmtime(&epoch_secs) : localtime(&epoch_secs);
	return(tod->tm_wday);
}	/* end of get_day */

/*+-----------------------------------------------------------------------
	char *epoch_secs_to_str(epoch_secs,type,buf)

  time of day types:
	0		hh:mm
	1		hh:mm:ss
	2		mm-dd-yyyy hh:mm
	3		mm-dd-yyyy hh:mm:ss
	4		mm-dd-yyyy hh:mm:ss (UTC hh:mm)
	5		mm-dd-yyyy
	6		hh:mmZ
	7		hh:mm:ssZ
	8		mm-dd-yyyy (UTC date)

	returns 'buf' address

------------------------------------------------------------------------*/
char *
epoch_secs_to_str(epoch_secs,type,buf)
long epoch_secs;
int type;
char *buf;
{
struct tm *tod;


	if(type < 6)
		tod = localtime(&epoch_secs);
	else
		tod = gmtime(&epoch_secs);

	switch(type)
	{
		default:
		case 6:
		case 0:
			sprintf(buf,"%02d:%02d",tod->tm_hour,tod->tm_min);
			if(type == 6)
				strcat(buf,"Z");
			break;

		case 7:
		case 1:
			sprintf(buf,"%02d:%02d:%02d",tod->tm_hour,tod->tm_min,tod->tm_sec);
			if(type == 7)
				strcat(buf,"Z");
			break;

		case 2:
			sprintf(buf,"%02d-%02d-%04d %02d:%02d",
				tod->tm_mon + 1,tod->tm_mday,tod->tm_year + 1900,
				tod->tm_hour,tod->tm_min);
			break;

		case 3:
			sprintf(buf,"%02d-%02d-%04d %02d:%02d:%02d",
				tod->tm_mon + 1,tod->tm_mday,tod->tm_year + 1900,
				tod->tm_hour,tod->tm_min,tod->tm_sec);
			break;

		case 4:
			sprintf(buf,"%02d-%02d-%04d %02d:%02d:%02d",
				tod->tm_mon + 1,tod->tm_mday,tod->tm_year + 1900,
				tod->tm_hour,tod->tm_min,tod->tm_sec);
			tod = gmtime(&epoch_secs);
			sprintf(&buf[strlen(buf) ]," (UTC %02d:%02d)",
				tod->tm_hour,tod->tm_min);
			break;

		case 8:
		case 5:
			sprintf(buf,"%02d-%02d-%04d",
				tod->tm_mon + 1,tod->tm_mday,tod->tm_year + 1900);
			break;

	}

	return(buf);
}	/* end of epoch_secs_to_str */

/*+-----------------------------------------------------------------------
	char *get_tod(type,buf)
returns 'buf' address
------------------------------------------------------------------------*/
char *
get_tod(type,buf)
int type;
char *buf;
{
long time();
	return(epoch_secs_to_str(time((long *)0),type,buf));
}	/* end of get_tod */

/*+-----------------------------------------------------------------------
	char *get_elapsed_time(elapsed_seconds)
	"hh:mm:ss" returned
  static string address is returned
------------------------------------------------------------------------*/
char *
get_elapsed_time(elapsed_seconds)
long elapsed_seconds;
{
static char elapsed_time_str[40];
long hh,mm,ss;

	hh = elapsed_seconds / 3600;
	elapsed_seconds -= hh * 3600;
	mm = elapsed_seconds / 60L;
	elapsed_seconds -= mm * 60L;
	ss = elapsed_seconds;

	sprintf(elapsed_time_str,"%02ld:%02ld:%02ld",hh,mm,ss);
	return(elapsed_time_str);
}	/* end of get_elapsed_time */

/* end of ecutime.c */
/* vi: set tabstop=4 shiftwidth=4: */
