/* CHK=0xA3B1 */
/*+-----------------------------------------------------------------
	ecuxenix.c -- very machine/OS dependent functions
	wht%n4hgf@gatech.edu

  Defined functions:
	morse_bell(xbell_type,count)
	rename(from,to)
	send_bell_fd(fd,count,nap_msec)
	send_get_response(token,narg,arg)
	set_bell_fd(fd,pitch,duration)
	set_default_escape_prompt()
	show_escape_prompt()
	signal_name_text(sig)
	xbell(type,count)
	xbell_fd(fd,type,count)
	bell_alarm(xbell_type)
	bell_notify(xbell_type)
	kbd_escape(xkey)

  This module is a grab bag for historical reasons.  Needs reorg.
------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-14:56-wht@n4hgf-my GCC is uses XENIX libs/turn on rename for it */
/*:04-19-1990-03:07-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-17-1990-14:22-wht@n4hgf-add icmd_history_manager call */
/*:03-27-1990-15:18-wht@n4hgf-overhaul for/wait/SIGCLD handling */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-23-1990-14:22-wht@n4hgf-add KACT_REDISPLAY processing */
/*:02-15-1990-16:02-wht-another use for signal name text - slight change */
/*:12-02-1989-17:11-wht-add UNIX-specific signal names to signal_name_text */
/*:11-23-1989-02:48-wht-UNIX has a rename system call */
/*:11-04-1989-03:23-wht-alt-a through alt-z automatic procedure execution */
/*:10-15-1989-02:52-wht-move MORSE definition to Makefile */
/*:10-14-1989-21:10-wht-allow user to change ecu color choices */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecukey.h"
#include "ecufkey.h"
#include "ecuxkey.h"
#include "ecufork.h"
#include "esd.h"

#if defined(MORSE)
/* the space between # and include defeats make depend from seeing this */
# include <local/morse_dvr.h>
#endif

char *get_ttyname();

extern char curr_dir[]; /* current working key defns */
extern KDE keyset_table[];
extern char keyset_name[];
extern int want_bell_notify;	/* want bell notify */
extern int rcvr_pid;
extern int interrupt;

ESD *icmd_prompt;		/* interactive command prompt */
int icmd_prompt_len;

/*+-------------------------------------------------------------------------
	show_escape_prompt()
returns number of character positions written to screen
--------------------------------------------------------------------------*/
show_escape_prompt()
{
char prompt_last;

	prompt_last = 'd';	/* dummy */
	if(icmd_prompt->cb)
		prompt_last = *(icmd_prompt->pb + icmd_prompt->cb - 1);

	icmd_prompt_len = 0;
	shmx_make_rcvr_sleep(1);
	if(!(colors_current & 0xFFFF0000L) || !(colors_current & 0xFFFFL))
		setcolor(colors_normal);
	stand_out();
	if(icmd_prompt->cb)
	{
		ff(se," %s",icmd_prompt->pb);
		icmd_prompt_len += icmd_prompt->cb + 1;
	}
	if(isalnum(prompt_last))
	{
		fputs(" %",se);
		icmd_prompt_len += 2;
	}
	stand_end();
	fputc(' ',se);
	icmd_prompt_len++;
	fflush(so);
	return(icmd_prompt_len);

}	/* end of show_escape_prompt */

/*+-------------------------------------------------------------------------
	set_default_escape_prompt()
--------------------------------------------------------------------------*/
void
set_default_escape_prompt()
{
register char *cptr;
char *getenv();

	if((cptr = getenv("ECUPROMPT")) != (char *)0)
	{
		strncpy(icmd_prompt->pb,cptr,icmd_prompt->maxcb);
		*(icmd_prompt->pb + icmd_prompt->maxcb - 1) = 0;
		icmd_prompt->cb = strlen(icmd_prompt->pb);
		null_terminate_esd(icmd_prompt);
	}

}	/* end of set_default_escape_prompt */

/*+-------------------------------------------------------------------------
	kbd_escape() -- xmtr got extended key -- process it
returns(1)
--------------------------------------------------------------------------*/
kbd_escape(xkey)
uchar xkey;
{
register itmp;
register backspace_count;
register KDE *tkde;
uchar icmd_buf[128];

	if((xkey >= XF_ALTA) && (xkey <= XF_ALTZ))
	{
	char alt_key_proc_name[8];
	char *arg = alt_key_proc_name;
	int narg = 1;
	int rcvr_alive = (rcvr_pid > 0);
		sprintf(alt_key_proc_name,"alt_%c",xkey - XF_ALTA + 'a');
		if(rcvr_alive)
			kill_rcvr_process(SIGUSR1);
		ttymode(2);
		ttyflush(0);
		if(do_proc(narg,&arg))
			ring_bell();
		proc_file_reset();
		ttymode(1);
		if(rcvr_alive)
			start_rcvr_process(0);
		return(1);
	}

	switch(xkey)
	{
		case XFhome:	 /* home key pressed ? */
			backspace_count = show_escape_prompt();
GET_ICMD:
			ttygets(icmd_buf,sizeof(icmd_buf),2);
			switch(icmd_buf[0])
			{
			case XFhome:
			case XFend:
			case XFpgup:
			case XFpgdn:
			case XFcurup:
			case XFcurdn:
				if(icmd_history_manager(icmd_buf[0],icmd_buf,sizeof(icmd_buf)))
					goto GET_ICMD;
			}
			if(!icmd_buf[0] || (icmd_buf[0] == ESC))
			{
				while(backspace_count--)
					ff(se,"\b \b");
				break;
			}
			if(icmd(icmd_buf))
				hangup(HANGUP_OK);
			break;

		default:
			if((itmp = xf_to_keyid(xkey)) < 0)
			{
				ring_bell();
				break;
			}
			tkde = &keyset_table[itmp];
			if((itmp = tkde->outcount) > 0)
			{
			register char *cptr = tkde->outstr;
				while(itmp--)
					lputc(*cptr++);
			}
			else if(tkde->outcount < 0)
			{
				switch(tkde->outcount)
				{
					case KACT_LOCAL_SHELL:
						fputs("\r\n",se);
						stand_out();
						ff(se," local shell in %s ",curr_dir);
						stand_end();
						ff(se,"\r\n");
						shell("!");
						break;
					case KACT_REDISPLAY:
						redisplay_rcvr_screen();
				}
			}
			else
				ring_bell();
			break;
	}
	return(1);
}	/* end of kbd_escape */

/*+-------------------------------------------------------------------------
	set_bell_fd(fd,pitch,duration)
Example: 1B 5B 3D 34 30 30 30 3B 31 42 | .[=4000;1B
--------------------------------------------------------------------------*/
void
set_bell_fd(fd,pitch,duration)
int fd;
int pitch;
int duration;
{
char bell_cmd[32];
	sprintf(bell_cmd,"\033[=%d;%dB",pitch,duration);
	write(fd,bell_cmd,strlen(bell_cmd));
}	/* end of set_bell_fd */

/*+-------------------------------------------------------------------------
	send_bell_fd(fd,count,nap_msec)
--------------------------------------------------------------------------*/
void
send_bell_fd(fd,count,nap_msec)
register fd;
register count;
register nap_msec;
{
static char bellch = 0x07;

	if(count)
		while(count--)
		{
			write(fd,&bellch,1);	
			nap((long)nap_msec);
		}
}	/* end of send_bell_fd */

/*+-------------------------------------------------------------------------
	xbell_fd(fd,type,count)
--------------------------------------------------------------------------*/
void
xbell_fd(fd,type,count)
register fd;
register type;
register count;
{
register itmp;

	if(count)
		while(count--)
		{
			switch(type)
			{
				case XBELL_DONE:		/* octaves */
					set_bell_fd(fd,1000,1);
					send_bell_fd(fd,1,100);
					set_bell_fd(fd,2000,1);
					send_bell_fd(fd,1,100);
					set_bell_fd(fd,4000,1);
					send_bell_fd(fd,1,100);
					break;
				case XBELL_ATTENTION:		/* morse .-.-.- ATTENTION */
					for(itmp = 0; itmp < 3; itmp++)
					{
						set_bell_fd(fd,2000,1);
						send_bell_fd(fd,1,140);
						set_bell_fd(fd,2000,3);
						send_bell_fd(fd,1,340);
					}
					break;
				case XBELL_C:		/* morse -.-. C */
					for(itmp = 0; itmp < 2; itmp++)
					{
						set_bell_fd(fd,2000,3);
						send_bell_fd(fd,1,320);
						set_bell_fd(fd,2000,1);
						send_bell_fd(fd,1,120);
					}
					break;
				case XBELL_3T:		/* 3 morse T's */
					set_bell_fd(fd,2000,3);
					send_bell_fd(fd,3,460);
					break;
				default:
					set_bell_fd(fd,2000,4);
					send_bell_fd(fd,1,100);
					break;
			}
		}
				
	set_bell_fd(fd,4000,1);
	
}	/* end of xbell_fd */

/*+-------------------------------------------------------------------------
	morse_bell(xbell_type,count)
--------------------------------------------------------------------------*/
#if defined(MORSE)
morse_bell(xbell_type,count)
int xbell_type;
int count;
{
register morse_fd;
int morse_frequency = 600;
char morse_char;
static int morse_ticks = 0;
extern int errno;

	if(!morse_ticks)
	{
	char *hzstr;
	char *getenv();
	int hz;
		if(!(hzstr = getenv("HZ")))
			hz = 50;
		else
			hz = atoi(hzstr);
		morse_ticks = hz / 25;
	}

	if((morse_fd = open("/dev/morse",O_WRONLY,0)) < 0)
		return(-1);

	ioctl(morse_fd,MORSE_SET_SPEED,&morse_ticks);
	ioctl(morse_fd,MORSE_SET_FREQUENCY,&morse_frequency);
	switch(xbell_type)
	{
		case XBELL_DONE:
			morse_char = 'd';
/*
			morse_frequency = 400;
			ioctl(morse_fd,MORSE_SET_FREQUENCY,&morse_frequency);
*/
			break;
		case XBELL_ATTENTION:
			morse_char = '.';
			break;
		case XBELL_C:
			morse_char = 'c';
			break;
		case XBELL_3T:
			morse_char = 'o';
			break;
		default:
			morse_char = BT;
			break;
	}
	while(count--)
		write(morse_fd,&morse_char,1);
	close(morse_fd);
	return(0);
}	/* end of morse_bell */
#endif

/*+-------------------------------------------------------------------------
	xbell(type,count)
--------------------------------------------------------------------------*/
void
xbell(type,count)
int type;
int count;
{
#if defined(MORSE)
	if(morse_bell(type,count))
#endif
		xbell_fd(1,type,count);
}	/* end of xbell */

/*+-------------------------------------------------------------------------
	bell_alarm(xbell_type)
  Under XENIX 2.2.1, ring bell on multiscreens;
  if morse driver included, use it
--------------------------------------------------------------------------*/
int
bell_alarm(xbell_type)
int xbell_type;
{
register notify_fd;
register fork_pid;
static long notify_time = 0L;
char *ttname;
char devname[64];
int devnum;
int ttnum;
long time(long *);

/* if ECU from modem port or console in single-user mode, dont do it */
/* 0123456789 */
/* /dev/tty10  */
/* /dev/tty1a  */
	ttname = get_ttyname();
	if(!isdigit(ttname[strlen(ttname) - 1]))
		return(0);
	ttnum = atoi(ttname + 8);

/* if happened less than 15 secs ago, forget it */
	if((time((long *)0) - notify_time) < 15L)
		return(0);

	notify_time = time((long *)0);

#if defined(MORSE)
	if(morse_bell(xbell_type,1))
	{
#endif
		if((fork_pid = fork()) != 0)
		{
#if defined(FORK_DEBUG)
			sprintf(devname,"DEBUG bell notify pid %d",fork_pid);
			ecu_log_event(devname);	/* bell notify */
#endif
			return(fork_pid > 0);
		}

		for(devnum = 1; devnum < 13; devnum++)
		{
			if(devnum == ttnum)		/* don't bell notify ecu's tty */
				continue;
			sprintf(devname,"/dev/tty%02d",devnum);
			if((notify_fd = open(devname,O_WRONLY,0)) >= 0)
			{
				xbell_fd(notify_fd,xbell_type,1);
				close(notify_fd);
			}
		}

		_exit(0);		/* end of child tine (of the fork, naturally) */
#if defined(MORSE)
	}
#endif
}	/* end of bell_alarm */

/*+-------------------------------------------------------------------------
	bell_notify(xbell_type)
--------------------------------------------------------------------------*/
void
bell_notify(xbell_type)
int xbell_type;
{

	if(want_bell_notify)
		bell_alarm(xbell_type);

}	/* end of bell_notify */

/*+-------------------------------------------------------------------------
	signal_name_text(sig)
--------------------------------------------------------------------------*/
char *
signal_name_text(sig)
int sig;
{
register char *cptr;
static char sigunknown[20];

	sig &= 0x7F;
	switch(sig)
	{
		case SIGHUP:	cptr = "SIGHUP"; break;
		case SIGINT:	cptr = "SIGINT"; break;
		case SIGQUIT:	cptr = "SIGQUIT"; break;
		case SIGILL:	cptr = "SIGILL"; break;
		case SIGTRAP:	cptr = "SIGTRAP"; break;
		case SIGIOT:	cptr = "SIGIOT"; break;
		case SIGEMT:	cptr = "SIGEMT"; break;
		case SIGFPE:	cptr = "SIGFPE"; break;
		case SIGKILL:	cptr = "SIGKILL"; break;
		case SIGBUS:	cptr = "SIGBUS"; break;
		case SIGSEGV:	cptr = "SIGSEGV"; break;
		case SIGSYS:	cptr = "SIGSYS"; break;
		case SIGPIPE:	cptr = "SIGPIPE"; break;
		case SIGALRM:	cptr = "SIGALRM"; break;
		case SIGTERM:	cptr = "SIGTERM"; break;
		case SIGUSR1:	cptr = "SIGUSR1"; break;
		case SIGUSR2:	cptr = "SIGUSR2"; break;
		case SIGCLD:	cptr = "SIGCLD"; break;
		case SIGPWR:	cptr = "SIGPWR"; break;
#if	defined(M_UNIX)
		case SIGSTOP:   cptr = "SIGSTOP"; break;
		case SIGTSTP:   cptr = "SIGTSTP"; break;
		case SIGCONT:   cptr = "SIGCONT"; break;
		case SIGTTIN:   cptr = "SIGTTIN"; break;
		case SIGTTOU:   cptr = "SIGTTOU"; break;
#endif
		default:
			sprintf(sigunknown,"SIGNAL %u",sig);
			return(sigunknown);
	}
	return(cptr);

}	/* end of signal_name_text */

/*+-------------------------------------------------------------------------
	rename(from,to)
--------------------------------------------------------------------------*/
#if !defined(M_UNIX) || defined(__GNUC__)
rename(from,to)
char *from;
char *to;
{
struct stat ss;
extern int errno;

	if(stat(to,&ss) == 0)	/* if to exists, flunk */
	{
		errno = EEXIST;	/* fake "file exists" error */
		return(-1);
	}
	if(link(from,to))	/* if cannot link, flunk */
		return(-1);
	if(unlink(from))	/* if cannot unlink, flunk */
		return(-1);
	return(0);
}	/* end of rename */
#endif

/*+-------------------------------------------------------------------------
	send_get_response(token,narg,arg)
--------------------------------------------------------------------------*/
void
send_get_response(token,narg,arg)
int token;
int narg;
char **arg;
{
register itmp;
register char *cptr;
int iarg;
int mode = atoi(arg[1]);
LRWT lr;
char buf[1024];
FILE *fplog;
char *fname = "ecu.sgr.log";
int rcvr_alive = (rcvr_pid > 0);

	if(narg < 3)
	{
		fputs("   usage: sgr mode cmd\r\n",se);
		return;
	}
	if((fplog = fopen(fname,"a")) == NULL)
	{
		fputs("\r\n",se);
		perror(fname);
		fputs("\r\n",se);
		return;
	}

	mode &= 0x0F;
	if(rcvr_alive)
		kill_rcvr_process(SIGUSR1);

	lr.to1 = 10 * 1000L;
	lr.to2 = 2 * 1000L;
	lr.raw_flag = 0x80 + mode; /* allow interrupts */
	lr.buffer = buf;
	lr.bufsize = sizeof(buf);
	lr.delim = (char *)0;
	lr.echo = 0;
	interrupt = 0;

	fputs("\r\n",se);

	fprintf(fplog,"\nMode: %d ",mode);
	fputs("Stimulus: ",fplog);
	for(iarg = 2; iarg < narg; iarg++)
	{
		fputs(arg[iarg],fplog);
		lputs_paced(20,arg[iarg]);
		if(iarg != (narg - 1))
		{
			fputc(' ',fplog);
			lputc_paced(20,' ');
		}
	}
	fputs("\nResponse: \n",fplog);
	lputc_paced(20,'\r');

	lgets_timeout(&lr);
	itmp = lr.count;
	cptr = strchr(buf,'\n') + 1;
	itmp -= (int)(cptr - buf);
	hex_dump_fp(fplog,cptr,itmp,(char *)0,1);

	itmp = lr.count;
	cptr = buf;
	while(itmp--)
	{
		if(*cptr == '\n')
			fputc('\r',se);
		fputc(*cptr++,se);
	}
		
	fclose(fplog);
	if(rcvr_alive)
		start_rcvr_process(0);
}	/* end of send_get_response */

/* end of ecuxenix.c */
/* vi: set tabstop=4 shiftwidth=4: */
