/* CHK=0x8412 */
/*+----------------------------------------------------------------
	esdutil.c - ecu extended string descriptor manipulation
	wht%n4hgf@gatech.edu

  Defined functions:
	append_zstr_to_esd(param,zstr)
	end_of_cmd(param)
	esdstrindex(esd1,esd2,index1_flag,index2_flag)
	fgetesd(param,fileptr)
	fputesd(param,fileptr,index_flag,nl_flag)
	free_esd(tesd)
	get_alpha_zstr(param,strbuf,strbuf_maxcb)
	get_alphanum_zstr(param,strbuf,strbuf_maxcb)
	get_cmd_char(param,pchar)
	get_numeric_value(param,value)
	get_numeric_zstr(param,strbuf,strbuf_maxcb)
	get_switches(param,switches,switches_max)
	get_word_zstr(param,strbuf,strbuf_maxcb)
	init_esd(tesd,cptr,maxcb)
	keyword_lookup(ktable,param)
	make_esd(maxcb)
	null_terminate_esd(tesd)
	show_esd(tesd,title)
	skip_cmd_break(param)
	skip_cmd_char(param,skipchar)
	skip_colon(param)
	skip_comma(param)
	skip_paren(param,fLeft)
	strindex(str1,str2)
	strip_trailing_spaces_esd(ztext)
	zero_esd(tesd)

-----------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-16-1990-05:16-wht@n4hgf-keyword_lookup now matches only whole string */
/*:11-06-1989-03:38-wht-skip_paren could return wrong error code */
/*:10-19-1989-01:42-wht-work on get_word_zstr/g */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include <ctype.h>
#include "ecu.h"
#include "ecuerror.h"
#include "esd.h"

/*+-------------------------------------------------------------------------
    null_terminate_esd(&esd)
    puts null at 'cb' position of string (standard esd always
    has one more byte in buffer than maxcb says)
--------------------------------------------------------------------------*/
void
null_terminate_esd(tesd)
register ESD *tesd;
{
	tesd->pb[tesd->cb] = 0;
}   /* end of null_terminate_esd */

/*+-----------------------------------------------------------------------
	zero_esd(tesd)  zero an esd 
------------------------------------------------------------------------*/
void
zero_esd(tesd)
register ESD *tesd;
{
	tesd->cb = 0;				/* current count == 0 */
	tesd->index = 0;			/* parse index to first position */
	tesd->old_index = 0;		/* parse index to first position */
	*tesd->pb = 0;				/* start with null terminated string */

}	/* end of zero_esd */

/*+-----------------------------------------------------------------------
	init_esd(tesd,cptr,maxcb)  init an esd 
------------------------------------------------------------------------*/
void
init_esd(tesd,cptr,maxcb)
register ESD *tesd;
char *cptr;
register maxcb;
{
	tesd->pb = cptr;			/* pointer to string */
	tesd->maxcb = maxcb;		/* max characters in buffer */
	zero_esd(tesd);

}	/* end of init_esd */

/*+-----------------------------------------------------------------------
	esdptr = make_esd(maxcb)	allocate an esd and buffer
------------------------------------------------------------------------*/
ESD *
make_esd(maxcb)
register maxcb;		/* desired maxcb */
{
register ESD *tesd;
register actual_cb;

	/* we get an extra character to ensure room for null past maxcb */
	actual_cb = maxcb + sizeof(ESD) + 1;
	if(actual_cb & 1)		/* even allocation */
		++actual_cb;

	if(!(tesd = (ESD *)malloc(actual_cb)))
		return((ESD *)0); 	/* return NULL if failure */

	init_esd(tesd,(char *)(tesd + 1),maxcb);
	return(tesd);

}	/* end of make_esd */

/*+-----------------------------------------------------------------------
	free_esd(esdptr)
------------------------------------------------------------------------*/
void
free_esd(tesd)
register ESD *tesd;
{
	tesd->maxcb = 0;
	tesd->cb = 0;
	free((char *)tesd);
}

/*+-------------------------------------------------------------------------
	show_esd(tesd,title)
--------------------------------------------------------------------------*/
void
show_esd(tesd,title)
ESD *tesd;
char *title;
{
register itmp;

	if(title && *title)
	{
		pputs(title);
		pputs("\n");
	}
	null_terminate_esd(tesd);
	pputs(tesd->pb);
	pputs("\n");
	for(itmp = 0; itmp <= tesd->cb; itmp++)
	{
		if(itmp == tesd->old_index)
			pputc('^');
		else if(itmp == tesd->index)
			pputc('^');
		else
			pputc(' ');
		if((itmp > tesd->old_index) && (itmp > tesd->index))
			break;
	}
	pprintf(" o%d i%d c%d\n",tesd->old_index,tesd->index,tesd->cb);

}	/* end of show_esd */

/*+----------------------------------------------------------------
    strindex:  string index function

  Returns position of 'str2' in 'str1' if found
  If 'str2' is null, then 0 is returned (null matches anything)
  Returns -1 if not found
-----------------------------------------------------------------*/
int
strindex(str1,str2)
char *str1;	/* the (target) string to search */
char *str2;	/* the (comparand) string to search for */
{
register istr1 = 0;
register lstr2 = strlen(str2);
register char *mstr = str1;	/* the (target) string to search */

	if(*str2 == 0)			/* null string matches anything */
		return(0);

	while(*mstr)
	{
		if(*mstr == *str2)
		{ /* we have a first char match... does rest of string match? */
			if(!strncmp(mstr,str2,lstr2))
				return(istr1);		/* if so, return match position */
		}
		mstr++;
		istr1++;
	}

	return(-1);		/* if we exhaust target string, flunk */

}   /* end of strindex */

/*+-------------------------------------------------------------------------
	esdstrindex(esd1,esd2,index1_flag,index2_flag)

  Call strindex with esd1->pb and esd2->pb.
  If index1_flag != 0, esd1->pb + esd1->index passed
  If index2_flag != 0, esd2->pb + esd2->index passed
--------------------------------------------------------------------------*/
esdstrindex(esd1,esd2,index1_flag,index2_flag)
register ESD *esd1;
register ESD *esd2;
register index1_flag;
register index2_flag;
{
	return(strindex((index1_flag) ? esd1->pb : esd1->pb + esd1->index,
	    (index2_flag) ? esd2->pb : esd2->pb + esd2->index));

}	/* end of esdstrindex */

/*+----------------------------------------------------------------
    keyword_lookup(ktable,param)

  Lookup string in keyword_table struct array
  Returns table->key_token if 'param' found in
  'table', else -1

  Beware substrings.  "type","typedef" will both match "type"
-----------------------------------------------------------------*/
keyword_lookup(ktable,param)
register KEYTAB *ktable;
register char *param;
{
/* register plen = strlen(param); */

	while(ktable->key_word)
	{
/*		if(!strncmp(ktable->key_word,param,plen)) */
		if(!strcmp(ktable->key_word,param))
			return(ktable->key_token);
		++ktable;
	}   /* end of while */

	return(-1);     /* search failed */

}   /* end of keyword_lookup */

/*+----------------------------------------------------------------
    skip_cmd_break(param)

  Finds next non-break

  'param' is an esd with valid 'index' field
  Returns  0             index field points to non-break character
           eNoParameter  end of command found
-----------------------------------------------------------------*/
int
skip_cmd_break(param)
register ESD *param;
{
register cb = param->cb;
register index = param->index;
register char *pb = param->pb;

	while(index < cb)
	{
		if(!isspace(*(pb + index)))
			break;
		index++;
	}
	param->old_index = param->index = index;
	if(index >= cb)
		return(eNoParameter);
	return(0);

}   /* end of skip_cmd_break */

/*+-------------------------------------------------------------------------
	end_of_cmd(param) - return 1 if at end of command
--------------------------------------------------------------------------*/
int
end_of_cmd(param)
register ESD *param;
{
	if(skip_cmd_break(param) || (*(param->pb + param->index) == ';') ||
			(*(param->pb + param->index) == '#'))
		return(1);
	return(0);
}	/* end of end_of_cmd */

/*+-------------------------------------------------------------------------
    erc = skip_cmd_char(param,skipchar)
--------------------------------------------------------------------------*/
int
skip_cmd_char(param,skipchar)
register ESD *param;
register char skipchar;
{
int erc;

	if(erc = skip_cmd_break(param))
		return(erc);

	if(param->pb[param->index] == skipchar)
	{
		++param->index;
		return(0);
	}

	return(eSyntaxError);

}   /* end of skip_cmd_char */

/*+-------------------------------------------------------------------------
    erc = skip_colon(param)
--------------------------------------------------------------------------*/
int
skip_colon(param)
register ESD *param;
{
	register erc;

	if(erc = skip_cmd_break(param))
		return(erc);

	if(param->pb[param->index] == ':')
	{
		++param->index;
		return(0);
	}

	return(eCommaExpected);

}   /* end of skip_colon */

/*+-------------------------------------------------------------------------
    erc = skip_comma(param)
--------------------------------------------------------------------------*/
int
skip_comma(param)
register ESD *param;
{
	register erc;

	if(erc = skip_cmd_break(param))
		return(erc);

	if(param->pb[param->index] == ',')
	{
		++param->index;
		return(0);
	}

	return(eCommaExpected);

}   /* end of skip_comma */

/*+-------------------------------------------------------------------------
    erc = skip_paren(fparam,LEFT or RIGHT)
--------------------------------------------------------------------------*/
int
skip_paren(param,fLeft)
register ESD *param;
int fLeft;
{
register erc;
register char search = (fLeft) ? '(' : ')';

	if(erc = skip_cmd_break(param))
		return(erc);

	if(param->pb[param->index] == search)
	{
		param->index++;
		return(0);
	}
	return((fLeft) ? eMissingLeftParen : eMissingRightParen);

}   /* end of skip_paren */

/*+-------------------------------------------------------------------------
	get_cmd_char(param,pchar)
--------------------------------------------------------------------------*/
int
get_cmd_char(param,pchar)
register ESD *param;
char *pchar;
{
int erc;

	if(erc = skip_cmd_break(param))
		return(erc);
	*pchar = param->pb[param->index++];
	return(0);

}	/* end of get_cmd_char */

/*+----------------------------------------------------------------
    get_alpha_zstr(&esd,&strbuf,strbuf_maxcb)

  places next alphabetic string token [A_Za_z_] into
  the null-terminated 'strbuf' string.  returns 0 or -1
  or skip_cmd_break error codes
-----------------------------------------------------------------*/
int
get_alpha_zstr(param,strbuf,strbuf_maxcb)
ESD *param;
register char *strbuf;
int strbuf_maxcb;
{
register izstr;
register schar;
register char *pb = param->pb;

	if(izstr = skip_cmd_break(param))
		return(izstr);
	izstr = 0;
	while( (izstr < strbuf_maxcb-1) && (param->index < param->cb) )
	{
		schar = pb[param->index];
		if((!isalpha(schar)) && (schar != '_'))
			break;
		strbuf[izstr++] = schar;
		param->index++;
	}

	strbuf[izstr] = 0;
	return(izstr ? 0 : eBadParameter);

}   /* end of get_alpha_zstr */

/*+----------------------------------------------------------------
    get_alphanum_zstr(&esd,&strbuf,strbuf_maxcb)

  places next alphanumeric string token [A-Za-z0-9_]
  into the null-terminated 'strbuf' string.  returns 0
  or -1 or skip_cmd_break error codes
-----------------------------------------------------------------*/
int
get_alphanum_zstr(param,strbuf,strbuf_maxcb)
register ESD *param;
register char *strbuf;
int strbuf_maxcb;
{
int izstr = 0;
int schar;
register cb = param->cb;
register index;

	if(izstr = skip_cmd_break(param))
		return(izstr);

	index = param->index;
	while( (izstr < strbuf_maxcb-1) && (index < cb))
	{
		schar = param->pb[index++];
		if(isalnum(schar) || (schar == '_'))
			strbuf[izstr++] = schar;
		else
		{
			--index;
			break;
		}
	}

	param->index = index;
	strbuf[izstr]=0;
	return(izstr ? 0 : eBadParameter);

}   /* end of get_alphanum_zstr */

/*+----------------------------------------------------------------
    get_numeric_zstr(&esd,&strbuf,strbuf_maxcb)
    gets next numeric string token places it
    into the null-terminated 'strbuf' string.  returns 0 or -1 
    or skip_cmd_break error codes
-----------------------------------------------------------------*/
int
get_numeric_zstr(param,strbuf,strbuf_maxcb)
register ESD *param;
register char *strbuf;
register strbuf_maxcb;
{
	register izstr;
	register schar;

	if(izstr = skip_cmd_break(param))
		return(izstr);

	while( (izstr < strbuf_maxcb-1) && (param->index < param->cb) )
	{
		schar = param->pb[param->index++];
		if( isdigit(schar) )
			strbuf[izstr++]=schar;
		else
		{
			--param->index;
			break;
		}
	}

	strbuf[izstr]=0;
	return(izstr ? 0 : eBadParameter);

}   /* end of get_numeric_zstr */

/*+-----------------------------------------------------------------------
	get_numeric_value(param,&long_var)
------------------------------------------------------------------------*/
get_numeric_value(param,value)
register ESD *param;
register long *value;
{
register erc;
char buf[32];

	if(erc = get_numeric_zstr(param,buf,sizeof(buf)))
		return(erc);
	sscanf(buf,"%ld",value);
	return(0);

}	/* end of get_numeric_value */

/*+----------------------------------------------------------------
    get_word_zstr(&esd,&strbuf,strbuf_maxcb)

  gets next word (continuous string of characters without spaces
  or tabs) returns 0 or -1 or skip_cmd_break error codes
-----------------------------------------------------------------*/
int
get_word_zstr(param,strbuf,strbuf_maxcb)
register ESD *param;
register char *strbuf;
register strbuf_maxcb;
{
	register izstr;
	register schar;

	if(izstr = skip_cmd_break(param))
		return(izstr);

	strbuf_maxcb--;
	while((izstr < strbuf_maxcb) && (param->index < param->cb))
	{
		schar = param->pb[param->index++];
		if( (schar > 0x20) && (schar <= 0x7e))
			strbuf[izstr++]=schar;
		else
		{
			--param->index;
			break;
		}
	}

	strbuf[izstr]=0;
	return(izstr ? 0 : eBadParameter);

}   /* end of get_word_zstr */

/*+-------------------------------------------------------------------------
    strip_trailing_spaces_esd(param)
--------------------------------------------------------------------------*/
void
strip_trailing_spaces_esd(ztext)
register ESD *ztext;
{
	while(ztext->cb && (ztext->pb[ztext->cb-1] == 0x20))
		ztext->cb--;
}   /* end of strip_trailing_spaces_esd */

/*+-------------------------------------------------------------------------
	fgetesd(&esd,fileptr)

  stdio read from FILE *fileptr into esd
  returns param->cb set up not including trailing nl, param->index == 0
--------------------------------------------------------------------------*/
int
fgetesd(param,fileptr)
register ESD *param;
register FILE *fileptr;
{
register char *cptr;

	if(fgets(param->pb,param->maxcb,fileptr) == NULL)
		return(eEOF);
	if((cptr = strchr(param->pb,0x0A)) == NULL)
		return(eBufferTooSmall);
	param->cb = (int)(cptr - param->pb);
	null_terminate_esd(param);
	param->index = 0;
	param->old_index = 0;
	return(0);

}	/* end of fgetesd */

/*+-------------------------------------------------------------------------
	fputesd(&esd,fileptr,index_flag,nl_flag)

  write esd contents to stdio FILE *fileptr
  if index_flag is true, write from param->index thru end of esd
  otherwise, from start of esd
  if nl_flag is true, append nl to write, else just esd contents
--------------------------------------------------------------------------*/
int
fputesd(param,fileptr,index_flag,nl_flag)
register ESD *param;
FILE *fileptr;
int index_flag;
int nl_flag;
{
register char *cptr;
register write_length;

	if(index_flag)
	{
		cptr = &param->pb[param->index];
		write_length = param->cb - param->index;
	}
	else
	{
		cptr = param->pb;
		write_length = param->cb;
	}

	if(write_length)
		fwrite(cptr,write_length,1,fileptr);

	if(nl_flag)
		fputc(0x0A,fileptr);

	return(0);
}	/* end of fputesd */

/*+-----------------------------------------------------------------
	append_zstr_to_esd(param,zstr)
------------------------------------------------------------------*/
append_zstr_to_esd(param,zstr)
ESD *param;
char *zstr;
{
register zstrlen = strlen(zstr);
register erc = 0;

	if(zstrlen > (param->maxcb - param->cb))
	{
		zstrlen = param->maxcb - param->cb;
		erc = eBufferTooSmall;
	}

	if(zstrlen)
	{
		strncpy(param->pb + param->cb,zstr,zstrlen);
		param->cb += zstrlen;
	}
	return(erc);
}

/*+-------------------------------------------------------------------------
    get_switches(param,switches,switches_max)
--------------------------------------------------------------------------*/
int
get_switches(param,switches,switches_max)
ESD *param;
char *switches;
int switches_max;
{
register itmp;
register index;
register char *pb = param->pb;
int cb = param->cb;
char schar;

	*switches = 0;

	if(itmp = skip_cmd_break(param))
		return(itmp);

	index = param->index;
	if(*(param->pb + index) != '-')
		return(eNoSwitches);

	switches_max--;
	*switches++ = '-';
	index++;
	while((itmp < switches_max) && (index < cb))
	{
		schar = *(pb + index++);
		if(isspace(schar))
			break;
		*switches++ = schar;
	}

	param->index = index;
	*switches = 0;
	return(0);

}   /* end of get_switches() */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of esdutil.c */
