/* CHK=0xFC4B */
/*+-------------------------------------------------------------------------
	expresp.c - HDB expect/respond per SCO Devices file
	wht%n4hgf@gatech.edu

 Meaning of some of the escape characters:
 \p - pause (approximately 1/4-1/2 second delay)
 \d - delay (2 seconds)
 \D - phone number/token
 \T - phone number with Dialcodes and character translation
 \N - null byte
 \K - insert a BREAK
 \E - turn on echo checking (for slow devices)
 \e - turn off echo checking
 \r - carriage return
 \c - no new-line
 \n - send new-line
 \nnn - send octal number
 Speed - Hayes-specific speed-adaptive connect handler

  Defined functions:
	cmd_expresp(param)
	execute_expresp(expresp_script)
	expect(str)
	respond(str)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:04-02-1990-02:17-wht@n4hgf-Dialers support appears to work */
/*:04-01-1990-19:28-wht@n4hgf-more work */
/*:03-28-1990-02:38-wht-creation */

#include "ecu.h"
#include "ecuerror.h"
#include "esd.h"
#include "var.h"
#include "proc.h"


#define MAX_FIELDS	50	/* max fields in a chat script */
#define MAX_EXPRESP	511	/* max length of a chat script */
#define MAX_EXPECT	63	/* max length of expect string */
#define DEFAULT_TIMEOUT_MSECS (10*1000L)

#define ERDEBUG(verb,str,arg) if(expresp_verbosity >= verb) \
	pprintf(str,arg);

extern int proctrace;
extern int interrupt;

int expresp_verbosity = 0;
ulong expect_timeout_msecs = DEFAULT_TIMEOUT_MSECS;
int expresp_echo_check = 0;

char last_Speed_result[32];

/*+-------------------------------------------------------------------------
	expect(str) - expect (read) string from remote
return 0 on failure, 1 on success
--------------------------------------------------------------------------*/
int
expect(str)
char *str;
{
register char *parsee = str;
char parsebuf[MAX_EXPECT + 1];
register int remaining = MAX_EXPECT;
register char *parsed = parsebuf;
char s4[4];
int itmp;

	ERDEBUG(2,"expect: <<%s>>\n",str);
	if(!strlen(str) || !strcmp(str,"\"\""))
		goto GOT_EXPECTED;

	if(!strcmp(str,"Speed"))
	{
		LRWT lr;
		long ms_start;
		long ms_now;
		struct timeb now_timeb;
		ftime(&now_timeb);
		ms_start = (now_timeb.time * 1000) + now_timeb.millitm;
		do {
			last_Speed_result[0] = 0;
			lr.to1 = 90 * 100L;
			lr.to2 = 120L;
			/* allow interrupts + cooked read */
			lr.raw_flag = 0x80;
			lr.buffer = last_Speed_result;
			lr.bufsize = sizeof(last_Speed_result);
			lr.delim = "\n";
			lr.echo = 0;
			lgets_timeout(&lr);
			ftime(&now_timeb);
			ms_now = (now_timeb.time * 1000) + now_timeb.millitm;
		} while (!interrupt && !lr.count && ((ms_now - ms_start) < 90*1000L));

		if(interrupt || strncmp(lr.buffer,"CONNECT",7))
			goto DID_NOT_GET_EXPECTED;
		else
			goto GOT_EXPECTED;
	}

	while(remaining)
	{
		if(*parsee == '\\')
		{
			if(!*(++parsee))	/* if no character after escape, ... */
			{
				ERDEBUG(2," error: str ended with '\\'\n",0);
				goto DID_NOT_GET_EXPECTED;
			}

			if(isdigit(*parsee))	/* handle \ooo */
			{
				strncpy(s4,parsee,3);
				s4[3] = 0;
				sscanf(s4,"%o",&itmp);
				parsee += strspn(s4,"01234567");
				*parsed++ = (char)itmp;
				remaining--;
				continue;
			}
				
			switch(*parsee)
			{
				case 'n':
					*parsed++ = 0x0A;
					remaining--;
					break;
				case 'r':
					*parsed++ = 0x0D;
					remaining--;
					break;
				default:
					ERDEBUG(2," meaningless here: \\%c\n",*parsee);
					break;
			}
			parsee++;
		}
		else
		{
			*parsed++ = *parsee++;
			remaining--;
		}
	}
	*parsed = 0;

	if(remaining)
		ERDEBUG(1," expect string too long\n",0);

	if(expresp_verbosity >= 3)
		hex_dump(parsebuf,strlen(parsebuf),"expecting",1);

	if(llookfor(parsebuf,expect_timeout_msecs,(expresp_verbosity) ? 2 : 0))
	{
GOT_EXPECTED:
		ERDEBUG(2,"[EXPECT SUCCEEDED]\n",0);
		return(1);
	}

DID_NOT_GET_EXPECTED:
	ERDEBUG(2,"[EXPECT FAILED]\n",0);
	return(0);

}	/* end of expect */

/*+-------------------------------------------------------------------------
	respond(str) - send to remote
--------------------------------------------------------------------------*/
void
respond(str)
register char *str;
{
int itmp;
char s4[4];
int send_no_cr = 0;

	ERDEBUG(2,"respond: <<%s>>\n",str);
	while(*str)
	{
		if(*str == '\\')
		{
			/* handle \nnn - send octal number */
			if(isdigit(*++str))	/* handle \ooo */
			{
				strncpy(s4,str,3);
				s4[3] = 0;
				sscanf(s4,"%o",&itmp);
				str += strspn(s4,"01234567");
				lputc((char)itmp);
			}
			else switch(*str)
			{
				case 'p':  /* pause (approximately 1/4-1/2 second delay) */
					nap(400L);
					break;
				case 'd':  /* delay (2 seconds) */
					nap(2000L);
					break;
				case 'D':  /* phone number/token */
					if(expresp_echo_check)
						lputs_paced(40,shm->Ltelno);	/* fake it */
					else
						lputs(shm->Ltelno);
					break;
				case 'T':  /* phnum with Dialcodes and char translation */
					if(expresp_echo_check)
						lputs_paced(40,shm->Ltelno);	/* fake it */
					else
						lputs(shm->Ltelno);
					break;
				case 'N':  /* null byte */
					lputc(0);
					break;
				case 'K':  /* insert a BREAK */
					lbreak();
					break;
				case 'E':  /* turn on echo checking (for slow devices) */
					expresp_echo_check = 1;
					break;
				case 'e':  /* turn off echo checking */
					expresp_echo_check = 0;
					break;
				case 'r':  /* carriage return */
					lputc(0x0D);
					break;
				case 'c':  /* no new-line */
					send_no_cr = 1;
					break;
				case 'n':  /* send new-line */
					lputc(0x0D);
					break;
			}

		}
		else
			lputc(*str);


		if(expresp_echo_check)
		{
			nap(40L);		/* fake it */
			lflush(0);		/* flush input */
		}
		str++;
	}

	if(!send_no_cr)
		lputc(0x0D);
}	/* end of respond */

/*+-------------------------------------------------------------------------
	execute_expresp(expresp_script)

return 0 on success, else error code
--------------------------------------------------------------------------*/
int
execute_expresp(expresp_script)
char *expresp_script;
{
char *fields[MAX_FIELDS + 1];
int ifields;
int nfields;
char expresp_copy[MAX_EXPRESP + 1];
char *expect_this;
char *send_on_fail;

#define EXPECT_STATE (!(ifields & 1))	/* even numbered fields are expect */
	expresp_echo_check = 0;
	last_Speed_result[0] = 0;

	strncpy(expresp_copy,expresp_script,sizeof(expresp_copy));
	build_arg_array(expresp_copy,fields,MAX_FIELDS,&nfields);
	if(!nfields)	/* if no script, assume success */
	{
		ERDEBUG(2,"[EMPTY SCRIPT - EXPECT/RESPOND SUCCEEDED]\n",0);
		return(0);
	}

	for(ifields = 0; ifields < nfields; ifields++)
	{
		if(EXPECT_STATE)
		{
			expect_this = fields[ifields];
			while(1)	/* until break or return(-1) */
			{
				if(send_on_fail = strchr(expect_this,'-'))
					*send_on_fail++ = 0;
				if(expect(expect_this))
					break;
				if(!send_on_fail)
				{
					ERDEBUG(2,"[EXPECT/RESPOND FAILED]\n",0);
					return(eExpectRespondFail);
				}
				if(expect_this = strchr(send_on_fail,'-'))
					*expect_this++ = 0;
				respond(send_on_fail);
			}
		}
		else
			respond(fields[ifields]);
	}
	ERDEBUG(2,"[EXPECT/RESPOND SUCCEEDED]\n",0);
	return(0);

}	/* end of execute_expresp */

/*+-------------------------------------------------------------------------
	cmd_expresp(param)
expresp [-v[v...]] <exp-resp-str> [<timeout_msecs>]
--------------------------------------------------------------------------*/
int
cmd_expresp(param)
ESD *param;
{
int erc;
int itmp;
char *cptr;
ESD *tesd;
char switches[8];

	if((tesd = make_esd(MAX_EXPRESP + 1)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

	if(erc = gstr(param,tesd))
	{
		free_esd(tesd);
		return(erc);
	}

	expect_timeout_msecs = DEFAULT_TIMEOUT_MSECS;
	expresp_verbosity = (!!strchr(switches,'v')) || proctrace;
	if(expresp_verbosity)
	{
		cptr = switches, itmp = 0;
		while(*cptr)
			itmp += (*cptr++ == 'v');
		if(itmp > 1)
			expresp_verbosity = itmp;
	}
		
	if(erc = gint(param,&expect_timeout_msecs))
	{
		/* if something there non-integer */
		if(!end_of_cmd(param))
		{
			erc = eSyntaxError;
			goto RETURN;
		}
	}

	erc = execute_expresp(tesd->pb);

RETURN:
	free_esd(tesd);
	if(erc == eExpectRespondFail)
		iv[0] = 1;
	else if(!erc)
		iv[0] = 0;
	else
		return(erc);
	return(0);

}	/* end of cmd_expresp */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of expresp.c */
