/* CHK=0x826B */
char *rev="2.16";
/*+------------------------------------------------------------------------
	fcrc.c - calculate, check &/or store CRC for file

    CCITT CRC-16  x^16 + x^12 + x^5 + 1

	Since carriage returns and line feeds present a problem between
	MS-DOS and UNIX/XENIX, these two characters are not 
	included in a CRC calculation.  Refer to the main procedure
	for usage information.

    For MSDOS versions, take note that this source does NOT work
    with MSC 3.0 because under that system rename(newfile,oldfile)
    is the rule rather than MSC 4.0 rename(oldfile,newfile).  If you
    want to compile under MSC 3.0, swap rename() arguments.

  Compilation:
pyr BSD4.2	cc -O fcrc.c -o fcrc; strip fcrc
XENIX/286	cc -O -M2 -K fcrc.c -o fcrc; strip fcrc
sys V		cc -O -DATT fcrc.c -o fcrc; strip fcrc
MSDOS(MSC4) msc fcrc.c;  link fcrc+ssetargv/stack:2048; exepack fcrc...

  Defined functions:
	build_temp_names(base_fname)
	crc_clear()
	crc_finish(crc)
	filecrc(filename,crc_returned,crc_found,source_type)
	main(argc,argv,envp)
	str_index(str1,str2)
	to_upper(ch)
	trigger_type(source_type,trigger_str)
	usage()

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:11-23-1989-03:11-wht-conditional compile for M_XENIX or M_UNIX */
/*:06-21-1989-16:23-wht-add .68k support */
/*:06-14-1989-17:05-wht-add .ep support */
/*:06-13-1989-11:39-wht-add .sa as motorola asm */
/*:05-06-1989-15:14-wht-allow .l files (same as .c) */
/*:04-17-1989-13:41-wht-adapt to pyr 4.4 chgs need sys/types.h */
/*:04-15-1989-13:18-wht-do not allow -u operation if access(fn,2) fails */
/*:12-01-1988-13:23-wht-skip non-regular files */
/*:09-08-1988-21:22-wht-allow .y files (same as .c) */
/*:06-06-1988-20:27-wht-add -r, get rid of ulcmpb/ulindex */
/*:05-29-1988-14:53-wht-major overhaul -- use read/write not stdio for .NEW */
/*:05-29-1988-13:50-wht-assign version number */
/*:05-28-1988-16:58-wht-crc_update table driven: speed up order of magnitude! */
/*:02-21-1987-18:03-wht-use prototyping under XENIX and MSDOS */
/*:02-04-1987-16:20-wht-fix what happened if file matched *make*.c */
/*:01-30-1987-00:40-wht-unlink MSDOS savef before rename to it */
/*:01-29-1987-14:10-wht-better answer for MSDOS/Intermetrics asm diffs */
/*:12-18-1986-16:30-wht-use ':' in tempname rather than '$' */
/*:12-05-1986-15:00-wht-bug in BSD4.2 version did not unlink(savef) */
/*:12-04-1986-18:30-wht-make skip of 0x0D/0x0A optional */
/*:12-04-1986-15:30-wht-fflush stdout when write performed */
/*:11-18-1986-09:30-wht-move new program to Pyramid for BSD4.2 test */
/*:10-18-1986-14:00-wht-make perror calls more helpful */
/*:08-14-1986-17:30-WHT-add make source support */
/*:08-14-1986-15:30-WHT-add assembler source support */
/*:08-10-1986-15:20-WHT-add correct report code for no update_flag */
/*:07-30-1986-17:00-WHT-add code for creating CRC record in file */
/*:07-29-1986-15:30-WHT-Creation */

#include <stdio.h>

#define	STYPE_c		0
#define STYPE_asm	1	/* Intermetrics C68k */
#define STYPE_make	2
#define STYPE_bat	3

#define ff	fprintf
#define se	stderr

#if defined(M_XENIX) || defined(M_UNIX)
#define SYSV 1
#endif
#if defined(ATT)
#define SYSV 1
#endif

#include <sys/types.h>

#if defined(M_XENIX) || defined(M_UNIX)
#include <fcntl.h>
#endif

#if defined(ATT)
#include <fcntl.h>
#endif

#if MSDOS				/* MSC 4.00 */
#include <fcntl.h>
#include <process.h>
#include <io.h>
#include <string.h>
#endif

#include <sys/stat.h>

#ifndef MSDOS
#define strcmpi strcmp		/* only case-sensitive comparison under UNIX */
#define strncmpi strncmp	/* only case-sensitive comparison under UNIX */
#endif

#if defined(pyr)	/* Pyr OSx */
#include <sys/file.h>
#define	strchr	index
#define strrchr	rindex
extern char	*index();
extern char	*rindex();
#endif

#if SYSV
extern char	*strchr();
extern char	*strrchr();
#endif

#if MSDOS | XENIX
#if LINT_ARGS
char to_upper(char );
int filecrc(char *,unsigned int  *,char *,int );
int str_index(char *,char *);
unsigned short crc_clear(void);
unsigned short crc_finish(unsigned short );
void build_temp_names(char *);
void trigger_type(int ,char *);
void usage(void);
#endif
#endif

extern int errno;

char	rdbuf[32768-1024];
char	errmsg[80];
char	savef[128];		/* see build_temp_names */
char	tempf[128];		/* see build_temp_names */
char	trigger[32];
int buflen;
int skip_cr_nl = 1;	/* if we should ignore CR and LF chars */

#if MSDOS
int	msdos = 1;
#else
int	msdos = 0;
#endif
int	msdos_switch = 0;		/* -m switch */

/* crc_table calculated by Mark G. Mendel, Network Systems Corporation */
unsigned short crc_table[256] = 
{
	0x0000,0x1021,0x2042,0x3063,0x4084,0x50a5,0x60c6,0x70e7,
	0x8108,0x9129,0xa14a,0xb16b,0xc18c,0xd1ad,0xe1ce,0xf1ef,
	0x1231,0x0210,0x3273,0x2252,0x52b5,0x4294,0x72f7,0x62d6,
	0x9339,0x8318,0xb37b,0xa35a,0xd3bd,0xc39c,0xf3ff,0xe3de,
	0x2462,0x3443,0x0420,0x1401,0x64e6,0x74c7,0x44a4,0x5485,
	0xa56a,0xb54b,0x8528,0x9509,0xe5ee,0xf5cf,0xc5ac,0xd58d,
	0x3653,0x2672,0x1611,0x0630,0x76d7,0x66f6,0x5695,0x46b4,
	0xb75b,0xa77a,0x9719,0x8738,0xf7df,0xe7fe,0xd79d,0xc7bc,
	0x48c4,0x58e5,0x6886,0x78a7,0x0840,0x1861,0x2802,0x3823,
	0xc9cc,0xd9ed,0xe98e,0xf9af,0x8948,0x9969,0xa90a,0xb92b,
	0x5af5,0x4ad4,0x7ab7,0x6a96,0x1a71,0x0a50,0x3a33,0x2a12,
	0xdbfd,0xcbdc,0xfbbf,0xeb9e,0x9b79,0x8b58,0xbb3b,0xab1a,
	0x6ca6,0x7c87,0x4ce4,0x5cc5,0x2c22,0x3c03,0x0c60,0x1c41,
	0xedae,0xfd8f,0xcdec,0xddcd,0xad2a,0xbd0b,0x8d68,0x9d49,
	0x7e97,0x6eb6,0x5ed5,0x4ef4,0x3e13,0x2e32,0x1e51,0x0e70,
	0xff9f,0xefbe,0xdfdd,0xcffc,0xbf1b,0xaf3a,0x9f59,0x8f78,
	0x9188,0x81a9,0xb1ca,0xa1eb,0xd10c,0xc12d,0xf14e,0xe16f,
	0x1080,0x00a1,0x30c2,0x20e3,0x5004,0x4025,0x7046,0x6067,
	0x83b9,0x9398,0xa3fb,0xb3da,0xc33d,0xd31c,0xe37f,0xf35e,
	0x02b1,0x1290,0x22f3,0x32d2,0x4235,0x5214,0x6277,0x7256,
	0xb5ea,0xa5cb,0x95a8,0x8589,0xf56e,0xe54f,0xd52c,0xc50d,
	0x34e2,0x24c3,0x14a0,0x0481,0x7466,0x6447,0x5424,0x4405,
	0xa7db,0xb7fa,0x8799,0x97b8,0xe75f,0xf77e,0xc71d,0xd73c,
	0x26d3,0x36f2,0x0691,0x16b0,0x6657,0x7676,0x4615,0x5634,
	0xd94c,0xc96d,0xf90e,0xe92f,0x99c8,0x89e9,0xb98a,0xa9ab,
	0x5844,0x4865,0x7806,0x6827,0x18c0,0x08e1,0x3882,0x28a3,
	0xcb7d,0xdb5c,0xeb3f,0xfb1e,0x8bf9,0x9bd8,0xabbb,0xbb9a,
	0x4a75,0x5a54,0x6a37,0x7a16,0x0af1,0x1ad0,0x2ab3,0x3a92,
	0xfd2e,0xed0f,0xdd6c,0xcd4d,0xbdaa,0xad8b,0x9de8,0x8dc9,
	0x7c26,0x6c07,0x5c64,0x4c45,0x3ca2,0x2c83,0x1ce0,0x0cc1,
	0xef1f,0xff3e,0xcf5d,0xdf7c,0xaf9b,0xbfba,0x8fd9,0x9ff8,
	0x6e17,0x7e36,0x4e55,0x5e74,0x2e93,0x3eb2,0x0ed1,0x1ef0
};

/*+-------------------------------------------------------------------------
	crc_update(crc,cp)
  from article Copyright (C) 1986 Stephen Satchell. 
  NOTE: 
  First argument is referenced twice.
  Second argument must be in range 0 to 255.
 
 Programmers may incorporate any or all code into their programs, 
 giving proper credit within the source. Publication of the 
 source routines is permitted so long as proper credit is given 
 to Stephen Satchell, Satchell Evaluations and Chuck Forsberg, 
 Omen Technology.
--------------------------------------------------------------------------*/
#define crc_update(crc,data) \
	(crc_table[((crc >> 8) & 255)] ^ (crc << 8) ^ data)

/*+------------------------------------------------------------------------
	crc = crc_clear();

  This function clears the CRC to zero. It should be called prior to
  the start of the processing of a block for both received messages,
  and messages to be transmitted.
--------------------------------------------------------------------------*/
unsigned short crc_clear()
{
	return(0);
}

/*+------------------------------------------------------------------------
    crc = crc_finish(crc);

  This function must be called once after all the characters in a block
  have been processed for a message which is to be TRANSMITTED. It
  returns the calculated CRC bytes, which should be transmitted as the
  two characters following the block. The first of these 2 bytes
  must be taken from the high-order byte of the CRC, and the second
  must be taken from the low-order byte of the CRC. This routine is NOT
  called for a message which has been RECEIVED.
--------------------------------------------------------------------------*/
unsigned short crc_finish(crc)
unsigned short crc;
{
	/* Call crc_update twice, passing it a character of hex 00 each time, to */
	/* flush out the last 16 bits from the CRC calculation, and return the */
	/* result as the value of this function. */
	return(crc_update(crc_update(crc,'\0'),'\0'));
}

/*+-------------------------------------------------------------------------
	usage()
--------------------------------------------------------------------------*/
void
usage()
{

	ff(se,"fcrc %s\n",rev);
	ff(se,"Calculate file crc & compare with crc stored in file\n");
	ff(se,"Usage: fcrc [-a][-n][-u] <filelist>\n");
	ff(se," -u causes update of crc if not correct\n");
	ff(se,"    or adds CHK=0x???? record if not present\n");
	ff(se,"    (CRC will not be updated in file with unrecognized suffix.\n");
	ff(se," -r<fname> if -u, placed updated filenames in <fname>\n");
	ff(se," -n do not print files whose crc does not match\n");
	ff(se," -a include All characters in checksum (normally\n");
	ff(se,"    CRC does not include 0x0A or 0x0D characters)\n");
	ff(se," -m treat MSDOS assembler files correctly even\n");
	ff(se,"    under UNIX\n");
	ff(se,"CRC never includes trigger (first line of file).\n");
	ff(se,"Switch placement on command line is not critical.\n");
	exit(1);
}	/* end of usage */
/*+-------------------------------------------------------------------------
    to_upper()
  one would think that these were relatively standard
  types of thing, but MSC/Xenix specifies toupper() to convert to upper
  case if not already and Unix says to adjust without testing,
  so, two stupid little routines here
  ASCII only -- no EBCDIC gradoo here please
--------------------------------------------------------------------------*/
char to_upper(ch)
register char ch;
{ 
	return( ((ch >= 'a') && (ch <= 'z')) ? ch - 0x20 : ch);
}   /* end of to_upper() */

/*+-------------------------------------------------------------------------
    str_index:  Upper/Lower [case insensitive] Index functioni

  Returns position of 'str2' in 'str1' if found
  If 'str2' is null, then 0 is returned (null matches anything)
  Returns -1 if not found
--------------------------------------------------------------------------*/
int str_index(str1,str2)
register char *str1;	/* the (target) string to search */
register char *str2;	/* the (comparand) string to search for */
{
	register int istr1 = 0;				/* moving index into str1 */
	register char *mstr = str1;			/* moving string pointer */
	register int str2l = strlen(str2);	/* length of comparand string */

	if(str2[0] == '\0')             /* null string matches anything */
		return(0);
	while(1)
	{
		if(*mstr == '\0')           /* if we exhaust target string, flunk */
			return(-1);
		/* Can we find either case of first comparand char in target? */
		if(*mstr == *str2)
		{
			/* we have a first char match... does rest of string match? */
			if(strncmp(mstr,str2,str2l) == 0)	/* if the rest matches, ... */
				return(istr1);                  /* ... return match position */
		}
		/* we did not match this time... increment istr1, mstr and try again */
		++istr1;
		++mstr;
	}
}	/* end of str_index */

/*+-------------------------------------------------------------------------
	trigger_type(source_type,trigger_str)
--------------------------------------------------------------------------*/
void
trigger_type(source_type,trigger_str)
int source_type;
char	*trigger_str;
{
	switch(source_type)
	{
	case STYPE_c:
		strcpy(trigger_str,"/* CHK=0x");
		break;
	case STYPE_asm:
		if((msdos) || msdos_switch)
			strcpy(trigger_str,";  CHK=0x");
		else
			strcpy(trigger_str,"** CHK=0x");
		break;
	case STYPE_make:
		strcpy(trigger_str,"#  CHK=0x");
		break;
	case STYPE_bat:
		strcpy(trigger_str,"rem CK=0x");
		break;
	default:
		strcpy(trigger_str,"$%#$");
	}

}	/* end of trigger_type */

/*+-------------------------------------------------------------------------
	filecrc(filename,crc_returned,crc_found)
--------------------------------------------------------------------------*/
int
filecrc(filename,crc_returned,crc_found,source_type)
char *filename;
unsigned	*crc_returned;
char *crc_found;
int source_type;
{
	register unsigned itmp;
	register unsigned short crc;
	register int crc_char;
	/* register unsigned char x, y; */
	FILE	*fin;
	char	crc_string[128];
	char	crc_trigger[25];
	register char *rptr;
	extern int errno;

	trigger_type(source_type,crc_trigger);

	if( (fin = fopen(filename,"r")) == NULL)
		return(1);

	setbuf(fin,NULL);

	errno = -2;
	if(fgets(crc_string,sizeof(crc_string),fin) == NULL)
	{
		sprintf(errmsg,"Cannot read file %s: ",filename);
		if(errno == -2)
			fprintf(stderr,"%s: empty file\n",errmsg);
		else
			perror(errmsg);
		fclose(fin);
		return(2);
	}

	if((itmp = str_index(crc_string,crc_trigger)) != 0)
		strcpy(crc_found,"NONE");
	else
	{
		strncpy(crc_found,&crc_string[9],4);
		crc_found[4] = 0;
	}

	crc = crc_clear();

	if(*crc_found == 'N')	/* if we did not find the target string */
	{
		for(itmp = 0; itmp < strlen(crc_string); itmp++)
			crc = crc_update(crc,crc_string[itmp]);
	}

	while((itmp = read(fileno(fin),rdbuf,sizeof(rdbuf))) > 0)
	{
		rptr = rdbuf;
		while(itmp--)
		{
			if((skip_cr_nl == 0) || ((*rptr != 0x0A) && (*rptr != 0x0D)))
				crc = crc_update(crc,*rptr);
			rptr++;
		}
	}
	*crc_returned = crc_finish(crc);
	fclose(fin);
	return(0);

#if SkipThis	/* algorithm for checking incoming crc'd msg */
	x = (unsigned char)((crc & 0xff00) >> 8);
	y = (unsigned char)(crc & 0x00ff);
	crc = crc_clear();
	for(ibuf=0; ibuf<buflen; ++ibuf)
		crc = crc_update(crc,buf[ibuf]);
	crc = crc_update(crc,x);
	crc = crc_update(crc,y);
	/* If the result was 0, then message was received without error */
#endif

}	/* end of filecrc() */

/*+-----------------------------------------------------------------------
	build_temp_names()

  Build temp filenames for munging around
------------------------------------------------------------------------*/
void
build_temp_names(base_fname)
register char *base_fname;
{
	register char *cptr;

#if MSDOS
	sprintf(tempf,"$fcrctmp.tmp");
	sprintf(savef,"$fcrcsav.sav");
#else
	strcpy(tempf,base_fname);
	if((cptr = strrchr(tempf,'/')) == NULL)
		cptr = tempf;
	else
		++cptr;
	*cptr = '@';
	strcpy(savef,base_fname);
	if((cptr = strrchr(savef,'/')) == NULL)
		cptr = savef;
	else
		++cptr;
	*cptr = ':';
#endif

	unlink(tempf);
	unlink(savef);

}	 /* end of build_temp_names */

/*+-------------------------------------------------------------------------
	main()
--------------------------------------------------------------------------*/
main(argc,argv,envp)
int argc;
char	**argv;
char	**envp;
{
	register int itmp;
	register int update_flag = 0;
	register int no_print_ok = 0;
	register int did_update = 0;
	register int match = 0;
	register int source_type;
	register int iargv;
	register int fdin;
	register int fdout;
	register char *cptr;
	char	crc_calculated[6];
	char	crc_found[6];
	unsigned crc;
	FILE	*fp_report = NULL;
	struct stat statbuf;
#if DEBUG
	char s80[80];
#endif

	setbuf(stdout,NULL);
	setbuf(stderr,NULL);

	if(argc == 1)
		usage();

	for(iargv = 1; iargv < argc; iargv++)
	{
		if(*argv[iargv] != '-')
			continue;
		itmp = 1;
		while(argv[iargv][itmp])
		{
			switch(argv[iargv][itmp])
			{
			case 'm':
				msdos_switch = 1;
				if(msdos)
					ff(se,"-m switch not required under MSDOS\n");
				break;
			case 'u':
				update_flag = 1;
				break;
			case 'n':
				no_print_ok = 1;
				break;
			case 'a':		/* all chars */
				skip_cr_nl = 0;
				break;
			case 'r':
				if((fp_report = fopen(&argv[iargv][itmp+1],"a")) == NULL)
				{
					perror(&argv[iargv][itmp+1]);
					exit(1);
				}
				goto NEXT_ARG;
			default:
				ff(se,"Unrecognized switch: -%c\n",
				    argv[iargv][itmp]);
				usage();
			}
			itmp++;
		}
NEXT_ARG:	
		;
	}

	for(iargv = 1; iargv < argc; iargv++)
	{
		if(*argv[iargv] == '-')
			continue;
		if(stat(argv[iargv],&statbuf))
		{
			perror(argv[iargv]);
			continue;
		}
		if((statbuf.st_mode & S_IFMT) != S_IFREG)
		{
			ff(se,"not regular file: %s\n",argv[iargv]);
			continue;
		}

		source_type = -1;		/* start with bad number */
		itmp = strlen(argv[iargv]);

		cptr = strrchr(argv[iargv],'/');
		if(cptr == NULL)
			cptr = argv[iargv];
		else
			++cptr;

		if((itmp >= 2) && (strcmpi(&argv[iargv][itmp - 2],".h") == 0))
			source_type = STYPE_c;
		else if((itmp >= 2) && (strcmpi(&argv[iargv][itmp - 2],".c") == 0))
			source_type = STYPE_c;
		else if((itmp >= 2) && (strcmpi(&argv[iargv][itmp - 2],".y") == 0))
			source_type = STYPE_c;
		else if((itmp >= 2) && (strcmpi(&argv[iargv][itmp - 2],".l") == 0))
			source_type = STYPE_c;
		else if((itmp >= 4) && (strcmpi(&argv[iargv][itmp - 4],".asm") == 0))
			source_type = STYPE_asm;
		else if((itmp >= 4) && (strcmpi(&argv[iargv][itmp - 4],".68k") == 0))
			source_type = STYPE_asm;
		else if((itmp >= 4) && (strcmpi(&argv[iargv][itmp - 3],".sa") == 0))
			source_type = STYPE_asm;
		else if((itmp >= 3) && (strcmpi(&argv[iargv][itmp - 3],".eq") == 0))
			source_type = STYPE_asm;
		else if((itmp >= 2) && (strcmpi(&argv[iargv][itmp - 2],".s") == 0))
			source_type = STYPE_asm;
#ifdef MSDOS
		else if((itmp >= 4) && (strcmpi(&argv[iargv][itmp - 4],".MAK") == 0))
			source_type = STYPE_make;
#endif
		else if((itmp >= 5) && (strcmpi(&argv[iargv][itmp - 5],".make") == 0))
			source_type = STYPE_make;
		else if((itmp >= 3) && (strcmpi(&argv[iargv][itmp - 3],".ep") == 0))
			source_type = STYPE_make;
		else if((itmp >= 3) && (strcmpi(&argv[iargv][itmp - 3],".mi") == 0))
			source_type = STYPE_make;
		else if((itmp >=  5) && (strncmpi(cptr,"Make",4) == 0))
			source_type = STYPE_make;
#ifndef MSDOS
		else if((itmp >=  5) && (strncmpi(cptr,"make",4) == 0))
			source_type = STYPE_make;
#endif
#ifdef MSDOS
		else if((itmp >= 4) && (strcmpi(&argv[iargv][itmp - 4],".bat") == 0))
			source_type = STYPE_bat;
#endif

		if(itmp = filecrc(argv[iargv],&crc,crc_found,source_type))
		{
			switch(itmp)
			{
			case 1:
				sprintf(errmsg,"Cannot access file %s: ",argv[iargv]);
				perror(errmsg);
			default:
				break;
			}
		}
		else
		{
			sprintf(crc_calculated,"%04x",crc);
			for(crc = 0; crc < strlen(crc_calculated); crc++)
				crc_calculated[crc] = to_upper(crc_calculated[crc]);
			if(source_type < 0)
			{
				printf("%s %s %s --> unknown file type\n",crc_found,
				    crc_calculated,argv[iargv]);
				continue;
			}
			if(update_flag)
			{
				did_update = 0;
				if(crc_found[0] == 'N')
				{
					if(access(argv[iargv],2))
					{
						printf("%s %s %s --> update write access denied\n",
							crc_found,crc_calculated,argv[iargv]);
						continue;
					}
					build_temp_names(argv[iargv]);	/* global 'tempf'/'savef' */

					if((fdout = open(tempf,O_CREAT | O_WRONLY,0644)) < 0)
					{
						ff(se,"trigger add cannot open tempf...abort!\n");
						perror(tempf);
						exit(210);
					}
					trigger_type(source_type,trigger);
					if(write(fdout,trigger,strlen(trigger)) < 0)
					{
						ff(se,"trigger add temp file write error\n");
						perror(tempf);
						exit(211);
					}
					if(write(fdout,"....",4) < 0)
					{
						ff(se,"trigger add temp file write error\n");
						perror(tempf);
						exit(211);
					}
					if(source_type == STYPE_c)
						if(write(fdout," */",3) < 0)
						{
							ff(se,"trigger add temp file write error\n");
							perror(tempf);
							exit(213);
						}
					if(write(fdout,"\n",1) < 0)
					{
						ff(se,"trigger add temp file write error\n");
						perror(tempf);
						exit(215);
					}
					if((fdin = open(argv[iargv],O_RDONLY,0)) < 0)
					{
						ff(se,"Logic error: open for trigger add failed\n");
						perror(argv[iargv]);
						exit(200);
					}
					while((buflen = read(fdin,rdbuf,sizeof(rdbuf))) > 0)
					{
#if DEBUG
						printf("new file read count %d\n",buflen);
#endif
						if(write(fdout,rdbuf,buflen) < 0)
						{
							ff(se,"trigger add fwrite failure..abort!\n");
							perror(tempf);
							exit(201);
						}
					}
#if DEBUG
					printf("new file read count exit %d\n",buflen);
#endif
					if(buflen < 0)
					{
						ff(se,"read error: trigger add\n");
						perror(argv[iargv]);
						exit(200);
					}
					close(fdin);
					close(fdout);
#if MSDOS
					/* MSC 3.0 rename(to,from) */
					/* MSC 4.0 rename(from,to) */
					/* MSC 5.0 rename(from,to) (not verified, but logical) */
					if(rename(argv[iargv],savef))	/* MSC 4.0 */
					{
						ff(se,"trigger add rename %s to %s failed\n",
						    argv[iargv],savef);
						perror("rename");
						exit(202);
					}
					if(rename(tempf,argv[iargv]))	/* MSC 4.0 */
					{
						ff(se,"trigger add rename 2 %s to %s failed\n",
						    tempf,argv[iargv]);
						perror("rename");
						ff(se,"SORRY, BUT SEE THIS: YOUR FILE IS NOW IN %s\n",
						    savef);
						exit(202);
					}
					if(unlink(savef))
					{
						char	pmsg[80];
						sprintf(pmsg,"trigger add unlink of %s failed",savef);
						perror(pmsg);
						exit(202);
					}
#endif

#if SYSV
					if(link(argv[iargv],savef))
					{
						ff(se,"ln %s to %s failed\n",argv[iargv],savef);
						perror("trigger add link 1 failure");
						exit(202);
					}
#if DEBUG
					printf("link1:\n");
					sprintf(s80,"fstat %s %s",argv[iargv],savef);
					system(s80);
#endif
					if(unlink(argv[iargv]))
					{
						ff(se,"unlink %s failed\n",argv[iargv]);
						perror("trigger add unlink 1 failure");
						ff(se,"SORRY, BUT SEE THIS: YOUR FILE IS NOW IN %s\n",
						    savef);
						exit(202);
					}
#if DEBUG
					printf("unlink %s ok\n",argv[iargv]);
#endif
					if(link(tempf,argv[iargv]))
					{
						ff(se,"ln %s to %s failed\n",tempf,argv[iargv]);
						perror("trigger add link 2 failure");
						exit(202);
					}
#if DEBUG
					printf("link2:\n");
					sprintf(s80,"fstat %s %s",tempf,argv[iargv]);
					system(s80);
#endif
					if(unlink(savef))	/* ok to delete old file now */
					{
						char	pmsg[80];
						sprintf(pmsg,"trigger add unlink of %s failure",savef);
						perror(pmsg);
						exit(202);
					}
#if DEBUG
					printf("unlink %s ok\n",savef);
#endif
					if(unlink(tempf))	/* and temp link to new file */
					{
						char	pmsg[80];
						sprintf(pmsg,"trigger add unlink of %s failure",savef);
						perror(pmsg);
						exit(202);
					}
#if DEBUG
					printf("unlink %s ok\n",tempf);
#endif
#endif

#if defined(pyr)
					if(rename(argv[iargv],savef))	/* rename(from,to) */
					{
						ff(se,"trigger add rename 1 failure.abort!\n");
						exit(202);
					}
					if(rename(tempf,argv[iargv]))
					{
						ff(se,"trigger add rename 2 failure...abort!\n");
						ff(se,"SORRY, BUT SEE THIS: YOUR FILE IS NOW IN %s\n",
						    savef);
						exit(202);
					}
					if(unlink(savef))
					{
						char	pmsg[80];
						sprintf(pmsg,"trigger add unlink of %s failure",savef);
						perror(pmsg);
						exit(202);
					}
#endif
					if(itmp = filecrc(argv[iargv],&crc,crc_found,source_type))
					{
						ff(se,"2nd crc after trigger add failed\n");
						exit(220);
					}
					sprintf(crc_calculated,"%04x",crc);
					for(crc = 0; crc < strlen(crc_calculated); crc++)
						crc_calculated[crc] = to_upper(crc_calculated[crc]);
					strcpy(crc_found,".NEW");
				}	/* end of found no crc tag in file */

				if(strcmp(crc_found,crc_calculated) != 0)
				{
					if(access(argv[iargv],2))
					{
						printf("%s %s %s --> update write access denied\n",
							crc_found,crc_calculated,argv[iargv]);
						continue;
					}
					if((itmp = open(argv[iargv],O_RDWR,0755)) < 0)
					{
						sprintf(errmsg,"Cannot open for update write: %s: ",
						    argv[iargv]);
						perror(errmsg);
					}
					else
					{
#ifndef L_SET
						if(lseek(itmp,9L,0) < 0)
#else
							if(lseek(itmp,9L,L_SET) < 0)
#endif
							{
								sprintf(errmsg,"Cannot seek for update write: %s: ",
								    argv[iargv]);
								perror(errmsg);
								close(itmp);
							}
							else
							{
								if(write(itmp,crc_calculated,4) < 0)
								{
/* code folded from here */
	sprintf(errmsg,"Cannot write updated crc to %s",
	    argv[iargv]);
	perror(errmsg);
	exit(1);
/* unfolding */
								}
								close(itmp);
								did_update = 1;
								printf("%s %s %s --> updated\n",crc_found,
								    crc_calculated,argv[iargv]);
								if(fp_report)
/* code folded from here */
	fprintf(fp_report,"%s\n",argv[iargv]);
/* unfolding */
								continue;
							}	/* end of actually updating file */
					}		/* end of we could open for updating */
				}			/* end of file needed updating */
			}				/* end of if update_flag */

			match = (strcmp(crc_calculated,crc_found) == 0);
			if((match && (no_print_ok == 0)) || !match)
				printf("%s %s %s",crc_found,crc_calculated,argv[iargv]);
			if(!match)
				printf(" --> file crc not current");
			if((match && (no_print_ok == 0)) || !match)
				printf("\n");
		}
	}

	if(fp_report)
		fclose(fp_report);
	exit(0);

}	/* end of main() */
/* end of fcrc.c */

/* vi: set tabstop=4 shiftwidth=4: */
