/* CHK=0xD088 */
/*+-------------------------------------------------------------------------
    gstr.c - ecu get string parameter functions
	Copyright 1989 Warren H. Tucker, III.  All rights reserved.

  Defined functions:
	gstr(param,result)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:04-08-1990-19:15-wht@n4hgf-use new auto_create option on get_svptr */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-26-1989-23:59-wht-fix bug where svar too long didn't report error */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecuerror.h"
#include "esd.h"
#include "var.h"

extern int proctrace;

/*+-------------------------------------------------------------------------
    gstr() - get a string constant, variable, function or expression

Examples:

 set $s0='test ... '+%date+' '+%time+%chr(0x0D)+%chr(0x0A)
 hexdump $s0
0000  74 65 73 74 20 2E 2E 2E 20 30 36 2D 30 39 2D 31 | test ... 06-09-1 |
0010  39 38 39 20 31 37 3A 31 35 0D 0A                | 989 17:15..      |

 set $s0='12345678':1-6+'abc'
 set s0
$S00 = '234567abc'

--------------------------------------------------------------------------*/
int
gstr(param,result)
ESD *param;
ESD *result;
{
register char param_char;
register char *pb;
ESD *tesd;
ESD *svptr;
int cb = 0;
int segment_index;
int next_is_literal = 0;	/* last char was not a backslash */
ulong itmp1;
ulong itmp2;
ulong itmp3;
ulong itmp4;
int erc;
int param_index_save;
int result_remaining;
int in_quoted_string = 0;   /* not currently in quoted string */
int end_of_parameter = 0;

	if(erc = skip_cmd_break(param))
		return(erc);

	segment_index = 0;			
	result_remaining = result->maxcb;	/* number we can put into result */
	param_index_save = param->index;

	if((tesd = make_esd(result_remaining)) == (ESD *)0)
		return(eNoMemory);
	pb = tesd->pb;

CONCATENATE:
	while((param->index < param->cb) && !end_of_parameter)
	{
		param_char = param->pb[param->index];
		if(in_quoted_string)
		{
			++param->index;
			if(next_is_literal)
			{
				next_is_literal = 0;
				switch(param_char)
				{
					case 'b' : param_char = 0x08; break;
					case 'n' : param_char = 0x0A; break;
					case 'r' : param_char = 0x0D; break;
					case 't' : param_char = 0x09; break;
					case '\'': param_char = '\''; break;
				}
				if(result_remaining-- == 0)
				{
					erc = eBufferTooSmall;
					goto RETURN;
				}
				*(pb + cb++) = param_char;
			}
			else if(param_char == '\\')
				next_is_literal = 1;
			else if(param_char == '\'')
				in_quoted_string = 0;
			else
			{
				if(result_remaining-- == 0)
				{
					erc = eBufferTooSmall;
					goto RETURN;
				}
				*(pb + cb++) = param_char;
			}
		}
		else /* not in quoted string */
		{
			param->old_index = param->index;
			switch(param_char)
			{
			case '\'':      /* apostrophe denotes literal text */
				++param->index;
				in_quoted_string = 1;
				break;

			case '%':
				++param->index;
				tesd->cb = cb;
				if(erc = feval_str(param,tesd))
					goto RETURN;
				cb = tesd->cb;
				result_remaining = (result->maxcb - cb);
				break;

			case '$':           /* '$Snn' variable reference? */
				/* must be at least two more character */
				if(param->index >= param->cb-2)
				{
					erc = eSyntaxError;
					goto RETURN;
				}
				param->old_index = ++param->index;
				if(to_lower(param->pb[param->index++]) != 's' )
				{
					erc = eIllegalVarType;
					goto RETURN;
				}
				if(erc = get_svptr(param,&svptr,0))
					goto RETURN;
				if(svptr->cb > (result->maxcb - cb))
				{
					erc = eBufferTooSmall;
					goto RETURN;
				}
				else if(svptr->cb)
				{
					memcpy(&pb[cb],svptr->pb,svptr->cb);
					cb += svptr->cb;
					result_remaining -= svptr->cb;
				}
				break;

			case ':':
/*
itmp1 holds col 1 (0-n) of substring operation
itmp2 holds col 2 (0-n) of operation adjusted to reflect end of string segment
itmp3 holds length of string segment
itmp4 holds length of substring segment output by substring operation
*/
				if(erc = gcol_range(param,&itmp1,&itmp2))
					goto RETURN;
				if((itmp3 = cb - segment_index)
				    &&
					(itmp4 = ((itmp2<itmp3)?itmp2:itmp3) - itmp1 + 1))
				{
					if(itmp1)
						memcpy(&pb[segment_index],
							&pb[segment_index+(int)itmp1],(int)itmp4);
					cb -= ((int)itmp3 - (int)itmp4);
				}
				break;

			case '+':
				segment_index = cb;
				++param->index;
				goto CONCATENATE;

			case ';':
			case '#':
				end_of_parameter = 1;
				break;

			default:
				null_terminate_esd(result);
				erc = 0;
				if((param->index < param->cb) &&
					isalnum(*(param->pb + param->index)))
					erc = eSyntaxError;
				else if(param_index_save == param->index)
					erc = eBadParameter;
				end_of_parameter = 1;
				break;
			}   /* end of switch (param_char) */
		}       /* end of else not in quoted string */
	}           /* end of while(index<cb) */


RETURN:
	if(cb)
		strncpy(result->pb,pb,cb);
	result->cb = cb;
	null_terminate_esd(result);
	free_esd(tesd);
	return(erc);
}   /* end of gqstr */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of qstr.c */
