/* CHK=0xC504 */
/*+-------------------------------------------------------------------------
	pcmd.c - ecu procedure commands
	wht%n4hgf@gatech.edu

  Defined functions:
	cmd_baud(param)
	cmd_cd(param)
	cmd_clrx(param)
	cmd_dial(param)
	cmd_duplex(param)
	cmd_echo(param)
	cmd_exit(param)
	cmd_flush(param)
	cmd_hangup(param)
	cmd_hexdump(param)
	cmd_lbreak(param)
	cmd_lgets(param)
	cmd_logevent(param)
	cmd_lookfor(param)
	cmd_nap(param)
	cmd_parity(param)
	cmd_prompt(param)
	cmd_ptrace(param)
	cmd_send(param)
	cmd_set(param)
	cmd_system(param)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-03:08-wht@n4hgf-GCC run found unused vars -- rm them */
/*:04-08-1990-19:15-wht@n4hgf-use new auto_create option on get_?vptr */
/*:03-27-1990-15:18-wht@n4hgf-overhaul for/wait/SIGCLD handling */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:03-25-1990-12:33-wht@n4hgf-allow whole spec for int vars in cmd_getf */
/*:03-24-1990-19:01-wht@n4hgf-use auto_create switch on find_mkv routines */
/*:03-12-1990-04:42-wht@n4hgf-add cmd_rname */
/*:03-11-1990-13:09-wht@n4hgf-hangup command now returns 0 if no line */
/*:02-25-1990-02:59-wht-add -m switch to nap command */
/*:11-20-1989-16:05-wht-add cmd_getf and unimplemented cmd_putf */
/*:11-04-1989-06:21-wht-use make_char_graphic when tracing svar assignment */
/*:10-14-1989-21:10-wht-allow user to change ecu color choices */
/*:07-27-1989-15:33-wht-was not freeing delimiter esd in cmd_lgets */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecuerror.h"
#include "ecuhangup.h"
#include "ecukey.h"
#include "esd.h"
#include "var.h"
#include "proc.h"

char *strchr();

extern int proctrace;
extern int rcvr_pid;
extern ulong colors_current;

/*+-------------------------------------------------------------------------
	cmd_baud(param)
--------------------------------------------------------------------------*/
int
cmd_baud(param)
ESD *param;
{
long new_baud;
int erc;

	if(shm->Liofd < 0)
		return(eNoLineAttached);

	if(erc = gint(param,&new_baud))
		return(erc);
	if(!valid_baud_rate((uint)new_baud))
	{
		pprintf("invalid baud rate: %lu\n",new_baud);
		return(eFATAL_ALREADY);
	}
	shm->Lbaud = (uint)new_baud;
	lset_baud_rate(1);
	if(proctrace)
	{
		pprintf("baud rate set to %u\n",shm->Lbaud);
	}
	return(0);

}	/* end of cmd_baud */

/*+-------------------------------------------------------------------------
	cmd_cd(param)
--------------------------------------------------------------------------*/
int
cmd_cd(param)
ESD *param;
{
int erc;
ESD *tesd = make_esd(256);
extern char curr_dir[];		/* current working directory */

	if(!tesd)
		return(eNoMemory);
	if(erc = gstr(param,tesd))
		goto RETURN;
	if(expand_dirname(tesd->pb,tesd->maxcb))
	{
		param->index = param->old_index;
		erc = eFATAL_ALREADY;
		goto RETURN;
	}
	if(chdir(tesd->pb) < 0)		/* now change to the new directory */
	{
		pperror(tesd->pb);		/* print error if we get one */
		pputs("\n");
		erc = eFATAL_ALREADY;
		goto RETURN;
	}
	get_curr_dir(curr_dir,256);

RETURN:
	free_esd(tesd);
	return(erc);
}	/* end of cmd_cd */

/*+-------------------------------------------------------------------------
	cmd_clrx(param)
--------------------------------------------------------------------------*/
int
cmd_clrx(param)
ESD *param;
{
	if(shm->Liofd < 0)
		return(eNoLineAttached);

	lclear_xmtr_xoff();
	if(proctrace)
		pputs("transmitter XOFF cleared\n");
	return(0);
}	/* end of cmd_clrx */

/*+-------------------------------------------------------------------------
	cmd_dial(param)

  hayes_dial does the following:
  sets I0 to 0==connect,
             1==failed to connect,
             2==interrupted,
             3==modem error
  sets S0 to modem result code
--------------------------------------------------------------------------*/
int
cmd_dial(param)
ESD *param;
{
int erc;
ESD *tesd = (ESD *)0;

	if(shm->Lmodem_off_hook)
	{
		pputs("Already off hook\n");
		return(eFATAL_ALREADY);
	}
	if(!(tesd = make_esd(64)))
		return(eNoMemory);
	if(erc = gstr(param,tesd))
	{
		free_esd(tesd);
		return(erc);
	}
	strcpy(shm->Llogical,tesd->pb);
	lclear_xmtr_xoff();
	if(lookup_logical_telno())
		(void)hayes_dial();
	else
	{
		pprintf("No such entry: %s\n",tesd->pb);
		erc = eFATAL_ALREADY;
	}

	if(!erc && (shm->Liofd < 0))
		erc = eNoLineAttached;

	free_esd(tesd);

	return(erc);
}	/* end of cmd_dial */

/*+-------------------------------------------------------------------------
	cmd_duplex(param)
duplex [f | h]
duplex ['f' | 'h']
duplex <int>  0 == half, non-0 == full
--------------------------------------------------------------------------*/
int
cmd_duplex(param)
ESD *param;
{
int erc;
int new_duplex;
ESD *tesd;

	if(erc = skip_cmd_break(param))
		return(erc);
	if(!(tesd = make_esd(64)))
		return(eNoMemory);
	erc = gstr(param,tesd);
	new_duplex = to_upper((erc) ? param->pb[param->index] : *tesd->pb);
	free_esd(tesd);

	switch(new_duplex)
	{
		case 'f':
			shm->Lfull_duplex = 1;
			break;
		case 'h':
			shm->Lfull_duplex = 0;
			break;
		default:
			erc = eBadParameter;
	}
	if(proctrace && !erc)
		pprintf("duplex set to %s\n",(shm->Lfull_duplex) ? "full" : "half");
	return(erc);

}	/* end of cmd_duplex */

/*+-------------------------------------------------------------------------
	cmd_echo(param)
echo [-n] <str>
--------------------------------------------------------------------------*/
int
cmd_echo(param)
ESD *param;
{
int erc;
ESD *tesd;
char switches[8];

	if((tesd = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

	if(erc = gstr(param,tesd))
	{
		free_esd(tesd);
		return(erc);
	}
	pputs(tesd->pb);
	if(!strchr(switches,'n'))	/* if no -n */
		pputs("\n");
	free_esd(tesd);
	return(0);

}	/* end of cmd_echo */

/*+-------------------------------------------------------------------------
	cmd_exit(param)
--------------------------------------------------------------------------*/
int
cmd_exit(param)
ESD *param;
{
long int1;
ulong colors_at_entry = colors_current;

	if(!gint(param,&int1) && int1)
	{
		setcolor(colors_error);
		pprintf("[procedure exiting ecu: user code %ld]\n",int1);
		setcolor(colors_at_entry);
		hangup(HANGUP_INIT_PROC_ERROR+(int)(int1 & 0x1F));
	}
	setcolor(colors_notify);
	pputs("[procedure exiting ecu: normal exit]\n");
	setcolor(colors_at_entry);
	hangup(0);
}	/* end of cmd_exit */

/*+-------------------------------------------------------------------------
	cmd_lgets(param)

lgets [-er] <strvar> <int1> <int2> [<str>]

read string into string variable number <stvar>
waiting <int1> 1/10th secs for first char,
waiting <int2> 1/10th secs for subsequent chars,
optionally terminating read upon detection of <str>
-e echos to screen
-r completely raw, else strip CRs & NLs from either end of string
$i0 receives the length of the read
<strvar> receives the string
--------------------------------------------------------------------------*/
int
cmd_lgets(param)
ESD *param;
{
int erc;
long int2;
long int3;
ESD *tesd1 = (ESD *)0;
ESD *svptr;
LRWT lr;
char switches[8];
ESD *make_esd();
char ctmp;

	if(shm->Liofd < 0)
		return(eNoLineAttached);

	get_switches(param,switches,sizeof(switches));

	skip_cmd_char(param,'$');
	if(erc = get_cmd_char(param,&ctmp))
		return(erc);
	if(to_lower(ctmp) != 's')
		return(eIllegalVarType);
	if(erc = get_svptr(param,&svptr,1))
		return(erc);

	if(erc = gint(param,&int2))
		return(erc);

	if(erc = gint(param,&int3))
		return(erc);

	if((tesd1 = make_esd(64)) == (ESD *)0)
		return(eNoMemory);
	if(gstr(param,tesd1))	/* optional delimiter */
	{
		free_esd(tesd1);
		tesd1 = (ESD *)0;
	}	

	zero_esd(svptr);

	lr.to1 = int2 * 100L;
	lr.to2 = int3 * 100L;
	/* allow interrupts + raw read per -r */
	lr.raw_flag = (strchr(switches,'r')) ? 0x81 : 0x80;
	lr.buffer = svptr->pb;
	lr.bufsize = svptr->maxcb;
	lr.delim = (tesd1) ? tesd1->pb : (char *)0;
	lr.echo = (strchr(switches,'e') != (char *)0);
	(void)lgets_timeout(&lr);
	if(tesd1)
		free_esd(tesd1);

	svptr->cb = lr.count;
	null_terminate_esd(svptr);
	iv[0] = (long)lr.count;
	if(proctrace)
		pprintf("lgets read %d chars\n",lr.count);
	return(0);

}	/* end of cmd_lgets */

/*+-------------------------------------------------------------------------
	cmd_flush(param)
--------------------------------------------------------------------------*/
int
cmd_flush(param)
ESD *param;
{
	if(shm->Liofd < 0)
		return(eNoLineAttached);

	lflush(2);
	if(proctrace)
		pputs("line flushed\n");
	return(0);
}	/* end of cmd_flush */

/*+-------------------------------------------------------------------------
	cmd_hangup(param)
--------------------------------------------------------------------------*/
int
cmd_hangup(param)
ESD *param;
{
	if(shm->Liofd < 0)
	{
		if(proctrace)
			pputs("no line attached ... hangup ignored\n");
		hayes_modem_on_hook();
		return(0);
	}

	if(proctrace)
		pputs("hanging up ... ");
	hayes_ATH();
	if(proctrace)
		pputs("line on hook\n");
	return(0);
}	/* end of cmd_hangup */

/*+-------------------------------------------------------------------------
	cmd_hexdump(param)

hexdump [-s] <str>
hexdump -t[s] <str1> <str>
<str> buf to dump
<str1> title (if -t)
-s short (terse) dump
--------------------------------------------------------------------------*/
int
cmd_hexdump(param)
ESD *param;
{
int erc;
ESD *title = (ESD *)0;
ESD *buf;
char switches[8];
extern FILE *plog_fp;

	if((buf = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

	if(strchr(switches,'t'))	/* if -t */
	{
		if((title = make_esd(256)) == (ESD *)0)
		{
			erc = eNoMemory;
			goto RETURN;
		}
		if(erc = gstr(param,title))
			goto RETURN;
	}

	if(erc = gstr(param,buf))
		goto RETURN;

	hex_dump(buf->pb,buf->cb,(title) ? title->pb : "",
		(strchr(switches,'s')) ? 1 : 0);

	if(plog_fp)
		hex_dump_fp(plog_fp,buf->pb,buf->cb,(title) ? title->pb : "",
			(strchr(switches,'s')) ? 1 : 0);

RETURN:
	free_esd(buf);
	if(title)
		free_esd(title);
	return(erc);

}	/* end of cmd_hexdump */

/*+-------------------------------------------------------------------------
	cmd_lbreak(param)
--------------------------------------------------------------------------*/
int
cmd_lbreak(param)
ESD *param;
{
	if(shm->Liofd < 0)
		return(eNoLineAttached);

	lbreak();
	return(0);
}	/* end of cmd_lbreak */

/*+-------------------------------------------------------------------------
	cmd_logevent(param)

logevent 'cmd'
--------------------------------------------------------------------------*/
int
cmd_logevent(param)
ESD *param;
{
int erc;
ESD *eventstr;
extern int last_child_wait_status;
char switches[8];

	if((eventstr = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

/* a hack */
	strcpy(eventstr->pb,"PROC ");
	eventstr->pb += 5;
	eventstr->maxcb -= 5;

	if(erc = gstr(param,eventstr))
	{
		eventstr->pb -= 5;		/* be nice */
		eventstr->maxcb += 5;	/* or surely this will haunt us one day */
		free_esd(eventstr);
		return(erc);
	}

/* rehack */
	eventstr->pb -= 5;
	eventstr->maxcb += 5;
	eventstr->cb += 5;

	ecu_log_event(eventstr->pb);
	free_esd(eventstr);
	return(0);

}	/* end of eventstr_logevent */

/*+-------------------------------------------------------------------------
	cmd_lookfor(param)

lookfor [-e] [quiet | <str>] [<int>]

-e echo to screen while looking
quiet means look for quiet
<str> means look for string
<int> number 1/10ths secs (default 5.0 second) for timeout

in case of lookfor <str>, $i0 plugged 1 if found, else 0
--------------------------------------------------------------------------*/
int
cmd_lookfor(param)
ESD *param;
{
int erc;
char switches[8];
char *cptr = (char *)0;
ESD *tesd = (ESD *)0;
ulong decisecs = 50; /* default wait is 5 seconds */
int echo_flag;
char quiet[8];
long start_secs;
long time();

	if(shm->Liofd < 0)
		return(eNoLineAttached);

	get_switches(param,switches,sizeof(switches));
	echo_flag = (strchr(switches,'e') != (char *)0);

	if(!get_alpha_zstr(param,quiet,sizeof(quiet)))
	{
		if(strcmp(quiet,"quiet"))
			return(eSyntaxError);
	} 
	else
	{
		if((tesd = make_esd(64)) == (ESD *)0)
			return(eNoMemory);
		if(erc = gstr(param,tesd))
			goto RETURN;
		if(!tesd->cb)
		{
			pputs("lookfor null string\n");
			erc = eFATAL_ALREADY;
			goto RETURN;
		}
		cptr = tesd->pb;
	}

	if(erc = gint(param,&decisecs))
	{
		/* if something there non-integer */
		if(!end_of_cmd(param))
		{
			erc = eSyntaxError;
			goto RETURN;
		}
	}
	erc = 0;

	if(proctrace)
		time(&start_secs);

	if(cptr)
	{
		iv[0] = (long)llookfor(cptr,decisecs * 100L,echo_flag);
		if(proctrace)
			pprintf("llookfor set $i00 = %ld\n",iv[0]);
	}
	else
		lquiet(decisecs * 100L,echo_flag);

	if(proctrace)
		pprintf("waited %ld secs\n",time((long *)0) - start_secs);

RETURN:
	if(tesd)
		free_esd(tesd);
	return(erc);

}	/* end of cmd_lookfor */

/*+-------------------------------------------------------------------------
	cmd_nap(param)
nap [-m] <int>
<int> number 1/10ths secs, except if -m, nap <int> milliseconds
--------------------------------------------------------------------------*/
int
cmd_nap(param)
ESD *param;
{
int erc;
char switches[8];
ulong interval;

	get_switches(param,switches,sizeof(switches));

	if(erc = gint(param,&interval))
		return(erc);
	if(interval)
	{
		if(strchr(switches,'m'))
			nap(interval);
		else
			nap(interval * 100L);
	}
	return(0);
}	/* end of cmd_nap */

/*+-------------------------------------------------------------------------
	cmd_parity(param)
parity [e | o | n]
parity ['e' | 'o' | 'n']
--------------------------------------------------------------------------*/
int
cmd_parity(param)
ESD *param;
{
int erc;
int new_parity = 0;
ESD *tesd;

	if(shm->Liofd < 0)
		return(eNoLineAttached);

	if(erc = skip_cmd_break(param))
		return(erc);
	if(!(tesd = make_esd(64)))
		return(eNoMemory);
	erc = gstr(param,tesd);
	new_parity = to_lower((erc) ? param->pb[param->index] : *tesd->pb);
	if(!erc && !tesd->cb)
		new_parity = 0;
	free_esd(tesd);

	switch(new_parity)
	{
		case 'n':
			new_parity = 0;
		case 'e':
		case 'o':
			shm->Lparity = new_parity;
			lset_parity(1);
			break;
		default:
			erc = eBadParameter;
	}
	if(proctrace && !erc)
		pprintf("parity set to %s\n",
			(shm->Lparity) ? ((shm->Lparity == 'e') ? "even" : "odd")
			               : "none");
	return(erc);

}	/* end of cmd_parity */

/*+-------------------------------------------------------------------------
	cmd_prompt(param)
--------------------------------------------------------------------------*/
int
cmd_prompt(param)
ESD *param;
{
extern ESD *icmd_prompt;

	return(gstr(param,icmd_prompt));
}	/* end of cmd_prompt */

/*+-------------------------------------------------------------------------
	cmd_ptrace(param)
--------------------------------------------------------------------------*/
int
cmd_ptrace(param)
ESD *param;
{
char s8[8];
register itmp;

	if(get_alpha_zstr(param,s8,sizeof(s8)))
		return(eSyntaxError);
	itmp = strlen(s8);
	while(itmp--)
		s8[itmp] = to_lower(s8[itmp]);
	if(!strcmp(s8,"on"))
		proctrace = 1;
	else if(!strcmp(s8,"off"))
		proctrace = 0;
	else
		return(eSyntaxError);
	return(0);
}	/* end of cmd_ptrace */

/*+-------------------------------------------------------------------------
	cmd_rname(param) - set remote name
--------------------------------------------------------------------------*/
int
cmd_rname(param)
ESD *param;
{
int erc;
ESD *rname;

	if(shm->Liofd < 0)
		return(eNoLineAttached);
	if(!shm->Lmodem_off_hook)
	{
		pputs("Not connected\n");
		return(eFATAL_ALREADY);
	}

	if((rname = make_esd(sizeof(shm->Lrname) - 1)) == (ESD *)0)
		return(eNoMemory);

	if(!(erc = gstr(param,rname)))
	{
		strcpy(shm->Lrname,rname->pb);
		if(proctrace)
			pprintf("rname set to '%s'\n",rname->pb);
	}
	free_esd(rname);
	return(erc);

}	/* end of cmd_rname */

/*+-------------------------------------------------------------------------
	cmd_send(param)
send [-n] <str>
-n do not send trailing CR
--------------------------------------------------------------------------*/
int
cmd_send(param)
ESD *param;
{
int erc;
ESD *buf;
char switches[8];

	if(shm->Liofd < 0)
		return(eNoLineAttached);

	if((buf = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

	if(erc = gstr(param,buf))
	{
		free_esd(buf);
		return(erc);
	}

	lputs(buf->pb);

	if(!strchr(switches,'n'))
		lputc(CR);

	free_esd(buf);
	return(erc);
}	/* end of cmd_send */

/*+-------------------------------------------------------------------------
	cmd_set(param)
--------------------------------------------------------------------------*/
int
cmd_set(param)
ESD *param;
{
int erc;
int itmp;
ulong varnum;
uint varmax;
char vartype;
char varstr[16];
int show_status;
long *ivptr;
ESD *svptr;
char *cptr;
char *make_char_graphic();

	if(erc = skip_cmd_break(param))
		return(erc);

	do {
		/* $ is optional */
		if((erc = skip_cmd_char(param,'$')) && (erc != eSyntaxError))
			return(erc);
		/* get variable type */
		if(get_cmd_char(param,&vartype))
			return(eSyntaxError);
		/* validate variable type */
		vartype = to_lower(vartype);
		switch(vartype)
		{
			case 'i':
				varmax = IVQUAN;
				break;
			case 's':
				varmax = SVQUAN;
				break;
			default:
				return(eIllegalVarType);
		}

		if(!get_numeric_value(param,&varnum))
			goto TEST_VARNUM;
		else if(*(param->pb + param->index) == '[')
		{
			if(erc = get_subscript(param,&varnum))
				return(erc);
TEST_VARNUM:
			if((int)varnum >= varmax)
				return(eIllegalVarNumber);
			switch(vartype)
			{
				case 'i':
					ivptr = &iv[(int)varnum];
					break;
				default:
					svptr = sv[(int)varnum];
			}
		}
		else if(get_alphanum_zstr(param,varstr,sizeof(varstr)))
			return(eInvalidVarName);
		else
		{
			varnum = 0x1000L;
			switch(vartype)
			{
				case 'i':
					erc = find_mkvi(varstr,&ivptr,1);
					break;
				default:
					erc = find_mkvs(varstr,&svptr,1);
			}
			if(erc)
				return(erc);
		}
			
		show_status = 1;
		if(!skip_cmd_char(param,'='))	/* assignment */
		{
			switch(vartype)
			{
				case 'i':
					if(erc = gint(param,ivptr))
						return(erc);
					break;
				default:
					if(erc = gstr(param,svptr))
						return(erc);
					break;
			}
			if(!proctrace)
				show_status = 0;
		}
		if(show_status)
		{
			switch(vartype)
			{
				case 'i':
					if(varnum != 0x1000L)
						pprintf("$i%02ld = %7ld (0x%08lx,0%lo)\n",varnum,
							*ivptr,*ivptr,*ivptr);
					else
						pprintf("$i%s = %ld (0x%08lx,0%lo)\n",varstr,
							*ivptr,*ivptr,*ivptr);
					break;
				default:
					if(varnum != 0x1000L)
						pprintf("$s%02ld = '",varnum);
					else
						pprintf("$s%s = '",varstr);
					itmp = svptr->cb;
					cptr = svptr->pb;
					while(itmp--)
						pputs(make_char_graphic(*cptr++,0));
					pputs("'\n");
					break;
			}
		}
	} while(!skip_comma(param));

	if(!end_of_cmd(param))
		return(eSyntaxError);

	return(0);
}	/* end of cmd_set */

/*+-------------------------------------------------------------------------
	cmd_system(param)

system [-l] 'cmd'
-l makes comm line stdin/stdout
-s keeps all fds the same

returns $i0 set to exit status of program or 0x100 if interrupted
--------------------------------------------------------------------------*/
int
cmd_system(param)
ESD *param;
{
int erc;
ESD *cmd;
extern int last_child_wait_status;
char switches[8];

	if((cmd = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

/* a hack */
	*cmd->pb++ = (strchr(switches,'s')) ? '>' : 
		((strchr(switches,'l')) ? '$' : '!');

	cmd->maxcb--;

	if(erc = gstr(param,cmd))
	{
		cmd->pb--;		/* be nice */
		cmd->maxcb++;	/* or surely this will haunt us one day */
		free_esd(cmd);
		return(erc);
	}

/* rehack */
	cmd->pb--;
	cmd->cb++;
	cmd->maxcb++;

	if(proctrace)
	{
		pputs(cmd->pb + 1);
		pputs("\n");
	}

	last_child_wait_status = 0xFF00;
	shell(cmd->pb);
	iv[0] = (last_child_wait_status & 0xFF)
			? 0x100L : (long)last_child_wait_status >> 8;
	if(proctrace)
		pprintf("$i0 = %ld, (%s)\n",iv[0],
			(iv[0] == 0x100L) ? "interrupted" : "program exit status");

	free_esd(cmd);
	return(0);
}	/* end of cmd_system */

/*+-------------------------------------------------------------------------
	get_big_endian_16(ptr)
--------------------------------------------------------------------------*/
ushort
get_big_endian_16(ptr)
register uchar *ptr;
{
register ushort uint16 = ((ushort)ptr[0] << 8) | ptr[1];
 
	return(uint16);
 
}	/* end of get_big_endian_16 */
 
/*+-------------------------------------------------------------------------
	get_big_endian_32(ptr)
--------------------------------------------------------------------------*/
ulong
get_big_endian_32(ptr)
register uchar *ptr;
{
register ulong uint32 = ((ulong)*ptr++) << 24;
	uint32 |= ((ulong)*ptr++) << 16;
	uint32 |= ((ulong)*ptr++) <<  8;
	uint32 |=  (ulong)*ptr++;
	return(uint32);
 
}	/* end of get_big_endian_32 */

/*+-------------------------------------------------------------------------
	cmd_getf(param) - get friend memory

getf -x <int-var-spec> <offset>
where: -x ==
   -b byte
   -w word (little-endian)
   -W word (big-endian)
   -l 32-bits (little-endian)
   -L 32-bits (big-endian)
--------------------------------------------------------------------------*/
int
cmd_getf(param)
ESD *param;
{
int erc;
char switches[8];
long *piv;
long offset;
int size;
int big_endian;

	if(erc = get_switches(param,switches,sizeof(switches)))
		return(erc);
	if((strlen(switches) != 2) || !strchr("bwWlL",switches[1]))
	{
		pputs("invalid switch\n");
		return(eFATAL_ALREADY);
	}
	size = to_lower(switches[1]);
	big_endian = isupper(switches[1]);

/*
	if(!get_svptr(param,&psv))
		return(eNotImplemented);
	else
*/
	if(!strncmp(param->pb + param->index,"$i",2))
		param->index += 2;
	if(erc = get_ivptr(param,&piv,1))
		return(erc);

	if(erc = gint(param,&offset))
		return(erc);

	if(proctrace)
		pprintf("getf %s offset=0x%lx",switches,offset);

	switch(size)
	{
		case 'b':
			if(offset > ((long)sizeof(shm->friend_space) - 1))
				goto OFFSET_TOO_LARGE;
			*piv = *(((uchar *)shm->friend_space) + (int)offset) & 0xFF;
			break;
		case 'w':
			if(offset > ((long)sizeof(shm->friend_space) - 2))
				goto OFFSET_TOO_LARGE;
			if(big_endian)
				*piv = get_big_endian_16((uchar *)shm->friend_space +
						(int)offset);
			else
				*piv = *(((ushort *)shm->friend_space) + (int)offset) & 0xFFFF;
			break;
		case 'l':
			if(offset > ((long)sizeof(shm->friend_space) - 4))
				goto OFFSET_TOO_LARGE;
			if(big_endian)
			{
				*piv = get_big_endian_32((uchar *)shm->friend_space +
						(int)offset);
			}
			else
				*piv = *((long *)((char *)shm->friend_space + (int)offset));
			break;
	}

	if(proctrace)
		pprintf(" value=%ld (%08lx)\n",*piv,*piv);
	return(0);

OFFSET_TOO_LARGE:
	if(proctrace)
		pputs("\n");
	pprintf("offset 0x%02lx too large for -%c (0x%02x bytes available)\n",
		offset,switches[1],sizeof(shm->friend_space));
	return(eFATAL_ALREADY);

}	/* end of cmd_getf */

/*+-------------------------------------------------------------------------
	cmd_putf(param)
--------------------------------------------------------------------------*/
cmd_putf(param)
ESD *param;
{
	return(eNotImplemented);
}	/* end of cmd_putf */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of pcmd.c */
