/* CHK=0x36BE */
/*+-------------------------------------------------------------------------
	pcmdtty.c - tty (console) related procedure commands
	wht%n4hgf@gatech.edu

  Defined functions:
	cmd_cls(param)
	cmd_color(param)
	cmd_cursor(param)
	cmd_delline(param)
	cmd_eeol(param)
	cmd_fkey(param)
	cmd_home(param)
	cmd_icolor(param)
	cmd_insline(param)
	cmd_scrdump(param)
	cmd_vidcolor(param)
	cmd_vidnorm(param)
	cmd_vidrev(param)
	ifunc_colors(pvalue)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:04-19-1990-03:08-wht@n4hgf-GCC run found unused vars -- rm them */
/*:03-25-1990-14:11-wht@n4hgf------ x2.70 ------- */
/*:10-14-1989-21:10-wht-allow user to change ecu color choices */
/*:07-03-1989-22:57-wht------ x2.00 ----- */
/*:06-24-1989-16:53-wht-flush edits --- ecu 1.95 */

#include "ecu.h"
#include "ecuerror.h"
#include "esd.h"
#include "ecutty.h"

extern int proctrace;
extern ulong colors_current;

/*+-------------------------------------------------------------------------
	cmd_color(param)

Usage:   color [-r] [argument] [argument]
Options:
   color color      Set foreground and background normal video colors
   -r color color   Set foreground & background reverse video colors

Color names
   blue      magenta      brown      black
   lt_blue   lt_magenta   yellow     gray
   cyan      white        green      red
   lt_cyan   hi_white     lt_green   lt_red

--------------------------------------------------------------------------*/
cmd_color(param)
ESD *param;
{
register erc;
char switches[8];
int normal;
char s32[32];
ulong foreground;
ulong background;

	get_switches(param,switches,sizeof(switches));
	if(!strlen(switches))
		normal = 1;
	else if(switches[1] == 'r')
		normal = 0;		/* reverse */
	else
	{
		pputs("unrecognized switch\n");
		return(eFATAL_ALREADY);
	}

	if((erc = get_alpha_zstr(param,s32,sizeof(s32))) ||
			((foreground = color_name_to_num(s32))) < 0)
		goto ERROR;

	if(erc = get_alpha_zstr(param,s32,sizeof(s32)))
	{
		if(!end_of_cmd(param))
			goto ERROR;
		background = 0;
	}
	else if((background = color_name_to_num(s32)) < 0)
		goto ERROR;

	if(normal)
	{
		colors_current &= 0xFFFF0000;
		colors_current |= (foreground << 8) | background;
		if(proctrace > 1)
		{
			pprintf("normal %ld,%ld current=0x%08lx\n",
				foreground,background,colors_current);
		}
	}
	else
	{
		colors_current &= 0x0000FFFF;
		colors_current |= (foreground << 24) | (background << 16);
		if(proctrace > 1)
		{
			pprintf("reverse %ld,%ld current=0x%08lx\n",
				foreground,background,colors_current);
		}
	}

	setcolor(colors_current);
	return(0);

ERROR:
	if(erc)
		return(erc);
	pputs("invalid color\n");
	return(eFATAL_ALREADY);

}	/* end of cmd_color */

/*+-------------------------------------------------------------------------
	ifunc_colors(pvalue)
--------------------------------------------------------------------------*/
int
ifunc_colors(pvalue)
ulong *pvalue;
{
	*pvalue = colors_current;
	return(0);
}	/* end of ifunc_colors */

/*+-------------------------------------------------------------------------
	cmd_icolor(param)
--------------------------------------------------------------------------*/
cmd_icolor(param)
ESD *param;
{
int erc;
ulong new_colors;

	if(erc = gint(param,&new_colors))
		return(erc);

	setcolor(new_colors);
	return(0);
}	/* end of cmd_icolor */

/*+-------------------------------------------------------------------------
	cmd_cls(param)
--------------------------------------------------------------------------*/
int
cmd_cls(param)
ESD *param;
{
	clear_screen();
	return(0);
}	/* end of cmd_cls */

/*+-------------------------------------------------------------------------
	cmd_cursor(param)
--------------------------------------------------------------------------*/
int
cmd_cursor(param)
ESD *param;
{
int erc;
long row;
long col = 0;

	if(erc = gint(param,&row))
		return(erc);
	if(gint(param,&col))
	{
		/* if something there non-integer */
		if(!end_of_cmd(param))
			return(eSyntaxError);
	}
	curmove((int)row,(int)col);
	return(0);
}	/* end of cmd_cursor */

/*+-------------------------------------------------------------------------
	cmd_scrdump(param)
--------------------------------------------------------------------------*/
int
cmd_scrdump(param)
ESD *param;
{
int erc;
ESD *tesd;
FILE *fp;

	if((tesd = make_esd(256)) == (ESD *)0)
		return(eNoMemory);

	if(erc = gstr(param,tesd))
	{
		if(!end_of_cmd(param))
		{
			erc = eSyntaxError;
			goto RETURN;
		}
	}

	if(tesd->cb)
	{
		if((fp = fopen(tesd->pb,"a")) == NULL)
		{
			pperror(tesd->pb);
			erc = eFATAL_ALREADY;
			goto RETURN;
		}
		fclose(fp);
	}

	screen_dump((tesd->cb) ? tesd->pb : (char *)0);

RETURN:
	free_esd(tesd);
	return(erc);
}	/* end of cmd_scrdump */

/*+-------------------------------------------------------------------------
	cmd_vidnorm(param)
--------------------------------------------------------------------------*/
int
cmd_vidnorm(param)
ESD *param;
{
	stand_end();
	return(0);
}	/* end of cmd_vidnorm */

/*+-------------------------------------------------------------------------
	cmd_vidrev(param)
--------------------------------------------------------------------------*/
int
cmd_vidrev(param)
ESD *param;
{
	stand_out();
	return(0);
}	/* end of cmd_vidrev */

/*+-------------------------------------------------------------------------
	cmd_fkey(param)
--------------------------------------------------------------------------*/
cmd_fkey(param)
ESD *param;
{
int erc;
ESD *tesd;

	if((tesd = make_esd(64)) == (ESD *)0)
		return(eNoMemory);

	if(erc = gstr(param,tesd))
		goto RETURN;

	switch(keyset_read(tesd->pb))
	{
		case  0:
			if(proctrace)
				keyset_display();
			break;
		case -1:
			pprintf("cannot find ~/.ecu/keys\n");
			erc = eFATAL_ALREADY;
			break;
		case -2:
			pprintf("'%s' not found in ~/.ecu/keys\n",tesd->pb);
			erc = eFATAL_ALREADY;
			break;
	}

RETURN:
	free_esd(tesd);
	return(erc);
}	/* end of cmd_fkey */

/*+-------------------------------------------------------------------------
	cmd_vidcolor(param)

vidcolor normal|reverse|notify|success|alert|error fcolor [bcolor]
--------------------------------------------------------------------------*/
int
cmd_vidcolor(param)
ESD *param;
{
int erc;
int ntokens = 0;
char *tokens[3];
int param_index[3];
char tokenbuf[64];

	tokens[0] = tokenbuf;
	tokens[1] = tokenbuf + 20;
	tokens[2] = tokenbuf + 40;

	while(ntokens < 3)
	{
		skip_cmd_break(param);
		param_index[ntokens] = param->index;
		if(erc = get_word_zstr(param,tokens[ntokens],20))
			break;
		ntokens++;
	}

	if(erc && ((erc != eNoParameter) || (ntokens < 2)))
		return(erc);

	switch(erc = setcolor_internal(ntokens,tokens))
	{
		case 0:
			break;
		default:
			param->old_index = param->index = param_index[erc - 1];
			erc = eBadParameter;
	}
	return(erc);
}	/* end of cmd_vidcolor */

/*+-------------------------------------------------------------------------
	cmd_home(param) - home the cursor
--------------------------------------------------------------------------*/
int
cmd_home(param)
ESD *param;
{
	curmove(0,0);
	return(0);
}	/* end of cmd_home */

/*+-------------------------------------------------------------------------
	cmd_eeol(param) - erase to end of line
--------------------------------------------------------------------------*/
int
cmd_eeol(param)
ESD *param;
{
	eeol();
	return(0);
}	/* end of cmd_eeol */

/*+-------------------------------------------------------------------------
	cmd_insline(param) - insert line in display
--------------------------------------------------------------------------*/
int
cmd_insline(param)
ESD *param;
{
	ins_line();
	return(0);
}	/* end of cmd_insline */

/*+-------------------------------------------------------------------------
	cmd_delline(param) - delete line from display
--------------------------------------------------------------------------*/
int
cmd_delline(param)
ESD *param;
{
	del_line();
	return(0);
}	/* end of cmd_delline */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of pcmdtty.c */
