/* CHK=0x0D3F */
/*+-------------------------------------------------------------------------
	scurses.c -- ecu file transfer program curses interface

  000000000011111111112222222222333333333344444444445555555550
  012345678901234567890123456789012345678901234567890123456789
00.----------------------------------------------------------.
01|  SEAlink_6____  _39____________________________________  |
02|  File ### of ###: _38__________________________________  |
03|  File position:  _8______ length: _8______               |
04|  _55____________________________________________________ | transaction
05|  _55____________________________________________________ | last rx/tx hdr
06|  Comm I/O: rx _8______  tx _8______ bytes                |
07|  Baud rate: _5___ BINARY blklen: _____ comm mode: CBREAK |
08|  Time started: session: HH:MM:SS  this file: HH:MM:SS    |
09|                elpased: HH:MM:SS       time: HH:MM:SS    |
10|  Error counts: this file:  _4__  total: _6____           |
11|  _55____________________________________________________ |  err str
12|  _55____________________________________________________ |  comment str
13|  _55____________________________________________________ |  remote info
14`----------------------------------------------------------'

  Defined functions:
	clear_area(win,row,col,len)
	clear_area_char(win,row,col,len,fillchar)
	get_elapsed_time(elapsed_seconds)
	get_tod(type,tod)
	report_error_count()
	report_file_byte_io(count)
	report_file_close()
	report_file_open_length(length)
	report_file_open_tod()
	report_file_rcv_started(filename,length,last_mod_time)
	report_file_send_open(filename,filestat)
	report_init(title)
	report_last_rxhdr(rptstr,error_flag)
	report_last_txhdr(rptstr,error_flag)
	report_line(baud_rate,mode)
	report_protocol_crc_type(str)
	report_rx_ind(status)
	report_rx_tx_count()
	report_rxpos(rxpos)
	report_str(rptstr,error_flag)
	report_top_line(topstr)
	report_transaction(str)
	report_tx_ind(status)
	report_txpos(txpos)
	report_uninit()

------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:05-07-1990-21:24-wht@n4hgf---- 2.80 --- */
/*:03-25-1990-14:12-wht@n4hgf------- ecu x2.70 -------- */
/*:11-19-1989-00:43-wht-report_mode -> report_line */
/*:11-19-1989-00:19-wht-standend was omitted in title */
/*:07-03-1989-22:58-wht------ x2.00 ----- */
/*:03-03-1987-19:47-wht-adapt zcurses.c */

#include <curses.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <signal.h>
#include <time.h>
#include <sys/timeb.h>

#include "../pc_scr.h"
#include "lint_args.h"

long time();
extern char *tzname[];
struct tm *localtime();


#define WIN_LINES	15
#define WIN_COLS	60
#define WIN_TOPY	2
#define WIN_LEFTX	8

extern char curr_dir[];
extern char s128[];
extern int Filcnt;
extern int Restricted;
extern int sending_flag;	/* send == 1, receive == 0 */
extern int npaths;
extern long rxpos;
extern int log_packets;

WINDOW	*win;
int (*original_sigint_handler)();
int (*original_sigquit_handler)();
int (*original_sigterm_handler)();
int curses_installed = 0;		/* curses not yet active */
int this_file_errors = 0;
int total_errors = 0;
long current_seconds;
long start_seconds;
long elapsed_seconds;
unsigned long total_data_chars_xfered = 0L;

char *win_template[] =
{
/*00000000001111111111222222222233333333334444444444555555555 */
/*01234567890123456789012345678901234567890123456789012345678 */
/*.----------------------------------------------------------. */
  "  SEAlink                                                 ",	/* 1 */
  "  File ### of ###: _____________________________________  ",	/* 2 */
  "  File position:  ________ length: ________               ",	/* 3 */
  "                                                          ",	/* 4 */
  "  tx: ______________________  rx: ______________________  ",	/* 5 */
  "  Comm I/O: rx ________  tx ________ bytes                ",	/* 6 */
  "  Baud rate: _____ BINARY blklen: 128   comm mode: ______ ",	/* 7 */
  "  Time started: session: --:--:--  this file: --:--:--    ",	/* 8 */
  "                elapsed: --:--:--    current: --:--:--    ",	/* 9 */
  "  Error counts: this file:  ____  total: ______           ",	/* 10 */
  "                                                          ",	/* 11 */
  "                                                          ",	/* 12 */
  "                                                          ",	/* 13 */
/*`----------------------------------------------------------' */
(char *)0
};

/*+-----------------------------------------------------------------------
	char *get_elapsed_time(elapsed_seconds)
	hh:mm:ss returned
  static string address is returned
------------------------------------------------------------------------*/
char *get_elapsed_time(elapsed_seconds)
long elapsed_seconds;
{
	static char elapsed_time_str[10];
	long hh,mm,ss;

	hh = elapsed_seconds / 3600;
	elapsed_seconds -= hh * 3600;
	mm = elapsed_seconds / 60L;
	elapsed_seconds -= mm * 60L;
	ss = elapsed_seconds;

	sprintf(elapsed_time_str,"%02ld:%02ld:%02ld",hh,mm,ss);
	return(elapsed_time_str);
}	/* end of get_elapsed_time */

/*+-----------------------------------------------------------------------
	char *get_tod(type,tod)

  time of day types:
	0		hh:mm
	1		hh:mm:ss
	2		mm-dd-yyyy hh:mm

  static string address is returned
  if tod != (char *)0, time is returned there too
------------------------------------------------------------------------*/
char *
get_tod(type,tod)
int type;
char *tod;
{
	long cur_time = 0;
	struct tm *lt;			/* local time */
	static char tod_str[32];
	struct timeb tp;

	cur_time = time((long *)0);
	lt = localtime(&cur_time);
#if defined(M_XENIX) && defined(MESSED_UP_DST)
	ftime(&tp);				/* localtime() does not know about DST */
	if(tp.dstflag)			/* so we adjust with ftime, which does know */
	{
		cur_time += 3600L;	/* problem in BSD too, but no ftime there */
		lt = localtime(&cur_time);
		cur_time -= 3600L;
	}
#endif

	switch(type)
	{
	case 0:
		sprintf(tod_str,"%02d:%02d",lt->tm_hour,lt->tm_min);
		break;

	default:
	case 1:
		sprintf(tod_str,"%02d:%02d:%02d",lt->tm_hour,lt->tm_min,lt->tm_sec);
		break;

	case 2:
		sprintf(tod_str,"%02d-%02d-%04d %02d:%02d",
		    lt->tm_mon + 1,lt->tm_mday,lt->tm_year + 1900,
		    lt->tm_hour,lt->tm_min);
		break;
	}

	if(tod != (char *)0)
		strcpy(tod,tod_str);

	return(tod_str);
}	/* end of get_tod */

/*+-------------------------------------------------------------------------
	clear_area(win,row,col,len)
--------------------------------------------------------------------------*/
clear_area(win,row,col,len)
WINDOW	*win;
int row;
int col;
int len;
{
	wmove(win,row,col);
	while(len-- > 0)
		waddch(win,' ');
	wmove(win,row,col);

}	/* end of clear_area */

/*+-------------------------------------------------------------------------
	clear_area_char(win,row,col,len,fillchar)
--------------------------------------------------------------------------*/
clear_area_char(win,row,col,len,fillchar)
WINDOW	*win;
int row;
int col;
int len;
char fillchar;
{
	wmove(win,row,col);
	while(len-- > 0)
		waddch(win,fillchar);
	wmove(win,row,col);

}	/* end of clear_area_char */

/*+-------------------------------------------------------------------------
	report_top_line(topstr)
   top line: row 1 col 18 length 39
--------------------------------------------------------------------------*/
void
report_top_line(topstr)
char *topstr;
{
char s42[42];
	clear_area(win,1,18,39);
	if(strlen(topstr) < 39)
		waddstr(win,topstr);
	else
	{
		strncpy(s42,topstr,39);
		s42[39] = 0;
		waddstr(win,s42);
	}
}	/* end of report_top_line */

/*+-------------------------------------------------------------------------
	report_protocol_crc_type(str)

  protocol crc type:  row 1 col 10 length 6
--------------------------------------------------------------------------*/
report_protocol_crc_type(str)
register char *str;
{
char s8[8];

	if(strlen(str) > 6)
	{
		strncpy(s8,str,6);
		s8[6] = 0;
		str = s8;
	}
	clear_area(win,1,10,6);
	waddstr(win,str);
	wrefresh(win);
	if(log_packets)
	{
		write(log_packets,"chk:  ",6);
		write(log_packets,str,strlen(str));
		write(log_packets,"\n",1);
	}

}	/* end of report_protocol_crc_type */

/*+-------------------------------------------------------------------------
	report_uninit()
--------------------------------------------------------------------------*/
void
report_uninit()
{
float total = (float)total_data_chars_xfered;

	if(curses_installed)
	{
		if((total_data_chars_xfered != 0L) && (elapsed_seconds != 0L))
		{
			sprintf(s128,"Data xfer rate ~= %6.0f chars/sec",
			    total / (float)elapsed_seconds);
			if(log_packets)
			{
				write(log_packets,"info: ",6);
				write(log_packets,s128,strlen(s128));
				write(log_packets,"\n",1);
			}
			report_top_line(s128);
		}
		report_file_byte_io(0L);
		report_rx_tx_count();
		wmove(win,WIN_LINES - 1,WIN_COLS - 1);
		wrefresh(win);
		endwin();
		fprintf(stderr,"\r\n\r\n\r\n");
		fflush(stderr);
		curses_installed = 0;
	}

}	/* end of report_uninit */

/*+-------------------------------------------------------------------------
	report_init(title)
--------------------------------------------------------------------------*/
void
report_init(title)
char *title;
{
register int itmp;
register char *cptr;
char buf[48];

	if(curses_installed)
		return;

	initscr();
	crmode();
	noecho();
	nonl();
	clear();
	curses_installed = 1;
	win = newwin(WIN_LINES,WIN_COLS,WIN_TOPY,WIN_LEFTX);
	box(win,sVR,sHR);
	wmove(win,0,0); waddch(win,sTL);
	wmove(win,win->_maxy - 1,0); waddch(win,sBL);
	wmove(win,win->_maxy - 1,win->_maxx - 1); waddch(win,sBR);
	wmove(win,0,win->_maxx - 1); waddch(win,sTR);
	wmove(win,0,2);
	wstandout(win);
	waddch(win,'[');
	waddch(win,' ');
	waddstr(win,title);
	waddch(win,' ');
	waddch(win,']');
	wstandend(win);
	waddch(win,sHR);
	waddch(win,sHR);
	waddstr(win," dir: ");
	waddstr(win,curr_dir);
	waddch(win,' ');

	itmp = 0;
	while(1)
	{
		if(win_template[itmp] == (char *)0)
			break;
		wmove(win,itmp + 1,1);
		waddstr(win,win_template[itmp++]);
	}
	if(sending_flag)
	{
		clear_area(win,2,15,3);
		sprintf(s128,"%3d",npaths);
		waddstr(win,s128);
#if defined(FORK_DEBUG)
		sprintf(s128,"ecusz pid %d",getpid());
#endif
	}
	else	/* ecurz */
	{
		clear_area(win,2,11,8);	/* clear "of ###" */
		waddstr(win,":");
#if defined(FORK_DEBUG)
		sprintf(s128,"ecurz pid %d",getpid());
#endif
	}

#if defined(FORK_DEBUG)
	ecu_log_event(s128);
#endif

	clear_area(win,1,11,47);
	report_error_count();
	clear_area(win,8,26,8);		/* starting time */
	waddstr(win,get_tod(1,(char *)0));
	start_seconds = time((long *)0);
	current_seconds = start_seconds;

	wrefresh(win);

}	/* end of report_init */

/*+-------------------------------------------------------------------------
	report_rx_ind(status)
--------------------------------------------------------------------------*/
void
report_rx_ind(status)
int status;
{
	wmove(win,1,54);
	waddch(win,(status) ? 'R' : ' ');
	wmove(win,1,54);
	wrefresh(win);
}	/* end of report_rx_ind */

/*+-------------------------------------------------------------------------
	report_tx_ind(status)
--------------------------------------------------------------------------*/
void
report_tx_ind(status)
int status;
{
	wmove(win,1,56);
	waddch(win,(status) ? 'T' : ' ');
	wmove(win,1,56);
	wrefresh(win);
}	/* end of report_tx_ind */

/*+-------------------------------------------------------------------------
	report_rx_tx_count()

  rx char count: row 6 col 16 length 8 unsigned long
  tx char count: row 6 col 29 length 8 unsigned long
  now time of day: row 1 col 50 length 8 hh:mm:ss
  This procedure may be counted upon to perform wrefresh(win)

elapsed time row 9 col 26 length 8
current tod row 9 col 47 length 8
--------------------------------------------------------------------------*/
report_rx_tx_count()
{
	extern unsigned long rx_char_count;
	extern unsigned long tx_char_count;

	register char *cptr;

	sprintf(s128,"%8ld",rx_char_count);
	wmove(win,6,16);
	waddstr(win,s128);
	sprintf(s128,"%8ld",tx_char_count);
	wmove(win,6,29);
	waddstr(win,s128);

	/* now time of day */
	clear_area(win,9,47,8);
	cptr = get_tod(1,(char *)0);
	waddstr(win,cptr);
	current_seconds = time((long *)0);
	elapsed_seconds = current_seconds - start_seconds;
	cptr = get_elapsed_time(elapsed_seconds);
	clear_area(win,9,26,8);
	waddstr(win,cptr);
	wrefresh(win);		/* calling procs expect this to occur always */

}	/* end of report_rx_tx_count */

/*+-------------------------------------------------------------------------
	report_line(baud_rate,mode)
--------------------------------------------------------------------------*/
void
report_line(baud_rate,mode)
unsigned baud_rate;
char *mode;
{
char s16[16];
	wmove(win,7,14);
	sprintf(s16,"%5u",baud_rate);
	waddstr(win,s16);
	clear_area(win,7,52,6);
	waddstr(win,mode);
	wrefresh(win);
}	/* end of report_line */

/*+-------------------------------------------------------------------------
	report_rxpos(rxpos) row 3 col 19 len 8
--------------------------------------------------------------------------*/
void
report_rxpos(rxpos)
long rxpos;
{
char tmp[16];
char refr;

	if(rdchk(0))
	{
		read(0,&refr,1);
		if(refr == 0x0C || refr == 0x012)	/* ^L or ^R */
		{
			write(2,"\033[2J",4);
			nap((long)60);
			touchwin(stdscr);
			wrefresh(stdscr);
			touchwin(win);
			wrefresh(win);
		}
	}

	if((rxpos > 99999999L) || (rxpos < 0L))
		return;

	sprintf(tmp,"%8lu",rxpos);
	wmove(win,3,19);
	waddstr(win,tmp);
	wrefresh(win);
	report_rx_tx_count();	/* which will do a refresh */
}	/* end of report_rxpos */

/*+-------------------------------------------------------------------------
	report_txpos(txpos)
--------------------------------------------------------------------------*/
void
report_txpos(txpos)
long txpos;
{
	report_rxpos(txpos);
}	/* end of report_txpos */

/*+-------------------------------------------------------------------------
	report_error_count()
	DOES NOT PERFORM A REFRESH CYCLE
  this file: row 10 col 29 len 4
  total:     row 10 col 42 len 6
--------------------------------------------------------------------------*/
report_error_count()
{
	char tmp[16];

	clear_area(win,10,29,4);
	sprintf(tmp,"%4d",this_file_errors);
	waddstr(win,tmp);

	clear_area(win,10,42,6);
	sprintf(tmp,"%6d",total_errors);
	waddstr(win,tmp);
	wrefresh(win);

}	/* end of report_error_count */

/*+-------------------------------------------------------------------------
	report_last_txhdr(rptstr,error_flag)
	5,7,22
--------------------------------------------------------------------------*/
void
report_last_txhdr(rptstr,error_flag)
register char *rptstr;
int error_flag;
{
char s24[24];

	if(log_packets)
	{
		write(log_packets,"tx:   ",6);
		write(log_packets,rptstr,strlen(rptstr));
		write(log_packets,"\n",1);
	}

	if(strlen(rptstr) > 22)
	{
		strncpy(s24,rptstr,22);
		s24[23] = 0;
		rptstr = s24;
	}
	clear_area(win,5,7,22);
	waddstr(win,rptstr);
	wrefresh(win);

	if(error_flag)
	{
		++this_file_errors;
		++total_errors;
		report_error_count();
	}
}	/* end of report_last_txhdr */

/*+-------------------------------------------------------------------------
	report_last_rxhdr(rptstr,error_flag)
	5,35,22
--------------------------------------------------------------------------*/
void
report_last_rxhdr(rptstr,error_flag)
register char *rptstr;
int error_flag;
{
char s24[24];
extern int log_packets;

	if(log_packets)
	{
		write(log_packets,"rx: ",4);
		write(log_packets,(error_flag) ? "E " : "  ",2);
		write(log_packets,rptstr,strlen(rptstr));
		write(log_packets,"\n",1);
	}

	if(strlen(rptstr) > 22)
	{
		strncpy(s24,rptstr,22);
		s24[23] = 0;
		rptstr = s24;
	}
	clear_area(win,5,35,22);
	waddstr(win,rptstr);
	wrefresh(win);

	if(error_flag)
	{
		++this_file_errors;
		++total_errors;
		report_error_count();
	}

}	/* end of report_last_rxhdr */

/*+-------------------------------------------------------------------------
	report_str(rptstr,error_flag) row 11/12 col 3 len 55

  error_flag == 0 for status/progress message
             == 1 for bump error count, unless rptstr is null
                  in which case, merely clear error string area
             == 2 write string on bottom line (not an error)
             == 3 write string on transaction line (not an error)
             == -1 use error line but do not bump error count
--------------------------------------------------------------------------*/
void
report_str(rptstr,error_flag)
register char *rptstr;
int error_flag;
{
char s60[60];
extern int log_packets;

	if(strlen(rptstr) > 55)
	{
		strncpy(s60,rptstr,55);
		s60[56] = 0;
		rptstr = s60;
	}

	switch(error_flag)
	{
		case 0:
			clear_area(win,12,3,55);
			break;
		case 1:
			++this_file_errors;
			++total_errors;
			report_error_count();
		case -1:
			clear_area(win,11,3,55);
			break;
		case 2:
			clear_area(win,13,3,55);
			break;
		case 3:
			clear_area(win,4,3,55);
			break;
	}

	waddstr(win,rptstr);
	wrefresh(win);

	if(log_packets)
	{
		write(log_packets,"info: ",6);
		sprintf(s60,"%2d ",error_flag);
		write(log_packets,s60,3);
		write(log_packets,rptstr,strlen(rptstr));
		write(log_packets,"\n",1);
	}

}	/* end of report_str */

/*+-------------------------------------------------------------------------
	report_transaction()
--------------------------------------------------------------------------*/
void
report_transaction(str)
char *str;
{
	report_str(str,3);
}	/* end of report_transaction */

/*+-------------------------------------------------------------------------
	report_file_open_tod() -- time of start of this file

  this file open time: row 8 col 47 length 8
--------------------------------------------------------------------------*/
void
report_file_open_tod()
{
	clear_area(win,8,47,8);
	waddstr(win,get_tod(1,(char *)0));
	wrefresh(win);
}	/* end of report_file_open_tod */

/*+-------------------------------------------------------------------------
	report_file_open_length(long_length)
  length:   row 3 col 36 len  8
--------------------------------------------------------------------------*/
report_file_open_length(length)
long length;
{
	clear_area(win,3,36,8);
	if(length <= 0)
		waddstr(win,"unknown");
	else
	{
		sprintf(s128,"%8lu",length);
		waddstr(win,s128);
	}
	wrefresh(win);
}	/* end of report_file_open_length */

/*+-------------------------------------------------------------------------
	report_file_send_open(filename,filestat)

  filename: row 2 col 20 len 38
  number:   row 2 col 8 len 3
  length:   row 3 col 36 len  8
  mode:     row 3 col 46 len 10
  time of start of this file: row 4 col 47 length 8 hh:mm:ss
--------------------------------------------------------------------------*/
void
report_file_send_open(filename,filestat)
char *filename;
struct stat *filestat;
{
char s50[50];
register char *cptr = filename;
char s256[256];

	if(log_packets)
	{
		write(log_packets,"file: ",6);
		write(log_packets,filename,strlen(filename));
		write(log_packets,"\n",1);
	}

	/* number */
	clear_area(win,2,8,3);
	sprintf(s50,"%3d",Filcnt);
	waddstr(win,s50);

	/* filename */
	if(strlen(filename) > 38)
	{
		strncpy(s50,filename,38);
		s50[39] = 0;
		cptr = s50;
	}
	clear_area(win,2,20,38);
	waddstr(win,cptr);

#if defined(LOG_XFER)
	sprintf(s256,"sending %s",filename);
	ecu_log_event(s256);
#endif

	/* length */
	report_file_open_length(filestat->st_size);

	/* time of start of this file */
	report_file_open_tod();

	this_file_errors = 0;
	report_error_count();
}	/* end of report_file_send_open */

/*+-------------------------------------------------------------------------
	report_file_rcv_started(filename,length,last_mod_time)

  filename: row 2 col 7 len 50
  length:   row 3 col 36 len  8 if not xmodem
  time of start of this file: row 4 col 47 length 8 hh:mm:ss
--------------------------------------------------------------------------*/
report_file_rcv_started(filename,length,last_mod_time)
char *filename;
long length;					/* if < 0, "UNKNOWN" */
long last_mod_time;			/* not currently displayed */
{
register char *cptr;
char s50[50];
char s256[256];

	if(log_packets)
	{
		write(log_packets,"file: ",6);
		write(log_packets,filename,strlen(filename));
		write(log_packets,"\n",1);
	}

	/* filename */
	if(strlen(filename) > 38)
	{
		strncpy(s50,filename,38);
		s50[39] = 0;
		cptr = s50;
	}
	else
		cptr = filename;

#if defined(LOG_XFER)
	sprintf(s256,"receiving %s",filename);
	ecu_log_event(s256);
#endif

	clear_area(win,2,20,38);
	waddstr(win,cptr);

	/* file number */
	clear_area(win,2,8,3);
	sprintf(s50,"%3d",Filcnt);	/* rz uses as file number 1-n */
	waddstr(win,s50);

/* if remote sender provides a file count, display it */
	if(npaths)
	{
		clear_area(win,2,12,7);	/* clear "of ###" */
		sprintf(s50,"of %3d:",npaths);
		waddstr(win,s50);
	}

	/* length */
	report_file_open_length(length);

	/* time of start of this file */
	report_file_open_tod();

	this_file_errors = 0;
	report_error_count();
}	/* end of report_file_rcv_started */

/*+-------------------------------------------------------------------------
	report_file_close()
--------------------------------------------------------------------------*/
void report_file_close()
{
	report_str("End of file",0);
	wrefresh(win);

}	/* end of report_file_close */

/*+-------------------------------------------------------------------------
	report_file_byte_io(count)
--------------------------------------------------------------------------*/
report_file_byte_io(count)
long count;
{

	total_data_chars_xfered += (long)count;
	if(total_data_chars_xfered)
	{
		sprintf(s128,"Total file bytes transferred: %lu",
			total_data_chars_xfered);
		report_str(s128,-1);
	}

}	/* end of report_file_byte_io */

/* end of scurses.c */
/* vi: set tabstop=4 shiftwidth=4: */
